/*--------------------------------------------------------------------------
**      WCommctl.c                  Twin           From:  Twin/dlls/commctrl
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      Copyright 1997 Willows Software, Inc.
**------------------------ < File Content Description > --------------------
**
**  Module:	dlls/commctrl/WCommctl.c 
**
**  Description:Utilities Library
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/dlls/commctrl/WCommctl.c,v 1.4 2000/01/14 20:32:45 robf Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#define WCOMMCTRL
#include "WCommCtrl.h"
#ifndef macintosh
#include "WHeader.h"
#include "WListView.h"
#endif

#include "Log.h"


HINSTANCE                           hLibInstance = ( HINSTANCE )NULL;

extern int WINAPI  TabCtl_Initialize ( HINSTANCE hInstance );
extern int WINAPI  Tooltip_Initialize ( HINSTANCE hLibInstance );
extern int WINAPI  Trackbar_Initialize ( HINSTANCE hLibInstance );
extern int WINAPI  SpinBtn_Initialize ( HINSTANCE hLibInstance );
extern int WINAPI  Toolbr_Initialize ( HINSTANCE hLibInstance );
extern int WINAPI  Statbr_Initialize ( HINSTANCE hLibInstance );
extern int WINAPI  TView_Initialize ( HINSTANCE hLibInstance );
extern int WINAPI  ProgBar_Initialize ( HINSTANCE hLibInstance );
extern int WINAPI  LView_Initialize ( HINSTANCE hLibInstance );
extern int WINAPI  Headr_Initialize ( HINSTANCE hLibInstance );

void EXPORT WINAPI WInitCommonControls ()
{
    LView_Initialize ( hLibInstance );
    Headr_Initialize ( hLibInstance );
    Trackbar_Initialize ( hLibInstance );
    Tooltip_Initialize ( hLibInstance );
    SpinBtn_Initialize ( hLibInstance );
    TabCtl_Initialize ( hLibInstance );
    Toolbr_Initialize ( hLibInstance );
    Statbr_Initialize ( hLibInstance );
    TView_Initialize ( hLibInstance );
    ProgBar_Initialize ( hLibInstance );
}

/*----------------------------------------------------------------------------
**   InitCommonControlsEx
**
**   This function allows a fine control over how the common controls
** are initialized, allowing the user to only load a part of the library
** if desired.
**
** FIXME:  Determine if this routine can be/should be able to be
**         called multiple times.
**--------------------------------------------------------------------------*/
BOOL EXPORT WINAPI InitCommonControlsEx (LPINITCOMMONCONTROLSEX lpInitCtrls)
{
    APISTR((LF_APICALL, "InitCommonControlsEx lpInitCtrls 0x%x, dwICC 0x%x\n",
                lpInitCtrls, lpInitCtrls->dwICC));
    if (!lpInitCtrls)
    {
        APISTR((LF_APIRET, "InitCommonControlsEx returns false, must have !NULL\n"));
	return FALSE;
    }
    if (lpInitCtrls->dwSize < sizeof(INITCOMMONCONTROLSEX))
    {
        APISTR((LF_APIRET, "InitCommonControlsEx returns false, size %d less than required size %d\n",
            lpInitCtrls->dwSize, sizeof(INITCOMMONCONTROLSEX)));
	return FALSE;
    }

    if (lpInitCtrls->dwICC & ICC_LISTVIEW_CLASSES)
        LView_Initialize ( hLibInstance );
    if (lpInitCtrls->dwICC & ICC_TREEVIEW_CLASSES ) 
        TView_Initialize ( hLibInstance );
    if (lpInitCtrls->dwICC & ICC_BAR_CLASSES)
    {
        Toolbr_Initialize ( hLibInstance );
        Statbr_Initialize ( hLibInstance );
        ProgBar_Initialize ( hLibInstance );
        Trackbar_Initialize ( hLibInstance );
    }

    if (lpInitCtrls->dwICC & ICC_TAB_CLASSES)
        TabCtl_Initialize ( hLibInstance );

    /* FIXME : Process remaining InitCommonControlsEx flags */
    if (lpInitCtrls->dwICC & ICC_UPDOWN_CLASS      
        || lpInitCtrls->dwICC & ICC_PROGRESS_CLASS    
        || lpInitCtrls->dwICC & ICC_HOTKEY_CLASS      
        || lpInitCtrls->dwICC & ICC_ANIMATE_CLASS     
        || lpInitCtrls->dwICC & ICC_DATE_CLASSES      
        || lpInitCtrls->dwICC & ICC_USEREX_CLASSES    
        || lpInitCtrls->dwICC & ICC_COOL_CLASSES      
        || lpInitCtrls->dwICC & ICC_INTERNET_CLASSES  
        || lpInitCtrls->dwICC & ICC_PAGESCROLLER_CLASS
        || lpInitCtrls->dwICC & ICC_NATIVEFNTCTL_CLASS)
        APISTR((LF_APISTUB, "InitCommonControlsEx contains flags (0x%x) we do not process\n",
                    lpInitCtrls->dwICC));

    /*------------------------------------------------------------------------
    ** These initializations didn't seem to fit under one of the
    **   above classes, so we always do them.
    **----------------------------------------------------------------------*/
    Headr_Initialize ( hLibInstance );
    Tooltip_Initialize ( hLibInstance );
    SpinBtn_Initialize ( hLibInstance );

    APISTR((LF_APIRET, "InitCommonControlsEx returns TRUE\n"));
    return TRUE;
}

static void WINAPI WTerminateCommonControls()
{
        LView_Terminate ( hLibInstance );
        Headr_Terminate ( hLibInstance );
        Trackbar_Terminate ( hLibInstance );
        Tooltip_Terminate ( hLibInstance );
        SpinBtn_Terminate ( hLibInstance );
	TabCtl_Terminate ( hLibInstance );
	Toolbr_Terminate ( hLibInstance );
	Statbr_Terminate ( hLibInstance );
	TView_Terminate ( hLibInstance );
	ProgBar_Terminate ( hLibInstance );
}

BOOL WINAPI DllMain (
    HINSTANCE                           hInstance,
    DWORD                               dReason,
    LPVOID                              pReserved )

{

    BOOL                                ReturnValue = TRUE;

    switch ( dReason )
    {
        case DLL_PROCESS_ATTACH:
            hLibInstance = hInstance;
            break;

        case DLL_THREAD_ATTACH:
        case DLL_THREAD_DETACH:
            break;

        case DLL_PROCESS_DETACH:
	    WTerminateCommonControls();
            break;
    }

    return ( ReturnValue );

}

BOOL WINAPI EXPORT Commctrl_LibMain (
    HINSTANCE                           hInstance,
    LPSTR				lpCmdLine,
    int					nCmdShow )

{

    BOOL                                ReturnValue = TRUE;

    hLibInstance = hInstance;
    return ( ReturnValue );
}

int WINAPI EXPORT WEP ( )
{
    WTerminateCommonControls();
    return 0;
}

/*------------------------- < Full Revision History > ----------------------
** $Log: WCommctl.c,v $
** Revision 1.4  2000/01/14 20:32:45  robf
** added termination
**
** Revision 1.3  2000/01/14 04:55:30  jwhite
** Implement first pass at InitCommonControlsEx
**
** Revision 1.2  1999/07/12 15:38:20  mwalsh
** Updated comments to standard header and footer style
**
**-------------------------------------------------------------------------*/
