/*--------------------------------------------------------------------------
**      WImgList.c                 Twin           From:  Twin/dlls/commctrl
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      Copyright 1997 Willows Software, Inc.
**------------------------ < File Content Description > --------------------
**
**  Module:	dlls/commctrl/WImgList.c
**
**  Description:
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/dlls/commctrl/WImgList.c,v 1.17 2000/01/28 08:21:38 robf Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#define _WIMAGELIST_SOURCE
#include "windows.h"
#include "wtypes.h"
#include "WLists.h"
#include "oleauto.h"
#include "windowsx.h"
#include "wine/obj_storage.h"

#define MAXOVERLAY	4

typedef struct _WIMAGE
{
	HDC			hMemoryDC;
	HDC			hMaskDC;
	int			NumImages;
	int			NumUsed;
} WIMAGE;

typedef struct TAG_WIMAGELIST
{
	HGLOBAL		hMemImageList;
	int		Height;
	int		Width;
	UINT		CreateFlags;
	int		InitialSize;
	int		GrowBy;
	COLORREF	BackgroundColor;
	COLORREF	ForegroundColor;
	COLORREF	MaskColor;
    	WIMAGE      	ImgList;
	HBRUSH		hBlend25Brush;
	HBRUSH		hBlend50Brush;
	int		OverlayIndices[MAXOVERLAY];
} _WIMAGELIST;

typedef _WIMAGELIST*			HIMAGELIST;

#include "WCommCtrl.h"
#include "WImgList.h"
#include "WImageIO.h"
#include "WBmp.h"
#include "Log.h"

typedef struct _WDRAGIMAGE
{
	WHIMAGELIST		pImageList;
	int			ImageIndex;
	HWND			hDragWnd;
	HDC			hBkgroundDC;
	HDC			hImageDC;
	HDC			hMaskDC;
	POINT			ptPos;
	POINT			ptSize;
	POINT			ptHotSpot;
	BOOL			bShow;
} WDRAGIMAGE;


#define WIL_ADDINDEX			-1
#define ISVALIDOVERLAY(i) (i >= 1 && i <= 4)
#define GETOVERLAYINDEX(i) ( ( DrawFlags & ILD_OVERLAYMASK ) >> 8 )

static char*			szProperty = "WDragImage";

BOOL WINAPI
Image_IRemove
( 
	WIMAGE				*Image,
	int			    	nSubImageRemoved,
	int			    	nHeight,
	int			    	nWeight,
	BOOL				bMask
);

int 
ImageList_IAddImage
( 
	WHIMAGELIST			hImageList, 
	int			    	nAddedImages,
	HBITMAP				hBitmap,
	HBITMAP				hMask
);

static BOOL 
W_IDrawMaskedImage 
( 
	WHIMAGELIST			hImageList,
	int			    	ImageIndex,
	HDC				hDstDC,
	int				X,
	int				Y,
	int				DX,
	int				DY,
	COLORREF			BackgroundColor,
	COLORREF			ForegroundColor,
	UINT				DrawFlags
);

static BOOL 
W_IRemoveAll 
( 
   WIMAGE*			pImage
);

/*----------------------------------------------------------------------------*/
#if 1
#define DUMPDC(s,dc)	
#else

#define DUMPDC(s,dc)	W_DUMPDC(s,dc,__FILE__,__LINE__)

static void W_DUMPDC(char *s,HDC hdc,char *file,int line)
{
	logstr(LF_CONSOLE,"DumpDC: %s:%d %s\n",file,line,s);
#if defined(DEBUG)
	DEBUG_DumpDC(hdc,0,0);
#endif
}
#endif

int WINAPI
ImageList_ICopyImage
( 
	WHIMAGELIST			hImageList, 
	int				ImageIndex,
	HBITMAP				hBitmap,
	HBITMAP				hMask
)

{

	BITMAP				Bitmap;
	BITMAP				Mask;
	HBITMAP				hTempBitmap = ( HBITMAP )NULL;
	HDC				hTempDC     = ( HDC )NULL;
	HDC				hDeskDC     = GetDC ( GetDesktopWindow () );
	int				NumImages   = 1;
	WIMAGE				*pImage     = &hImageList->ImgList;
	
	int				ErrorCode   = 0;

	memset ( &Bitmap, 0, sizeof ( BITMAP ) );
	memset ( &Mask, 0, sizeof ( BITMAP ) );

	if ( ! ( GetObject ( hBitmap, sizeof ( BITMAP ), &Bitmap ) ) )
		ErrorCode = WIL_ERR_GETOBJECT;
	else
	if ( ( hImageList->CreateFlags & ILC_MASK ) && ( hMask ) && 
		( ! ( GetObject ( hMask, sizeof ( BITMAP ), &Mask ) ) ) )
		ErrorCode = WIL_ERR_GETOBJECT;
	else
	if ( ! ( hTempDC = CreateCompatibleDC ( hDeskDC ) ) )
		ErrorCode = WIL_ERR_CREATECOMPATIBLEDC;
	else
	{
		SelectObject ( hTempDC, hBitmap );

		if ( pImage->hMemoryDC ) {

			BitBlt ( pImage->hMemoryDC, 
				ImageIndex * hImageList->Width, 0, 
				hImageList->Width, hImageList->Height, hTempDC,
				0, 0, SRCCOPY );
		} else
		if ( ! ( pImage->hMemoryDC = CreateCompatibleDC ( hDeskDC ) ) )
			ErrorCode = WIL_ERR_CREATECOMPATIBLEDC;
		else
		if ( ! ( hTempBitmap = CreateBitmap((int)(Bitmap.bmWidth / hImageList->Width ) * hImageList->Width,
			Bitmap.bmHeight < hImageList->Height ? 
				Bitmap.bmHeight : 
				hImageList->Height, 
			Bitmap.bmPlanes,
			Bitmap.bmBitsPixel, ( void* )NULL ) ) )
			ErrorCode = WIL_ERR_CREATEBITMAP;
		else
		{

			SelectObject( pImage->hMemoryDC, hTempBitmap );

			BitBlt ( pImage->hMemoryDC, 0, 
				Bitmap.bmHeight < hImageList->Height ? hImageList->Height - Bitmap.bmHeight : 0, 
				( int )( Bitmap.bmWidth / hImageList->Width ) * hImageList->Width,
				Bitmap.bmHeight < hImageList->Height ? Bitmap.bmHeight : hImageList->Height,
				hTempDC, 0, 0, SRCCOPY );

			pImage->NumUsed = ( int )( Bitmap.bmWidth / hImageList->Width );
			pImage->NumImages = hImageList->GrowBy;
			if ( pImage->NumUsed > hImageList->GrowBy )
				pImage->NumImages = pImage->NumUsed;
			NumImages = pImage->NumImages;
		}
		if ( hTempBitmap )
		{
			DeleteObject ( hTempBitmap );
			hTempBitmap = ( HBITMAP ) NULL;
		}
		if ( ( hMask ) && ( hImageList->CreateFlags & ILC_MASK ) )
		{
			SelectObject ( hTempDC, hMask );
			if ( pImage->hMaskDC )
				BitBlt(pImage->hMaskDC,
					ImageIndex * hImageList->Width, 0, 
					hImageList->Width, hImageList->Height, 
					hTempDC, 0, 0, 
					SRCCOPY );
			else
			if ( ! ( pImage->hMaskDC = CreateCompatibleDC(hDeskDC)))
				ErrorCode = WIL_ERR_CREATECOMPATIBLEDC;
			else
			if ( ! ( hTempBitmap = CreateCompatibleBitmap(hDeskDC,
					(int)(Mask.bmWidth / hImageList->Width)*
						 hImageList->Width,
					Mask.bmHeight < hImageList->Height ? 
						Mask.bmHeight : 
						hImageList->Height ) ) )
				ErrorCode = WIL_ERR_CREATECOMPATIBLEBITMAP;
			else
			{
				SelectObject ( pImage->hMaskDC, hTempBitmap );
				BitBlt ( pImage->hMaskDC, 0,
					Mask.bmHeight < hImageList->Height ? 
					   hImageList->Height - Bitmap.bmHeight:
					   0,
					(int)(Mask.bmWidth/hImageList->Width)*
						hImageList->Width,
					Mask.bmHeight < hImageList->Height ? 
						Mask.bmHeight : 
						hImageList->Height,
					hTempDC, 0, 0, SRCCOPY );
			}

		}
	}

	if ( hTempDC )
		DeleteDC ( hTempDC );

	if ( hTempBitmap )
		DeleteObject ( hTempBitmap );

	if ( hDeskDC )
		ReleaseDC ( GetDesktopWindow (), hDeskDC );

	return ( ErrorCode );

}

/*-------------------------------------------------------------------------
 * Parameters:
 * pImage : pointer to WIMAGE structure that would contain the HDC space 
 *	for the bitmaps to be copied into.
 * nAddedImages : number of new images being copied from hBitmap into 
 *	WIMAGE::hMemoryDC starting from position indexed by ImageIndex
 *------------------------------------------------------------------------*/
int 
ImageList_IAddImage
( 
	WHIMAGELIST			hImageList, 
	int				nAddedImages,
	HBITMAP				hBitmap,
	HBITMAP				hMask
)
{
	BITMAP				Bitmap;
	BITMAP				Mask;
	WIMAGE				*pImage     = &hImageList->ImgList;
	HBITMAP				hTempBitmap = ( HBITMAP )NULL;
	HDC				hTempDC     = ( HDC )NULL;
	HDC				hMemDC      = ( HDC )NULL;
	HDC				hMaskDC     = ( HDC )NULL;
	HDC				hDeskDC     = GetDC ( GetDesktopWindow () );
	int				ImageIndex  = -1;
	int				NewWidth    = 0;
	int				ErrorCode   = 0;

	memset ( &Bitmap, 0, sizeof ( BITMAP ) );
	memset ( &Mask, 0, sizeof ( BITMAP ) );

	if ( ! ( GetObject ( hBitmap, sizeof ( BITMAP ), &Bitmap ) ) )
		ErrorCode = WIL_ERR_GETOBJECT;
	else
	if ( ( hImageList->CreateFlags & ILC_MASK ) && ( hMask ) && 
		( ! ( GetObject ( hMask, sizeof ( BITMAP ), &Mask ) ) ) )
		ErrorCode = WIL_ERR_GETOBJECT;
	else
	if ( ! ( hTempDC = CreateCompatibleDC ( hDeskDC ) ) )
		ErrorCode = WIL_ERR_CREATECOMPATIBLEDC;
    	else
    	{
        	ImageIndex = pImage->NumUsed;

		/* allocate new space */
        	if ( nAddedImages > pImage->NumImages - pImage->NumUsed )
        	{
	     		hMemDC = CreateCompatibleDC ( hDeskDC );
    	 		NewWidth = max ( nAddedImages + pImage->NumImages, 
					 hImageList->GrowBy );
	     		hTempBitmap = CreateBitmap ( 
				NewWidth * hImageList->Width,
			    	hImageList->Height, Bitmap.bmPlanes,
			    	Bitmap.bmBitsPixel, ( void* )NULL );

             		if ( hMemDC && hTempBitmap )
             		{
                		hTempBitmap = SelectObject(hMemDC,hTempBitmap);

				if ( pImage->hMemoryDC )
				{
		    		   BitBlt ( hMemDC, 0, 0, 
					 pImage->NumUsed * hImageList->Width, 
					 hImageList->Height, 
					  pImage->hMemoryDC, 0, 0, SRCCOPY );
                			DeleteDC ( pImage->hMemoryDC );
				}

                		pImage->hMemoryDC = hMemDC;

                		hMemDC = ( HDC )NULL;
    				pImage->NumImages = NewWidth;

                		if ( hImageList->CreateFlags & ILC_MASK )
                		{
    		        		hMaskDC = CreateCompatibleDC ( hDeskDC );
        				DeleteObject ( hTempBitmap );
		            		hTempBitmap = CreateBitmap ( NewWidth * hImageList->Width,
			            	hImageList->Height, Bitmap.bmPlanes,
			            	Bitmap.bmBitsPixel, ( void* )NULL );
                    			if ( hMaskDC && hTempBitmap )
                    			{
                        			hTempBitmap = SelectObject ( hMaskDC, hTempBitmap );
                        			if ( pImage->hMaskDC )
						{
							BitBlt ( hMaskDC, 0, 0, 
	                        	 	 	 	 pImage->NumUsed * 
								 hImageList->Width, 
					 	 		 hImageList->Height, 
	                        	 	 		 pImage->hMaskDC, 0,
								 0, SRCCOPY );
                        				DeleteDC ( pImage->hMaskDC );
						}
                        			pImage->hMaskDC = hMaskDC;
                        			hMaskDC = ( HDC )NULL;
                    			}
                		}
             		}
        	}

		if ( pImage->hMemoryDC )
		{
            		if ( hImageList->CreateFlags & ILC_MASK )
            		{
    		    		hMask = SelectObject ( hTempDC, hMask );
				StretchBlt ( pImage->hMaskDC, pImage->NumUsed * 
				     	     hImageList->Width, 0, 
				     	     nAddedImages * hImageList->Width, 
                                      	     hImageList->Height, hTempDC, 0, 0, 
					     Mask.bmWidth, Mask.bmHeight, SRCCOPY );
    		    		hMask = SelectObject ( hTempDC, hMask );
            		}
    			hBitmap = SelectObject ( hTempDC, hBitmap );

			if(nAddedImages == 1 && 0) {
			    BitBlt ( pImage->hMemoryDC, 
				pImage->NumUsed * hImageList->Width, 0, 
				hImageList->Width, hImageList->Height, 
				hTempDC, 0, 0, SRCCOPY );
			} else {
			    StretchBlt ( pImage->hMemoryDC, 
				pImage->NumUsed * hImageList->Width, 0, 
				nAddedImages * hImageList->Width, 
				hImageList->Height, hTempDC, 0, 0, 
				Bitmap.bmWidth, Bitmap.bmHeight, SRCCOPY );
			}

    			hBitmap = SelectObject ( hTempDC, hBitmap );
			pImage->NumUsed += nAddedImages;
			/*DEBUG_DumpDC(pImage->hMemoryDC,0,16);*/
		}

    	}

	if ( hTempDC )
		DeleteDC ( hTempDC );

	if ( hTempBitmap )
		DeleteObject ( hTempBitmap );

	if ( hDeskDC )
		ReleaseDC ( GetDesktopWindow (), hDeskDC );

	return ImageIndex;

}

/*----------------------------------------------------------------------------*/
int WINAPI
ImageList_Add
(
	WHIMAGELIST			hImageList,
	HBITMAP				hBitmap,
	HBITMAP				hMask
)

{

	BITMAP				Bitmap;
	UINT				ImageListSections;
	int				ImageIndex		    = -1;
	int				nAddedImages		    = 0;
	int				ErrorCode		    = 0;

    	APISTR((LF_APICALL,"ImageList_Add(IMAGELIST=%x,HBITMAP=%x,HBITMAP=%x)\n", 
		hImageList,hBitmap,hMask));

	memset ( &Bitmap, 0, sizeof ( BITMAP ) );

    	ImageListSections = hImageList->ImgList.NumUsed;
	if ( ! ( GetObject ( hBitmap, sizeof ( BITMAP ), &Bitmap ) ) )
		ErrorCode = WIL_ERR_GETOBJECT;
	else
	if ( hImageList->Height <= 0 || hImageList->Width <= 0 )
		ErrorCode = WIL_ERR_INVALIDVALUE;
	else
	{
		nAddedImages = Bitmap.bmWidth <= hImageList->Width ? 1:( Bitmap.bmWidth / hImageList->Width );  
		ImageIndex = ImageList_IAddImage ( hImageList, nAddedImages, hBitmap, hMask );
	}

    APISTR((LF_APIRET,"ImageList_Add: returns int %d\n",ImageIndex));
    return ImageIndex;
}

/*----------------------------------------------------------------------------*/
int WINAPI
ImageList_AddMasked
(
	WHIMAGELIST		hImageList,
	HBITMAP			hBitmap,
	COLORREF		MaskColor
)

{
	BITMAP			Bitmap;
	HDC			hDeskDC		= GetDC ( GetDesktopWindow () );
	HDC			hbmDC		= ( HDC )NULL;
	HDC			hMaskDC		= ( HDC )NULL;
	HBITMAP			hbmMask		= ( HBITMAP )NULL;
	int			NewIndex	= 0;
	int			ErrorCode	= 0;


	memset ( &Bitmap, 0, sizeof ( BITMAP ) );

	if ( ! ( hMaskDC = CreateCompatibleDC( hDeskDC ) )  ||
	     ! ( hbmDC   = CreateCompatibleDC( hDeskDC ) )
	   )
		ErrorCode = WIL_ERR_CREATECOMPATIBLEDC;
	else
	if ( ! ( GetObject ( hBitmap, sizeof ( BITMAP ), &Bitmap ) ) )
		ErrorCode = WIL_ERR_GETOBJECT;
	else
	if ( ! ( hbmMask = CreateBitmap ( ( int ) Bitmap.bmWidth, ( int ) Bitmap.bmHeight,
	                          	  1, 1, NULL ) ) )
		ErrorCode = WIL_ERR_CREATEBITMAP;
	else
	{
		/*create a mask */
		if ( MaskColor == CLR_NONE )
			MaskColor = RGB ( 255, 255, 255 );
		hBitmap = SelectObject ( hbmDC, hBitmap );
		SetBkColor ( hbmDC, MaskColor );
		hbmMask = SelectObject ( hMaskDC, hbmMask );
		BitBlt ( hMaskDC, 0, 0, ( int ) Bitmap.bmWidth, Bitmap.bmHeight,
			 hbmDC, 0, 0, SRCCOPY );
		hBitmap = SelectObject ( hbmDC, hBitmap );
		hbmMask = SelectObject ( hMaskDC, hbmMask );

		NewIndex = ImageList_Add ( hImageList, hBitmap, hbmMask );
	}
	
	if ( hbmMask )
		DeleteObject ( hbmMask );

	if ( hbmDC )
		DeleteDC ( hbmDC );
	if ( hMaskDC )
		DeleteDC ( hMaskDC );
	
	if ( hDeskDC )
		ReleaseDC ( GetDesktopWindow (), hDeskDC );

	return ( NewIndex );

}

/*----------------------------------------------------------------------------*/
BOOL WINAPI
ImageList_BeginDrag
(
	WHIMAGELIST		hImageList,
	int			ImageIndex,
	int			HotSpotX,
	int			HotSpotY
)

{
	POINT			ptNewSize;
	HGLOBAL			hDragImage;
	ICONINFO		IconInfo;
	HDC			hDeskDC		= GetDC ( GetDesktopWindow () );
	HICON			hIcon		= ( HICON )NULL;
	HBITMAP			hBkBitmap	= ( HBITMAP )NULL;
	WDRAGIMAGE*		pDragImage	= ( WDRAGIMAGE* )NULL;
	HGLOBAL			hPrevDragImage	= ( HGLOBAL )NULL;
	BITMAP			Bitmap;
	int			ErrorCode	= 0;

	memset ( &IconInfo, 0, sizeof ( ICONINFO ) );
	memset ( &Bitmap, 0, sizeof ( BITMAP ) );

	if ( ! ( hIcon = ImageList_GetIcon ( hImageList, ImageIndex, ILD_TRANSPARENT ) ) )
	{
		ErrorCode = WIL_ERR_GETICON;
		return FALSE;
	}
	if ( !GetIconInfo ( hIcon, &IconInfo ) )
	{
		ErrorCode =  WIL_ERR_GETIMAGEINFO;
		return FALSE;
	}

	GetObject ( IconInfo.hbmColor, sizeof ( BITMAP ), &Bitmap ); 
	ptNewSize.x = Bitmap.bmWidth;
	ptNewSize.y = Bitmap.bmHeight;

	if ( ! ( hDragImage = GlobalAlloc ( GHND, sizeof ( WDRAGIMAGE ) ) ) )
		ErrorCode = WIL_ERR_GLOBALALLOC;
	else
	if ( ! ( pDragImage = ( WDRAGIMAGE* )GlobalLock ( hDragImage ) ) )
		ErrorCode = WIL_ERR_GLOBALLOCK;
	else
	if ( ! ( pDragImage->hBkgroundDC = CreateCompatibleDC ( hDeskDC ) ) )
		ErrorCode = WIL_ERR_CREATECOMPATIBLEDC;
	if ( ! ( hBkBitmap = CreateCompatibleBitmap ( hDeskDC, ptNewSize.x, ptNewSize.y ) ) )
		ErrorCode = WIL_ERR_CREATECOMPATIBLEBITMAP;
	else
	if ( ! ( pDragImage->hImageDC = CreateCompatibleDC ( hDeskDC ) ) ||
	     ! ( pDragImage->hMaskDC = CreateCompatibleDC ( hDeskDC )  )
	   )
		ErrorCode = WIL_ERR_CREATECOMPATIBLEDC;
	else
	if ( ! ( pDragImage->pImageList =  ImageList_Create ( ptNewSize.x, ptNewSize.y, 
							hImageList->CreateFlags, 
							10, hImageList->GrowBy ) ) )
		ErrorCode =  WIL_ERR_LISTCREATE;
	else
	if ( ImageList_ReplaceIcon ( pDragImage->pImageList, -1, hIcon ) == -1 )
		ErrorCode =  WIL_ERR_REPLACEICON;
	else
	{
		IconInfo.hbmColor = SelectObject ( pDragImage->hImageDC, IconInfo.hbmColor );
		IconInfo.hbmMask = SelectObject ( pDragImage->hMaskDC, IconInfo.hbmMask );
		hBkBitmap = SelectObject ( pDragImage->hBkgroundDC, hBkBitmap );
		pDragImage->ptSize.x = ptNewSize.x;
		pDragImage->ptSize.y = ptNewSize.y;
		pDragImage->ptHotSpot.x = HotSpotX;
		pDragImage->ptHotSpot.y = HotSpotY;
		pDragImage->bShow = TRUE;

		if ( ! SetProp ( GetDesktopWindow (), szProperty, hDragImage ) )
			ErrorCode = WIL_ERR_SETWINPROP;
	}

	if ( hIcon )
		DestroyIcon ( hIcon );
	if ( hBkBitmap )
		DeleteObject ( hBkBitmap );
	if ( IconInfo.hbmColor )
		DeleteObject ( IconInfo.hbmColor );
	if ( IconInfo.hbmMask )
		DeleteObject ( IconInfo.hbmMask );
	
	if ( hPrevDragImage )
		GlobalFree ( hPrevDragImage );
	if ( pDragImage )
		GlobalUnlock ( hDragImage );

	return ( ErrorCode >= 0 );

}

/*----------------------------------------------------------------------------*/
WHIMAGELIST WINAPI
ImageList_Create
(
	int			SubImageWidth,
	int			SubImageHeight,
	UINT			CreateFlags,
	int			NumberOfSubImages,
	int			NumberOfImagesToGrowBy
)

{
	HGLOBAL			hMemImageList   = ( HGLOBAL )NULL;
	WHIMAGELIST		hImageList      = ( WHIMAGELIST )NULL;
	HBITMAP			hBitmap	        = ( HBITMAP )NULL;
	int			ErrorCode       = 0;

	static WORD wBlend25Bits [] =
		{ 0xAA, 0x00, 0x55, 0x00, 0xAA, 0x00, 0x55, 0x00 };
	static WORD wBlend50Bits [] =
		{ 0x55, 0xAA, 0x55, 0xAA, 0x55, 0xAA, 0x55, 0xAA };

    	APISTR((LF_APICALL,"ImageList_Create(%d,%d,UINT=%x,%d,%d)\n", 
		SubImageWidth,SubImageHeight,CreateFlags,
		NumberOfSubImages,NumberOfImagesToGrowBy));

	if ( ! ( hMemImageList = GlobalAlloc ( GHND, sizeof ( _WIMAGELIST ) ) ) )
		ErrorCode = WIL_ERR_GLOBALALLOC;
	else
	if ( ! ( hImageList = ( WHIMAGELIST ) GlobalLock ( hMemImageList ) ) )
		ErrorCode = WIL_ERR_GLOBALLOCK;
	else
	{
		hImageList->hMemImageList   = hMemImageList;
		hImageList->Height          = SubImageHeight;
		hImageList->Width           = SubImageWidth;
		hImageList->CreateFlags     = CreateFlags;
		hImageList->InitialSize     = NumberOfSubImages;
		hImageList->GrowBy          = NumberOfImagesToGrowBy;
		hImageList->ForegroundColor = CLR_DEFAULT;
		hImageList->MaskColor       = CLR_DEFAULT;
		hImageList->BackgroundColor = CLR_NONE;
		hImageList->hBlend25Brush   = ( HBRUSH )NULL;
		hImageList->hBlend50Brush   = ( HBRUSH )NULL;
		
		if ( ( CreateFlags & ILC_MASK ) )
		{
			hBitmap = CreateBitmap ( 8, 8, 1, 1, ( LPSTR ) wBlend25Bits );
			hImageList->hBlend25Brush = CreatePatternBrush ( hBitmap );
			DeleteObject ( hBitmap );

			hBitmap = CreateBitmap ( 8, 8, 1, 1, ( LPSTR ) wBlend50Bits );
			hImageList->hBlend50Brush = CreatePatternBrush ( hBitmap );
			DeleteObject ( hBitmap );
		}
	}

	if ( ErrorCode < 0 )
	{
		if ( hImageList )
		{
			GlobalUnlock ( hMemImageList );
			hImageList = ( WHIMAGELIST )NULL;
		}
		if ( hMemImageList )
			GlobalFree ( hMemImageList );
	}

    	APISTR((LF_APIRET,"ImageList_Create returns WHIMAGELIST %x\n",
		hImageList));
	
	return ( hImageList );

}

/*----------------------------------------------------------------------------*/
BOOL WINAPI
ImageList_Destroy
(
	WHIMAGELIST		hImageList
)

{

    	APISTR((LF_APICALL,"ImageList_Destroy(WHIMAGELIST=%x)\n", hImageList));

    	if ( hImageList->ImgList.hMemoryDC )
		DeleteDC ( hImageList->ImgList.hMemoryDC );
	if ( hImageList->ImgList.hMaskDC )
		DeleteDC ( hImageList->ImgList.hMaskDC );

	GlobalUnlock ( hImageList->hMemImageList );
	GlobalFree ( hImageList->hMemImageList );

	if ( hImageList->hBlend25Brush )
		DeleteObject ( hImageList->hBlend25Brush );
	if ( hImageList->hBlend50Brush )
		DeleteObject ( hImageList->hBlend50Brush );

	hImageList = ( WHIMAGELIST ) NULL;

    	APISTR((LF_APIRET,"ImageList_Destroy returns TRUE\n"));
	return ( TRUE );

}

/*----------------------------------------------------------------------------*/
BOOL WINAPI
ImageList_DragEnter
(
	HWND				hWnd,
	int				WindowX,  /* Note this is not ClientX */
	int				WindowY   /* Note this is not ClientY */
)

{
	HDC				hDragWndDC	= ( HDC )NULL;
	HGLOBAL				hDragImage	= ( HGLOBAL )NULL;
	WDRAGIMAGE*			pDragImage	= ( WDRAGIMAGE* )NULL;

	int				ErrorCode	= 0;

	if ( ! ( hDragImage = GetProp ( GetDesktopWindow (), szProperty ) ) ||
	     ! ( pDragImage = ( WDRAGIMAGE *) GlobalLock ( hDragImage  )    ) 
	   )
		ErrorCode = WIL_ERR_GLOBALLOCK;
	else
	if ( ! ( hDragWndDC = GetWindowDC ( hWnd ) ) )
		ErrorCode = WIL_ERR_INVALIDVALUE;
	else
	{
		pDragImage->hDragWnd = hWnd;
		pDragImage->ptPos.x = WindowX-pDragImage->ptHotSpot.x;
		pDragImage->ptPos.y = WindowY-pDragImage->ptHotSpot.y;
		/*record the background where the drag image would be drawn on*/
		BitBlt ( pDragImage->hBkgroundDC, 0, 0,
						  pDragImage->ptSize.x,
						  pDragImage->ptSize.y,
			 hDragWndDC, pDragImage->ptPos.x, 
				     pDragImage->ptPos.y, SRCCOPY );
		/*draw the drag image using True Mask method*/
		BitBlt ( hDragWndDC, pDragImage->ptPos.x, 
				 pDragImage->ptPos.y,
 				 pDragImage->ptSize.x, pDragImage->ptSize.y,
				 pDragImage->hImageDC, 0, 0, SRCINVERT );
		BitBlt ( hDragWndDC, pDragImage->ptPos.x,
				 pDragImage->ptPos.y,
				 pDragImage->ptSize.x, pDragImage->ptSize.y,
				 pDragImage->hMaskDC, 0, 0, SRCAND );
		BitBlt ( hDragWndDC, pDragImage->ptPos.x,
				 pDragImage->ptPos.y,
				 pDragImage->ptSize.x, pDragImage->ptSize.y,
				 pDragImage->hImageDC, 0, 0, SRCINVERT );
		
/*		LockWindowUpdate ( hWnd );*/
	}

	if ( hDragWndDC )
		ReleaseDC ( hWnd, hDragWndDC );
	if ( pDragImage )
		GlobalUnlock ( hDragImage );
	return ( ErrorCode  >= 0 );
}

/*----------------------------------------------------------------------------*/
BOOL WINAPI
ImageList_DragLeave
(
	HWND	hWnd
)

{

	int	ErrorCode = 0;

	InvalidateRect ( hWnd, NULL, TRUE );

	return ( ErrorCode >= 0 );

}

/*----------------------------------------------------------------------------*/
BOOL WINAPI
ImageList_DragMove
(
	int				WindowX,  /* Note this is not ClientX */
	int				WindowY   /* Note this is not ClientY */
)

{

	HDC				hDragWndDC = ( HDC )NULL;
	HGLOBAL				hDragImage = ( HGLOBAL )NULL;
	WDRAGIMAGE*			pDragImage = ( WDRAGIMAGE* )NULL;

	int				ErrorCode = 0;

	if ( ! ( hDragImage = GetProp ( GetDesktopWindow (), szProperty ) ) ||
	     ! ( pDragImage = ( WDRAGIMAGE *) GlobalLock ( hDragImage  )    ) 
	   )
		ErrorCode = WIL_ERR_GLOBALLOCK;
	else
	if ( ! ( hDragWndDC = GetWindowDC ( pDragImage->hDragWnd ) ) )
		ErrorCode = WIL_ERR_GETDC;
	else
	{
		/*restore the background where the image was previously drawn on*/
		BitBlt ( hDragWndDC, pDragImage->ptPos.x, pDragImage->ptPos.y,
						 pDragImage->ptSize.x, pDragImage->ptSize.y,
			 pDragImage->hBkgroundDC, 0, 0, SRCCOPY );

		pDragImage->ptPos.x = WindowX - pDragImage->ptHotSpot.x;
		pDragImage->ptPos.y = WindowY - pDragImage->ptHotSpot.y;
		/*record the background where the drag image is to be drawn on*/
		BitBlt ( pDragImage->hBkgroundDC, 0, 0,
						  pDragImage->ptSize.x,
						  pDragImage->ptSize.y,
			 hDragWndDC, pDragImage->ptPos.x, pDragImage->ptPos.y, SRCCOPY );
		if ( pDragImage->bShow )
		{
			/*draw the drag image using the true mask method*/
			BitBlt ( hDragWndDC, pDragImage->ptPos.x, pDragImage->ptPos.y,
 							 pDragImage->ptSize.x,
							 pDragImage->ptSize.y,
					 pDragImage->hImageDC, 0, 0, SRCINVERT );
			BitBlt ( hDragWndDC, pDragImage->ptPos.x, pDragImage->ptPos.y,
							 pDragImage->ptSize.x,
							 pDragImage->ptSize.y,
					 pDragImage->hMaskDC, 0, 0, SRCAND );
			BitBlt ( hDragWndDC, pDragImage->ptPos.x, pDragImage->ptPos.y,
							 pDragImage->ptSize.x,
							 pDragImage->ptSize.y,
					 pDragImage->hImageDC, 0, 0, SRCINVERT );
		}
	}

	if ( pDragImage )
		GlobalUnlock ( hDragImage );
	if ( hDragWndDC )
		ReleaseDC ( pDragImage->hDragWnd, hDragWndDC );

	return ( ErrorCode >= 0 );

}

/*----------------------------------------------------------------------------*/
BOOL WINAPI
ImageList_DragShowNolock
(
	BOOL				Show
)

{

	HGLOBAL				hDragImage = ( HGLOBAL )NULL;
	WDRAGIMAGE*			pDragImage = ( WDRAGIMAGE* )NULL;

	int				ErrorCode = 0;

	if ( ! ( hDragImage = GetProp ( GetDesktopWindow (), szProperty ) ) ||
	     ! ( pDragImage = ( WDRAGIMAGE *) GlobalLock ( hDragImage  )    ) 
	   )
		ErrorCode = WIL_ERR_GLOBALLOCK;
	else
	{
		 pDragImage->bShow = Show;
		 ImageList_DragMove ( pDragImage->ptPos.x, pDragImage->ptPos.y );
	}

	if ( pDragImage )
		GlobalUnlock ( hDragImage );
	
	return ( ErrorCode >= 0 );

}

/*----------------------------------------------------------------------------*/
BOOL WINAPI
ImageList_Draw
(
	WHIMAGELIST			hImageList,
	int				ImageIndex,
	HDC				hDC,
	int				X,
	int				Y,
	UINT				DrawFlags
)

{

	return ImageList_DrawEx ( hImageList, ImageIndex, hDC,
			X, Y, 0, 0, hImageList->BackgroundColor, 
			( COLORREF )GetSysColor ( COLOR_HIGHLIGHT ), DrawFlags);
}

/*----------------------------------------------------------------------------*/
BOOL WINAPI
ImageList_DrawEx
(
	WHIMAGELIST		hImageList,
	int			ImageIndex,
	HDC			hDC,
	int			X,
	int			Y,
	int			DX,
	int			DY,
	COLORREF		BackgroundColor,
	COLORREF		ForegroundColor,
	UINT			DrawFlags
)

{

	int	ErrorCode	        = FALSE;
	WIMAGE		*pImageBlock	= ( WIMAGE* )NULL;
	int	SrcX		= 0;
	int	SrcY	        = 0;


    	APISTR((LF_APICALL,"ImageList_DrawEx(WIMAGELIST=%x,%d,HDC=%x,%d,%d,%d,%dCOLORREF=%x,COLORREF=%x,UINT=%x)\n", 
		hImageList, ImageIndex, hDC,
		X, Y, DX, DY,
		BackgroundColor, ForegroundColor, DrawFlags));


    	if ( ImageIndex >= hImageList->ImgList.NumUsed ) {
    		APISTR((LF_APIRET,"ImageList_DrawEx: returns BOOL %x\n",
			ErrorCode));
        	return ErrorCode;
	}

	if ( BackgroundColor == CLR_NONE )
		DrawFlags |= ILD_TRANSPARENT;
	else {
	if ( BackgroundColor == CLR_DEFAULT )
		BackgroundColor = hImageList->BackgroundColor;

        }
	if ( ForegroundColor == CLR_NONE )
		ForegroundColor = GetBkColor ( hDC );
	else {
	if ( ForegroundColor == CLR_DEFAULT )
		ForegroundColor = GetSysColor ( COLOR_HIGHLIGHT );
	}

	if ( DX == 0 )
		DX = hImageList->Width;
	if ( DY == 0 )
		DY = hImageList->Height;

 	pImageBlock = &hImageList->ImgList;
	SrcX =  ImageIndex * hImageList->Width;

	if ( ( hImageList->CreateFlags & ILC_MASK ) ||
	     ( DrawFlags & ILD_TRANSPARENT ) ||
	     ( DrawFlags & ILD_MASK ) ) {

		/* blank out destination, where image will go */
		/* DSna */
		if(pImageBlock->hMaskDC) {
			BitBlt ( hDC, X, Y, DX, DY, pImageBlock->hMaskDC,
				SrcX, SrcY, SRCAND ); 
		}

		/* TBD do we copy to a temporary DC, and repeat the mask */
		/* before copying the bits? */

		/* DSo */
		if(pImageBlock->hMemoryDC) {
			HDC     hDesk 		= GetDC(0);
			HDC	hMaskDC		= CreateCompatibleDC(hDesk);
			HBITMAP	hbmNewMask	= ( HBITMAP )NULL;
			
			hbmNewMask = CreateCompatibleBitmap ( hMaskDC, DX,DY);
			hbmNewMask = SelectObject ( hMaskDC, hbmNewMask );

			BitBlt ( hMaskDC, 0, 0, DX, DY, pImageBlock->hMemoryDC, 
				SrcX, SrcY, SRCCOPY);
			BitBlt ( hMaskDC, 0, 0, DX, DY, pImageBlock->hMaskDC,
				SrcX, SrcY, 0x220326L);

#if 0
    			BitBlt ( hDC, X, Y, DX, DY, hMaskDC,
				0,0,SRCPAINT);
#else
    			BitBlt (hDC,X,Y,DX,DY,pImageBlock->hMemoryDC,
				SrcX,SrcY,SRCPAINT);
#endif

			SelectObject ( hMaskDC, hbmNewMask );

			DeleteObject(hbmNewMask);
			DeleteDC ( hMaskDC );
			ReleaseDC(0,hDesk);	
		}
		
	}
	else   /* draw normal*/
	{

    		if(pImageBlock->hMemoryDC) {

    			BitBlt ( hDC, X, Y, DX, DY, pImageBlock->hMemoryDC,
				SrcX,SrcY,SRCCOPY);

			ErrorCode = TRUE;
		}
	}

	
    	APISTR((LF_APIRET,"ImageList_DrawEx: returns BOOL %x\n",ErrorCode));
	return ( ErrorCode );

}

/*----------------------------------------------------------------------------*/
BOOL WINAPI
ImageList_EndDrag ( )
{

	HANDLE				hDragImage;
	WDRAGIMAGE*			pDragImage	= ( WDRAGIMAGE* )NULL;

	int				ErrorCode	= 0;

	if ( ! ( hDragImage = RemoveProp ( GetDesktopWindow (), szProperty ) ) )
		ErrorCode = WIL_ERR_INVALIDVALUE;
	else
	if ( ! ( pDragImage = ( WDRAGIMAGE* )GlobalLock ( hDragImage ) ) )
		ErrorCode = WIL_ERR_GLOBALLOCK;
	else
	{
		if ( pDragImage->hBkgroundDC )
			DeleteDC ( pDragImage->hBkgroundDC );
		if ( pDragImage->hImageDC )
			DeleteDC ( pDragImage->hImageDC );
		if ( pDragImage->hMaskDC )
			DeleteDC ( pDragImage->hMaskDC );
		if ( pDragImage->pImageList )
			ImageList_Destroy ( pDragImage->pImageList );

		GlobalUnlock ( hDragImage );
		GlobalFree ( hDragImage );	
	}

	return ( ErrorCode >= 0 );

}


/*----------------------------------------------------------------------------*/
COLORREF WINAPI
ImageList_GetBkColor
(
	WHIMAGELIST			hImageList
)

{

    	APISTR((LF_APICALL,"ImageList_GetBkColor(HIMAGELIST=%x)\n", 
		hImageList));
    	APISTR((LF_APIRET,"ImageList_GetBkColor returns COLORREF %x\n",
		hImageList->BackgroundColor));
	return ( hImageList->BackgroundColor );

}

/*----------------------------------------------------------------------------*/
WHIMAGELIST WINAPI
ImageList_GetDragImage
(
	POINT		*pDragPos,
	POINT		*pHotSpot
)

{
	HGLOBAL		hDragImage	= ( HGLOBAL )NULL;
	WDRAGIMAGE*	pDragImage	= ( WDRAGIMAGE* )NULL;
	WHIMAGELIST	pList		= ( WHIMAGELIST )NULL;

	int		ErrorCode	= 0;

    	APISTR((LF_APICALL,"ImageList_GetDragImage(LPPOINT=%p,LPPOINT=%p)\n", 
		pDragPos,pHotSpot));

	if ( ! ( hDragImage = GetProp ( GetDesktopWindow (), szProperty ) ) ||
	     ! ( pDragImage = ( WDRAGIMAGE *) GlobalLock ( hDragImage ) ) 
	   )
		ErrorCode = WIL_ERR_GLOBALLOCK;
	else
	if ( ! ( pList = pDragImage->pImageList ) )
		ErrorCode = WIL_ERR_INVALIDVALUE;
	else
	{
		pHotSpot->x = pDragImage->ptHotSpot.x;
		pHotSpot->y = pDragImage->ptHotSpot.y;
		pDragPos->x = pDragImage->ptPos.x;
		pDragPos->y = pDragImage->ptPos.y;
	}	

	if ( pDragImage )
		GlobalUnlock ( hDragImage );

    	APISTR((LF_APIRET,"ImageList_GetDragImage returns HIMAGELIST %x\n",
		pList));
	return ( pList );

}

/*----------------------------------------------------------------------------*/
HICON WINAPI
ImageList_GetIcon
(
	WHIMAGELIST	hImageList,
	int		ImageIndex,
	UINT		DrawFlags
)

{
	ICONINFO	IconInfo;
	HDC		hDeskDC		= GetDC ( GetDesktopWindow () );
	HDC		hMemDC		= ( HDC )NULL;
	HDC		hMemMaskDC	= ( HDC )NULL;
	HICON		hIcon		= ( HICON )NULL;
	WIMAGE		*pImage	        = ( WIMAGE* )NULL;

	int		ErrorCode = 0;

    	APISTR((LF_APICALL,"ImageList_GetIcon(HIMAGELIST=%x,%d,UINT=%x)\n", 
		hImageList,ImageIndex,DrawFlags));

	memset ( &IconInfo, 0, sizeof ( ICONINFO ) );

	pImage = &hImageList->ImgList;
    	if ( ImageIndex >= hImageList->ImgList.NumUsed )
		ErrorCode = WIL_ERR_INDEXOUTOFRANGE;	
	else
	if ( ! ( hMemDC = CreateCompatibleDC( hDeskDC ) ) ||
	     ! ( hMemMaskDC = CreateCompatibleDC( hDeskDC ) )
	   )
		ErrorCode = WIL_ERR_CREATECOMPATIBLEDC;
	else
	if ( ! ( IconInfo.hbmColor = CreateCompatibleBitmap( hDeskDC, 
			hImageList->Width, 
			hImageList->Height ) ) ||
	     ! ( IconInfo.hbmMask = CreateCompatibleBitmap( hMemMaskDC, 
			hImageList->Width, 
			hImageList->Height ) )
	   )
		ErrorCode = WIL_ERR_CREATECOMPATIBLEBITMAP;
	else
	{
		IconInfo.fIcon = TRUE;
		IconInfo.xHotspot = 0;
		IconInfo.yHotspot = 0;

		
		SelectObject( hMemMaskDC, IconInfo.hbmMask );
		if ( ( hImageList->CreateFlags & ILC_MASK ) &&
			( pImage->hMaskDC ) )
		{

			BitBlt( hMemMaskDC, 0, 0,
				hImageList->Width, 
				hImageList->Height, 
				pImage->hMaskDC,
				ImageIndex * hImageList->Width, 0,
				SRCCOPY ); 
		}
		else
			PatBlt ( hMemMaskDC,0,0,
				hImageList->Width, 
				hImageList->Height,
				BLACKNESS );

		SelectObject( hMemDC, IconInfo.hbmColor );
		BitBlt( hMemDC, 0, 0,
		        hImageList->Width, 
			hImageList->Height, 
			pImage->hMemoryDC,
		        ImageIndex * hImageList->Width , 
			0,SRCCOPY ); 

		hIcon = CreateIconIndirect ( &IconInfo );
	}

	if ( IconInfo.hbmColor )
		DeleteObject ( IconInfo.hbmColor );

	if ( IconInfo.hbmMask )
		DeleteObject ( IconInfo.hbmMask );

	if ( hMemDC )
		DeleteDC ( hMemDC );

	if ( hMemMaskDC )
		DeleteDC ( hMemMaskDC );

	if ( hDeskDC )
		ReleaseDC ( GetDesktopWindow (), hDeskDC );

    	APISTR((LF_APIRET,"ImageList_GetDragImage returns HICON %x\n",
		hIcon));
	return ( hIcon );

}

/*----------------------------------------------------------------------------*/
BOOL WINAPI
ImageList_GetIconSize
(
	WHIMAGELIST	hImageList,
	int		*pIconWidth,
	int		*pIconHeight
)

{

	int	ErrorCode = 0;

    	APISTR((LF_APICALL,"ImageList_GetIconSize(HIMAGELIST=%x,int *=%p,int *=%p)\n", 
		hImageList,pIconWidth,pIconHeight));

	if (hImageList) {
		if ( hImageList->Width <= 0  ||	hImageList->Height <= 0 )
			ErrorCode = WIL_ERR_INVALIDVALUE;
		else
		{
			*pIconWidth  = hImageList->Width;
			*pIconHeight = hImageList->Height;
		}
		ErrorCode = 1;
	}
	
    	APISTR((LF_APIRET,"ImageList_GetIconSize returns BOOL %x\n",
		ErrorCode >= 0));
	return ( ErrorCode >= 0 );
}

/*----------------------------------------------------------------------------*/
int WINAPI
ImageList_GetImageCount 
( 
	WHIMAGELIST		hImageList
)

{
    	APISTR((LF_APICALL,"ImageList_GetImageCount(HIMAGELIST=%x)\n", 
		hImageList));
    	APISTR((LF_APIRET,"ImageList_GetImageCount returns %d\n",
		hImageList->ImgList.NumUsed));

    	return hImageList->ImgList.NumUsed;
}

BOOL WINAPI
ImageList_GetImageInfo 
( 
	WHIMAGELIST	hImageList,
	int		ImageIndex,
	IMAGEINFO FAR   *pImageInfo
)

{
	HDC		hDeskDC		= GetDC ( GetDesktopWindow () );
	HDC		hMemDC		= ( HDC )NULL;
	WIMAGE		*pImage	        = ( WIMAGE* )NULL;
	int		ErrorCode	= 0;

    	APISTR((LF_APICALL,"ImageList_GetImageInfo(HIMAGELIST=%x,%d,IMAGEINFO *=%p)\n", 
		hImageList,ImageIndex,pImageInfo));

	memset ( pImageInfo, 0, sizeof ( IMAGEINFO ) );
	
    	pImage = &hImageList->ImgList;
    	if ( ImageIndex >= hImageList->ImgList.NumUsed )
		ErrorCode = WIL_ERR_INDEXOUTOFRANGE;	
	else
	if ( ! ( hMemDC = CreateCompatibleDC( hDeskDC ) ) )
		ErrorCode = WIL_ERR_CREATECOMPATIBLEDC;
	else
	if ( ! ( pImageInfo->hbmImage = CreateCompatibleBitmap ( hDeskDC, 
					pImage->NumUsed * hImageList->Width, 
					hImageList->Height ) ) ||
	       ( ( hImageList->CreateFlags & ILC_MASK ) &&
		 ! ( pImageInfo->hbmMask  = CreateCompatibleBitmap ( hDeskDC, 
					pImage->NumUsed * hImageList->Width, 
					hImageList->Height ) ) ) )
	{

		SelectObject ( hMemDC, pImageInfo->hbmImage );
		BitBlt( hMemDC, 0, 0,
		        pImage->NumUsed * hImageList->Width, 
			hImageList->Height, 
			pImage->hMemoryDC, 0, 0, SRCCOPY ); 
		 
		SelectObject ( hMemDC, pImageInfo->hbmMask );

		if ( hImageList->CreateFlags & ILC_MASK ) {
			if ( pImage->hMaskDC ) 
				BitBlt( hMemDC, 0, 0,
					pImage->NumUsed * hImageList->Width, 
					hImageList->Height, 
					pImage->hMaskDC, 0,	0, SRCCOPY ); 
			else
				PatBlt ( hMemDC, 0, 0, 
					 pImage->NumUsed * hImageList->Width, 
					 hImageList->Height, BLACKNESS );
		}
		pImageInfo->rcImage.left = ImageIndex * hImageList->Width;
		pImageInfo->rcImage.top  = 0;
		pImageInfo->rcImage.right = pImageInfo->rcImage.left + 
			hImageList->Width;
		pImageInfo->rcImage.bottom = hImageList->Height;		
	}

	if ( hMemDC )
		DeleteDC ( hMemDC );

	if ( hDeskDC )
		ReleaseDC ( GetDesktopWindow (), hDeskDC );
    	APISTR((LF_APIRET,"ImageList_GetImageInfo returns BOOL %x\n",
		ErrorCode >=0));
	return ( ErrorCode >= 0 );
}

/*----------------------------------------------------------------------------*/
WHIMAGELIST WINAPI
ImageList_LoadBitmap
(
	HINSTANCE			hInstance,
	LPCSTR				pstrBitmap,
	int				    SubImageWidth,
	int				    NumberOfImagesToGrowBy,
	COLORREF			MaskColor
)

{

	return ImageList_LoadImage(hInstance, pstrBitmap, 
		SubImageWidth,
		NumberOfImagesToGrowBy, 
		MaskColor, 
		IMAGE_BITMAP, 
		0);

}

/*----------------------------------------------------------------------------
 *Note: need revisit. 
 *----------------------------------------------------------------------------*/
WHIMAGELIST WINAPI
ImageList_LoadImageW
(
	HINSTANCE			hInstance,
	LPCSTR				pstrImage,
	int				SubImageWidth,
	int				NumberOfImagesToGrowBy,
	COLORREF			MaskColor,
	UINT				ImageType,
	UINT				LoadFlags
)
{
	return ImageList_LoadImage(hInstance,pstrImage,
		SubImageWidth,
		NumberOfImagesToGrowBy,
		MaskColor,
		ImageType,
		LoadFlags);
}

WHIMAGELIST WINAPI
ImageList_LoadImage
(
	HINSTANCE			hInstance,
	LPCSTR				pstrImage,
	int				SubImageWidth,
	int				NumberOfImagesToGrowBy,
	COLORREF			MaskColor,
	UINT				ImageType,
	UINT				LoadFlags
)

{

	BITMAP				bmColor;
	HBITMAP				hBitmap		= ( HBITMAP )NULL;
	HBITMAP				hbmMono		= ( HBITMAP )NULL;
	WHIMAGELIST			hImageList	= ( WHIMAGELIST )NULL;
	int				ErrorCode	= 0;
	UINT				CreateFlags	= 0;
	HDC				hDeskDC		= GetDC ( GetDesktopWindow () );
	HDC				hColorDC	= ( HDC )NULL;
	HDC				hMonoDC		= ( HDC )NULL;

	memset ( &bmColor, 0, sizeof ( BITMAP ) );

	if ( ! ( hBitmap = W_ILoadImage ( hInstance, pstrImage, 
					MaskColor, ImageType, LoadFlags ) ) )
		ErrorCode = WIL_ERR_LOADIMAGE;
	else
	if ( ! ( GetObject ( hBitmap, sizeof ( BITMAP ), &bmColor ) ) )
		ErrorCode = WIL_ERR_GETOBJECT;
	else
	if ( ( LoadFlags & LR_MONOCHROME ) &&
	     ( ! ( hColorDC = CreateCompatibleDC ( hDeskDC ) ) ||
	       ! ( hMonoDC = CreateCompatibleDC ( hDeskDC ) )  ||
	       ! ( hbmMono = CreateBitmap ( bmColor.bmWidth, bmColor.bmHeight, 1, 1, NULL ) )
	     )
	   )
		ErrorCode = WIL_ERR_CREATECOMPATIBLEDC;
	else
	{
		if ( LoadFlags & LR_MONOCHROME ) 
		{
			hBitmap = SelectObject ( hColorDC, hBitmap );
			hbmMono = SelectObject ( hMonoDC, hbmMono );
			BitBlt ( hMonoDC, 0, 0, bmColor.bmWidth, bmColor.bmHeight,
				 hColorDC, 0, 0, SRCCOPY );
			hBitmap = SelectObject ( hColorDC, hBitmap );
			hbmMono = SelectObject ( hMonoDC, hbmMono );
			DeleteObject ( hBitmap );
			hBitmap = hbmMono;
		}
		else
			CreateFlags |= ILC_MASK;	

		if ( SubImageWidth <= 0 )
		{
			SubImageWidth = ( ImageType & IMAGE_ICON ) ? GetSystemMetrics ( SM_CXICON ) :	
					( ImageType & IMAGE_CURSOR ) ? GetSystemMetrics ( SM_CXCURSOR ) :					
					bmColor.bmWidth;
		}
		
		if ( SubImageWidth <= 0 )
			ErrorCode = WIL_ERR_INVALIDVALUE;
		else
		if ( ! ( hImageList = ImageList_Create ( SubImageWidth, 
							  bmColor.bmHeight, CreateFlags,
							  bmColor.bmWidth / SubImageWidth,
							  NumberOfImagesToGrowBy ) ) )
			ErrorCode = WIL_ERR_LISTCREATE;
		else
		if ( ImageList_AddMasked ( hImageList, hBitmap, MaskColor ) == -1 )
		{
			ImageList_Destroy ( hImageList );
			hImageList = NULL;
			ErrorCode = WIL_ERR_ADDMASKED;
		}
	}

	if ( hBitmap )
		DeleteObject ( hBitmap );

	if ( hColorDC )
		DeleteDC ( hColorDC );
	if ( hMonoDC )
		DeleteDC ( hMonoDC );

	if ( hDeskDC )
		ReleaseDC ( GetDesktopWindow (), hDeskDC );

	return ( hImageList );

}

/*----------------------------------------------------------------------------*/
WHIMAGELIST WINAPI
ImageList_Merge
(
	WHIMAGELIST			hImageList1,
	int				ImageIndex1,
	WHIMAGELIST			hImageList2,
	int				ImageIndex2,
	int				DX,
	int				DY
)

{

	POINT				ptNewSize;
	IMAGEINFO			ImageInfo1;
	IMAGEINFO			ImageInfo2;
	HDC				hDeskDC		= GetDC ( GetDesktopWindow () );
	WHIMAGELIST			pNewImageList	= ( WHIMAGELIST )NULL;
	HICON				hIcon		= ( HICON )NULL;
	HICON				hIcon1		= ( HICON )NULL;
	HICON				hIcon2		= ( HICON )NULL;
	int				ErrorCode	= 0;

	memset ( &ImageInfo1, 0, sizeof ( IMAGEINFO ) );
	memset ( &ImageInfo2, 0, sizeof ( IMAGEINFO ) );

	if ( ! ( hIcon1 = ImageList_GetIcon ( hImageList1, ImageIndex1, ILD_TRANSPARENT ) ) ||

	     ! ( hIcon2 = ImageList_GetIcon ( hImageList2, ImageIndex2, ILD_TRANSPARENT ) )
	   )
		ErrorCode = WIL_ERR_GETICON;
	else
	if ( ! ( hIcon = W_IOverDrawIcon ( hIcon1, hIcon2, DX, DY, &ptNewSize) ) ) 
		ErrorCode =  WIL_ERR_DRAWICON;
	else
	if ( ! ( pNewImageList = ImageList_Create ( ptNewSize.x, ptNewSize.y,
				    hImageList1->CreateFlags, 10, 10 ) )
	   )
		ErrorCode =  WIL_ERR_LISTCREATE ;
	else
	{
		ImageList_ReplaceIcon ( pNewImageList, -1, hIcon );
	}

	if ( ImageInfo1.hbmImage )
		DeleteObject ( ImageInfo1.hbmImage );
	if ( ImageInfo1.hbmMask )
		DeleteObject ( ImageInfo1.hbmMask );
	if ( ImageInfo2.hbmImage )
		DeleteObject ( ImageInfo2.hbmImage );
	if ( ImageInfo2.hbmMask )
		DeleteObject ( ImageInfo2.hbmMask );

	if ( hDeskDC )
		ReleaseDC ( GetDesktopWindow (), hDeskDC );

	return pNewImageList;

}

/*----------------------------------------------------------------------------*/
BOOL WINAPI
ImageList_Remove 
( 
	WHIMAGELIST	hImageList, 
	int		ImageIndex
)

{

	WIMAGE		*pImage	        = &hImageList->ImgList;
	int		ErrorCode	= 0;

    	APISTR((LF_APICALL,"ImageList_Remove(HIMAGELIST=%x,%d)\n", 
		hImageList,ImageIndex));

    	if ( ImageIndex == -1)
		W_IRemoveAll ( pImage );
	else
	if ( ! Image_IRemove( pImage, ImageIndex ,
			       hImageList->Height, hImageList->Width,
			       hImageList->CreateFlags & ILC_MASK ) )
		ErrorCode = WIL_ERR_LISTREMOVE;
	
    	APISTR((LF_APIRET,"ImageList_Remove returns BOOL %x\n",
		ErrorCode >= 0));
	return ( ErrorCode >= 0 );

}

/*----------------------------------------------------------------------------*/
BOOL WINAPI
ImageList_Replace
(
	WHIMAGELIST		hImageList,
	int			ImageIndex,
	HBITMAP			hBitmap,
	HBITMAP			hMask
)
{
	int			ErrorCode           = 0;

    	APISTR((LF_APICALL,"ImageList_Replace(IMAGELIST=%x,%d,%x,%x)\n", 
		hImageList,ImageIndex,hBitmap,hMask));

	if ( ( hImageList->ImgList.NumUsed == 0 ) && 
         ( ImageIndex != WIL_ADDINDEX ) )
		ErrorCode = WIL_ERR_INDEXOUTOFRANGE;
	else
	{
		if ( ImageIndex != WIL_ADDINDEX )
		{ /*--- Replacing rather than adding ---*/
			ErrorCode = ImageList_ICopyImage ( hImageList, ImageIndex, hBitmap, hMask );
		}
		else
		{ /*--- Adding an image as opposed to replacing ---*/
			ErrorCode = ( ImageList_IAddImage ( hImageList, 1, hBitmap, hMask ) == -1 );
		}
	}

    	APISTR((LF_APIRET,"ImageList_Replace returns BOOL %x\n",
		ErrorCode >= 0));
	return ( ErrorCode >= 0 );

}

/*----------------------------------------------------------------------------*/
int WINAPI
ImageList_ReplaceIcon
(
	WHIMAGELIST		hImageList,
	int		    	ImageIndex,
	HICON			hIcon
)

{

	int		    	ErrorCode = 0;
	ICONINFO		IconInfo;

	memset ( &IconInfo,0, sizeof ( ICONINFO ) );
	
	if ( ImageIndex < -1 )
		 ErrorCode = WIL_ERR_INVALIDVALUE;
	else
	if ( ! GetIconInfo ( hIcon, &IconInfo ) )
		 ErrorCode =  WIL_ERR_GETICON;
	else
	if ( ImageIndex == -1 )
		 ImageIndex = ImageList_IAddImage ( hImageList, 1, 
                            IconInfo.hbmColor, IconInfo.hbmMask );
	else
	if ( ! ImageList_Replace ( hImageList, ImageIndex, 
			IconInfo.hbmColor, IconInfo.hbmMask ) )
		 ErrorCode =  WIL_ERR_REPLACEICON;

	if ( ErrorCode >= 0 )
		return ImageIndex;
	else
		return -1;

}

/*----------------------------------------------------------------------------*/
COLORREF WINAPI
ImageList_SetBkColor
(
	WHIMAGELIST			hImageList,
	COLORREF			BackgroundColor
)

{

	COLORREF ReturnColor = hImageList->BackgroundColor;

	hImageList->BackgroundColor = BackgroundColor;
	
	return ( ReturnColor );

}

/*----------------------------------------------------------------------------*/
BOOL WINAPI
ImageList_SetDragCursorImage
(
	WHIMAGELIST		hImageList,
	int		    	ImageIndex,
	int		    	HotSpotX,
	int		        HotSpotY
)

{
	POINT			ptNewSize;
	ICONINFO		IconInfo;
	HDC		    	hDeskDC		= GetDC ( GetDesktopWindow () );
	HICON			hIcon		= ( HICON )NULL;
	HICON			hIcon1		= ( HICON )NULL;
	HICON			hIcon2		= ( HICON )NULL;
	HBITMAP			hBkBitmap	= ( HBITMAP )NULL;
	HGLOBAL			hDragImage	= ( HGLOBAL )NULL;
	WDRAGIMAGE*		pDragImage	= ( WDRAGIMAGE* )NULL;

	int				    ErrorCode	= 0;

	memset ( &IconInfo, 0, sizeof ( ICONINFO ) );

	if ( ! ( hDragImage = GetProp ( GetDesktopWindow (), szProperty ) ) ||
	     ! ( pDragImage = ( WDRAGIMAGE *) GlobalLock ( hDragImage  )    ) 
	   )
		ErrorCode = WIL_ERR_GLOBALLOCK;
	else
	if ( ! ( hIcon2 = ImageList_GetIcon ( hImageList, ImageIndex, ILD_TRANSPARENT ) ) ||
	     ! ( hIcon1 = ImageList_GetIcon ( pDragImage->pImageList, 0, ILD_TRANSPARENT ) )	   
	   )
		ErrorCode = WIL_ERR_GETICON;
	else
	if ( ! ( hIcon = W_IOverDrawIcon ( hIcon1, hIcon2, HotSpotX, HotSpotY, &ptNewSize) ) ) 
		ErrorCode =  WIL_ERR_DRAWICON;
	else
	if ( ! GetIconInfo ( hIcon, &IconInfo ) )
		ErrorCode = WIL_ERR_GETIMAGEINFO;
	else
	if ( ! ( hBkBitmap = CreateCompatibleBitmap ( hDeskDC, ptNewSize.x, ptNewSize.y ) ) )
		ErrorCode = WIL_ERR_CREATECOMPATIBLEBITMAP;
	else
	{
		IconInfo.hbmColor = SelectObject ( pDragImage->hImageDC, IconInfo.hbmColor );
		IconInfo.hbmMask = SelectObject ( pDragImage->hMaskDC, IconInfo.hbmMask );
		hBkBitmap = SelectObject ( pDragImage->hBkgroundDC, hBkBitmap );
		pDragImage->ptSize.x = ptNewSize.x;
		pDragImage->ptSize.y = ptNewSize.y;
		pDragImage->ptHotSpot.x = HotSpotX;
		pDragImage->ptHotSpot.y = HotSpotY;
	}

	
	if ( hBkBitmap )
		DeleteObject ( hBkBitmap );

	if ( hIcon )
		DestroyIcon ( hIcon );
	if ( hIcon1 )
		DestroyIcon ( hIcon1 );
	if ( hIcon2 )
		DestroyIcon ( hIcon2 );

	if ( IconInfo.hbmColor )
		DeleteObject ( IconInfo.hbmColor );
	if ( IconInfo.hbmMask )
		DeleteObject ( IconInfo.hbmMask );

	if ( hDeskDC )
		ReleaseDC ( GetDesktopWindow (), hDeskDC );

	if ( pDragImage )
		GlobalUnlock ( hDragImage );

	return ( ErrorCode >= 0 );

}

#if ! defined(TWIN_BIN32)
#define BIStream_Read(p,a,b,c) (*((FARPROC)p->lpvtbl->fnRead))(p,(LPSTR)a,b,c)
#else
extern int RelayStdcall_4(FARPROC,LPSTREAM, LPSTR, UINT, ULONG *);
#define BIStream_Read(p,a,b,c) RelayStdcall_4((FARPROC)p->lpvtbl->fnRead,p,(LPSTR)a,b,c)
#endif

typedef struct {
	USHORT	type;
	USHORT	sizelo;
	USHORT	sizehi;
	USHORT	res1;
	USHORT	res2;
	USHORT	offbitslo;
	USHORT	offbitshi;
} BINBITMAPFILEHEADER;

/*----------------------------------------------------------------------------*
**  Convenience function for ImageList_Read()
**----------------------------------------------------------------------------*/
static HANDLE DIBitmapFromStream(LPSTREAM pstm)
{
    HANDLE hDIB;

    BITMAPFILEHEADER bf;
    BITMAPINFOHEADER bi;
    BINBITMAPFILEHEADER bbh;

    DWORD dwHeaderSize;
    DWORD dwColorTableSize;
    DWORD dwImageSize;
    DWORD dwTotalSize;

    /* Offsets into the GlobalAlloc() memory */
    BYTE *pInfoHeader;
    BYTE *pColorTable;
    BYTE *pImageData;

    HRESULT                 rc;
    ULONG                   bytes_read;

    /*-----------------------------------------------------------------------
    **  Read the bitmap headers:
    **   1) BITMAPFILEHEADER
    **   2) BITMAPINFOHEADER
    **---------------------------------------------------------------------*/

    rc = BIStream_Read(pstm, &bbh, sizeof(bbh), &bytes_read);
    if (rc != S_OK || bytes_read != sizeof(bbh))
    {
	LOGSTR((LF_LIBFAIL, "DIBitmapFromStream() returning NULL...\n"
		"  (Failed to read BITMAPFILEHEADER)\n"));
        return (HANDLE)NULL;
    }

    bf.bfType = bbh.type;
    bf.bfSize = MAKELONG(bbh.sizehi,bbh.sizelo);
    bf.bfOffBits = MAKELONG(bbh.offbitshi,bbh.offbitslo); 
 
    rc = BIStream_Read(pstm, &bi, sizeof(bi), &bytes_read);
    if (rc != S_OK || bytes_read != sizeof(bi))
    {
	LOGSTR((LF_LIBFAIL, "DIBitmapFromStream() returning NULL...\n"
		"  (Failed to read BITMAPINFOHEADER)\n"));
        return (HANDLE)NULL;
    }

    /*-----------------------------------------------------------------------
    **  Copy all bitmap info into a global memory chunk.  This includes a
    **  BITMAPINFOHEADER, a (variable size) color table, and raw pixel data.
    **  This global memory chunk will be binary equivalent to a loaded DIB.
    **---------------------------------------------------------------------*/

    /* Make code easier to read */
    dwHeaderSize = bi.biSize;
    dwColorTableSize = (DWORD)W_IPaletteSize((LPSTR)&bi);
    dwImageSize = bi.biSizeImage;
    dwTotalSize = dwHeaderSize + dwColorTableSize + dwImageSize;

    /* Allocate space for everything */
    hDIB = GlobalAlloc(GHND, dwTotalSize);
    if (hDIB == (HANDLE)NULL)
    {
	LOGSTR((LF_LIBFAIL, "DIBitmapFromStream() returning NULL...\n"
		"  (Failed to allocate %ld bytes for DIB)\n", dwTotalSize));
        return (HANDLE)NULL;
    }

    /* Start with BITMAPINFOHEADER */
    pInfoHeader = (BYTE*)GlobalLock(hDIB);
    *(LPBITMAPINFOHEADER)pInfoHeader = bi;

    /* Copy color table into our block of memory */
    pColorTable = pInfoHeader + dwHeaderSize;

    if(dwColorTableSize) {
    	rc = BIStream_Read(pstm, pColorTable, dwColorTableSize, &bytes_read);
	
    	if (rc != S_OK || bytes_read != dwColorTableSize) {
		LOGSTR((LF_LIBFAIL, "DIBitmapFromStream() returning NULL...\n"
			"  (Failed to read color table)\n"));
        	return (HANDLE)NULL;
    	}
    }

    /* Copy bitmap data from the stream to a temporary block of memory */
    /*  and then into our global memory */
    pImageData = pInfoHeader + dwHeaderSize + dwColorTableSize;

    rc = BIStream_Read(pstm, pImageData, dwImageSize, &bytes_read);
    if (rc != S_OK || bytes_read != dwImageSize)
    {
	LOGSTR((LF_LIBFAIL, "DIBitmapFromStream() returning NULL...\n"
		"  (Failed to read image data into DIB)\n"));
        return (HANDLE)NULL;
    }

    /* Release pInfoHeader and other related pointers */
    GlobalUnlock(hDIB);

    return (hDIB);
}

typedef struct {
	USHORT	sig;
	USHORT	ver;
	USHORT  images_possible_now;
	USHORT  images_initial_count;
	USHORT  images_extra_allocate;
	USHORT	subwidth; 
	USHORT	subheight;
	USHORT	dummy1;
	USHORT	dummy2;
	char	fill;
	char    flags;
	USHORT	dummy3[4];
} IMAGELISTFILEHEADER;

#if SEVERE
static void
DumpBitmap(char *p,HBITMAP hBitmap,int x,int y)
{
	HDC hDC,hDCMem;
	BITMAP bm;
	POINT  ptSize,ptOrg;

	hDC = GetDC(0);
	hDCMem = CreateCompatibleDC(hDC);

	SelectObject(hDCMem,hBitmap);
	SetMapMode(hDCMem,hDC);
	GetObject(hBitmap,sizeof(BITMAP),(LPVOID) &bm);
	
	LOGSTR((LF_CONSOLE,"dump bitmap %s: %d,%d\n",
		p,bm.bmWidth,bm.bmHeight));

	ptSize.x = bm.bmWidth;
	ptSize.y = bm.bmHeight;
	DPtoLP(hDC,&ptSize,1);
	ptOrg.x = 0;
	ptOrg.y = 0;
	DPtoLP(hDCMem,&ptOrg,1);
	BitBlt(hDC,x,y,ptSize.x,ptSize.y,hDCMem,ptOrg.x,ptOrg.y,SRCCOPY);

	DeleteDC(hDCMem);
	ReleaseDC(0,hDC);
}
#endif

/*************************************************************************
 * STUB From Wine cvs 7/16/99.  Wine License applies.
 *
 * ImageList_Read [COMCTL32.66]
 *
 * Reads an image list from a stream.
 *
 * PARAMS
 *     pstm [I] pointer to a stream
 *
 * RETURNS
 *     Success: handle to image list
 *     Failure: NULL
 *
 * NOTES
 *     Makes use of reverse-engineered IMAGELISTFILEHEADER struct, so 
 *     it's possible we've missed some image formatting possibilities.
 *
 * BUGS
 *     
 */
HIMAGELIST WINAPI ImageList_Read (LPSTREAM pstm)
{
    /*-----------------------------------------------------------------------
    **  Local variables
    **---------------------------------------------------------------------*/
    HRESULT  rc;
    ULONG bytes_read;
    int i;

    HIMAGELIST list_handle;
    IMAGELISTFILEHEADER il;
    HDC hDC;
    HDC hDCSrc,hDCSrcMask;
    HDC hDCDst,hDCDstMask;

    HANDLE hDIB;
    HBITMAP hBitmap = (HBITMAP)NULL;
    HBITMAP hMaskBitmap = (HBITMAP)NULL;

    HBITMAP hBMDst     = (HBITMAP)NULL;
    HBITMAP hBMDstMask = (HBITMAP)NULL;

    APISTR((LF_APICALL,"ImageList_Read(LPSTREAM = %x)\n", pstm));

    /*-----------------------------------------------------------------------
    **  Read the IMAGELISTFILEHEADER header
    **---------------------------------------------------------------------*/
    rc = BIStream_Read(pstm, &il, sizeof(il), &bytes_read);
    if (rc != S_OK || bytes_read != sizeof(il))
    {
	APISTR((LF_APIFAIL, "ImageList_Read() returning NULL...\n"
		"  (Failed to read IMAGELISTFILEHEADER)\n"));
        return(NULL);
    }

    /*-----------------------------------------------------------------------
    **  Load in Image bitmap (as a DIB) from IStream, then convert to BMP
    **---------------------------------------------------------------------*/

    hDIB = DIBitmapFromStream(pstm);
    hBitmap = W_IDIBToBitmap(hDIB);

    GlobalFree(hDIB);

    /*-----------------------------------------------------------------------
    **  Load in Mask bitmap, if ILC_MASK is set
    **---------------------------------------------------------------------*/

    hDIB = DIBitmapFromStream(pstm);
    hMaskBitmap = W_IDIBToBitmap(hDIB);

    /*-----------------------------------------------------------------------
    **  Create an empty ImageList
    **---------------------------------------------------------------------*/

    {
	int flag = ILC_MASK;
    	list_handle = ImageList_Create(il.subwidth, il.subheight, 
		il.flags|flag,
		il.images_initial_count, 
		il.images_extra_allocate);
    }

    if (list_handle == NULL)
    {
	APISTR((LF_APIFAIL, "ImageList_Read() returning NULL\n"));
        return (NULL);
    }

    /*-----------------------------------------------------------------------
    **  BitBlt image-sized chunks of the initial bitmap into a temporary
    **  bitmap, then add one at a time to the ImageList.
    **
    **  hBitmap - color bitmap with all images in one big chunk
    **  hMaskBitmap - B&W mask bitmap with all masks in one big chunk
    **
    **---------------------------------------------------------------------*/

    hDC = GetDC(GetDesktopWindow());
    hDCSrc= CreateCompatibleDC(hDC);
    hDCSrcMask= CreateCompatibleDC(hDC);

    hDCDst= CreateCompatibleDC(hDC);
    hDCDstMask= CreateCompatibleDC(hDC);

    hBMDst     = CreateCompatibleBitmap(hDC, il.subwidth, il.subheight);
    hBMDstMask = CreateCompatibleBitmap(hDC, il.subwidth, il.subheight);

    /* these hold the source bitmaps */
    SelectBitmap(hDCSrc, hBitmap);
    SelectBitmap(hDCSrcMask,hMaskBitmap);
#if 0
DEBUG_DumpDC(hDCSrc,0,0);
DEBUG_DumpDC(hDCSrcMask,16,0);

APISTR((LF_CONSOLE, "ImageList_Read() dumped bitmaps\n"));
debuggerbreak();
#endif

    /* these hold the destination sub-bitmaps */
    SelectBitmap(hDCDst, hBMDst);
    SelectBitmap(hDCDstMask,hBMDstMask);

    for (i = 0; i < il.images_possible_now; i++)
    {
        int x, y;
	int cx,cy;

	x = (i % 4) * il.subwidth;
	y = (i / 4) * il.subheight;
	cx = il.subwidth; 
	cy = il.subheight;

	/* copy the subsection of the bitmap */
        BitBlt(hDCDst, 0, 0, il.subwidth, il.subheight,
	       hDCSrc, x, y, SRCCOPY);

	/* copy the mask portion of the bitmap */
	BitBlt(hDCDstMask, 0, 0, il.subwidth, il.subheight,
	       hDCSrcMask, x, y, SRCCOPY);

	ImageList_Add(list_handle, hBMDst, hBMDstMask);

    }

    /* Clean up our trash */
    ReleaseDC((HWND)NULL, hDC);

    /* these are the src/dst copy DC's */
    DeleteDC(hDCSrc);
    DeleteDC(hDCSrcMask);
    DeleteDC(hDCDst);
    DeleteDC(hDCDstMask);

    /* these are what we read in */
    DeleteBitmap(hBitmap);
    DeleteBitmap(hMaskBitmap);

    /* these are the transient copies */
    DeleteBitmap(hBMDst);
    DeleteBitmap(hBMDstMask);

    APISTR((LF_APIRET, "ImageList_Read() returning %x\n", list_handle));
    return(list_handle);
}


/*************************************************************************
 * STUB From Wine cvs 7/16/99.  Wine License applies.
 *
 * ImageList_Write [COMCTL32.80]
 *
 * Writes an image list to a stream.
 *
 * PARAMS
 *     himl [I] handle to image list
 *     pstm [O] Pointer to a stream.
 *
 * RETURNS
 *     Success: TRUE
 *     Failure: FALSE
 *
 * NOTES
 *     This function can not be implemented yet, because
 *     IStream32::Write is not implemented.
 *
 * BUGS
 *     empty stub.
 */

BOOL WINAPI
ImageList_Write (HIMAGELIST himl, LPSTREAM pstm)
{
    if (!himl)
	return FALSE;

    LOGSTR((LF_APISTUB,"ImageList_Write\n"));

    return FALSE;
}


/*----------------------------------------------------------------------------*/
BOOL WINAPI
ImageList_SetIconSize
(
	WHIMAGELIST		hImageList,
	int		    	SubImageHeight,
	int		    	SubImageWidth
)

{

	int		    	ErrorCode = 0;

	if ( ! hImageList )
		ErrorCode = WIL_ERR_INVALIDVALUE;
	else
	{
		hImageList->Height = SubImageHeight;
		hImageList->Width = SubImageWidth;
		return ImageList_Remove( hImageList, -1 );
	}

	return ( ErrorCode >= 0 );

}

/*----------------------------------------------------------------------------*/
BOOL WINAPI
ImageList_SetOverlayImage
(
	WHIMAGELIST		hImageList,
	int			ImageIndex,
	int			OverlayIndex
)

{

	int			ErrorCode = 0;

	if ( OverlayIndex <= 0  || OverlayIndex > MAXOVERLAY )
		ErrorCode = WIL_ERR_INDEXOUTOFRANGE;
	else
	{
		hImageList->OverlayIndices[OverlayIndex-1] = ImageIndex;
	}

	return ( ErrorCode >= 0);

}

/*----------------------------------------------------------------------------*
 *----------------------------------------------------------------------------*/

BOOL WINAPI
Image_IRemove
( 
	WIMAGE			*pImage,
	int			iRemoved,
	int			Height,
	int			Width,
	BOOL			bMask
)
{
	
	HDC		    	hNewDC		= ( HDC )NULL;
	HDC			hMaskDC		= ( HDC )NULL;
	HBITMAP			hNewBitmap	= ( HBITMAP )NULL;
	HBITMAP			hNewMask	= ( HBITMAP )NULL;
	int			ErrorCode	= 0;

    	if ( iRemoved >= pImage->NumUsed ) 
        	return FALSE;

	if(pImage->hMemoryDC == 0)
		return FALSE;

	if ( ! ( hNewDC = CreateCompatibleDC( pImage->hMemoryDC ) ) ||
	       ( bMask && ! ( hMaskDC = CreateCompatibleDC( pImage->hMaskDC ) ) )	)
	{
	       ErrorCode = WIL_ERR_CREATECOMPATIBLEDC;
	       if ( hNewDC )
			DeleteDC( hNewDC );
	       if ( hMaskDC )
			DeleteDC( hMaskDC );
	}
	else
	if ( ! ( hNewBitmap = CreateCompatibleBitmap ( pImage->hMemoryDC, 
			                      Width * pImage->NumImages, Height ) ) ||
	       ( bMask && ! ( hNewMask = CreateCompatibleBitmap( pImage->hMaskDC,
					      Width * pImage->NumImages, Height ) ) )
	   )
		ErrorCode = WIL_ERR_CREATECOMPATIBLEBITMAP;
	else
	{
		
		hNewBitmap = SelectObject ( hNewDC, hNewBitmap );
        	if ( iRemoved > 0 )
			BitBlt ( hNewDC, 0, 0, 
				 Width * iRemoved, Height,
				 pImage->hMemoryDC, 0, 0, SRCCOPY );

		BitBlt ( hNewDC, Width * iRemoved, 0, 
			     pImage->NumUsed - 1, Height,
			     pImage->hMemoryDC, Width * ( iRemoved + 1 ), 0, SRCCOPY );
		
		DeleteDC( pImage->hMemoryDC );
		pImage->hMemoryDC = hNewDC;
		DeleteObject ( hNewBitmap );

		if ( bMask )
		{
			hNewMask = SelectObject ( hMaskDC, hNewMask );
            		if ( iRemoved > 0 )
			    BitBlt ( hMaskDC, 0, 0, 
				     Width * iRemoved, Height,
				     pImage->hMaskDC, 0, 0, SRCCOPY );

		    	BitBlt ( hMaskDC, Width * iRemoved, 0, 
			         pImage->NumUsed - 1, Height,
			         pImage->hMaskDC, Width * ( iRemoved + 1 ), 0, SRCCOPY );
			
			DeleteDC( pImage->hMaskDC );
			pImage->hMaskDC = hMaskDC;
			DeleteObject ( hNewMask );
		}

		pImage->NumUsed--;
	}

	return (ErrorCode >= 0);
}

static BOOL 
W_IRemoveAll 
( 
   WIMAGE*			pImage
)
{
	int			    ErrorCode = 0;
	
	if ( !pImage )
		ErrorCode = WIL_ERR_INVALIDVALUE;
	else
	{
		if ( pImage->hMemoryDC )
			DeleteDC ( pImage->hMemoryDC );
		if ( pImage->hMaskDC )
			DeleteDC ( pImage->hMaskDC );
        	pImage->NumUsed = pImage->NumImages = 0;
	}

	return ( ErrorCode >= 0 );
}

/*------------------------- < Full Revision History > ----------------------
** $Log: WImgList.c,v $
** Revision 1.17  2000/01/28 08:21:38  robf
** fixes and dead code
**
** Revision 1.16  2000/01/26 19:55:22  jwhite
** This includes a wide range of changes required to get Twine to work
** with MFC on the Alpha platform.
**
** Revision 1.15  2000/01/19 03:56:07  robf
** protection on hImageList == 0
**
** Revision 1.14  2000/01/10 08:53:34  robf
** modified imagelist_write
**
** Revision 1.13  2000/01/05 22:00:15  robf
** debug for 24bits
**
** Revision 1.12  1999/12/21 16:06:21  robf
** fixed loading and draw functions
**
** Revision 1.11  1999/12/09 07:22:12  robf
** need HRESULT from wtypes.h
**
** Revision 1.10  1999/12/08 20:19:25  robf
** updated comments
**
** Revision 1.9  1999/11/30 01:04:59  robf
** hacks for binary interface, only works with binaries, no source
**
** Revision 1.8  1999/10/14 21:09:33  jsheets
** Fixed poor logging in ImageList_Read()
**
** Revision 1.7  1999/10/14 19:55:51  jsheets
** Added implementation for ImageList_Read, including support for masks
**
** Revision 1.6  1999/08/03 15:25:48  jwhite
** Added some stubs for Image List Read/Write
**
** Revision 1.5  1999/07/12 15:38:20  mwalsh
** Updated comments to standard header and footer style
**
**-------------------------------------------------------------------------*/
