/*--------------------------------------------------------------------------
**      DrvCursor.c           Twin           From:  Twin/graphics/x11drv
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      Copyright 1997 Willows Software, Inc.
**------------------------ < File Content Description > --------------------
**
**  Module:	graphics/x11drv/DrvCursor.c
**
**  Description:
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/graphics/x11drv/DrvCursor.c,v 1.5 1999/11/30 17:45:15 robf Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/
#include <X11/Xlib.h>
#include <X11/Xutil.h>

/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "windows.h"
#include "windowsx.h"
#include "Driver.h"
#include "Log.h"
#include "DrvDP.h"
#include "mtlock.h"

static DWORD DrvCursorSetCursor(LPARAM,LPARAM,LPVOID);
static DWORD DrvCursorGetCursorPos(LPARAM,LPARAM,LPVOID);
static DWORD DrvCursorSetCursorPos(LPARAM,LPARAM,LPVOID);
DWORD DrvCursorSetClipCursor(LPARAM,LPARAM,LPVOID);
static DWORD DrvCursorGetClipCursor(LPARAM,LPARAM,LPVOID);
static DWORD DrvCursorDoNothing(LPARAM,LPARAM,LPVOID);

DWORD DrvCursorTab(void);

static TWINDRVSUBPROC DrvCursorEntryTab[] = {
	DrvCursorDoNothing,
	DrvCursorDoNothing,
	DrvCursorDoNothing,
	DrvCursorSetCursor,
	DrvCursorGetCursorPos,
	DrvCursorSetCursorPos,
	DrvCursorSetClipCursor,
	DrvCursorGetClipCursor,
	DrvCursorDoNothing,
	DrvCursorDoNothing
};

DWORD
DrvCursorTab(void)
{
	return (DWORD)DrvCursorEntryTab;
}

static DWORD
DrvCursorSetCursor(LPARAM dwParam1, LPARAM dwParam2, LPVOID lpStruct)
{
	PRIVATEDISPLAY *dp = GETDP();
	static BOOL fDefinedRootCursor = FALSE;
	Window frame_win;

	LOCK_SECTION( _MTLOCK_VIDEO);
	if (fDefinedRootCursor) {
		XUndefineCursor(dp->display,DefaultRootWindow(dp->display));
		fDefinedRootCursor = FALSE;
	}
	if ((frame_win = (Window)dwParam2))
		XDefineCursor(dp->display,frame_win,(Cursor)dwParam1);
	else {
		XDefineCursor(dp->display,DefaultRootWindow(dp->display),
			(Cursor)dwParam1);
		fDefinedRootCursor = TRUE;
	}
	UNLOCK_SECTION( _MTLOCK_VIDEO);
	return 1L;
}

extern int	x11root_x;
extern int     x11root_y;
static DWORD
DrvCursorGetCursorPos(LPARAM dwParam1, LPARAM dwParam2, LPVOID lpStruct)
{
#if 0
        PRIVATEDISPLAY *dp = GETDP();
        Window root,child;
        int root_x,root_y,child_x,child_y;
        unsigned int key_mask;

	LOCK_SECTION( _MTLOCK_VIDEO);
	XQueryPointer(dp->display,
		DefaultRootWindow(dp->display),
		&root,&child,&root_x,&root_y,
		&child_x,&child_y,&key_mask);
	UNLOCK_SECTION( _MTLOCK_VIDEO);
#endif

	((LPPOINT)lpStruct)->x = x11root_x;
	((LPPOINT)lpStruct)->y = x11root_y;
	return 1L;
}

static DWORD
DrvCursorSetCursorPos(LPARAM dwParam1, LPARAM dwParam2, LPVOID lpStruct)
{
        PRIVATEDISPLAY *dp = GETDP();

	LOCK_SECTION( _MTLOCK_VIDEO);
	XWarpPointer(dp->display,None,
		DefaultRootWindow(dp->display),
		0,0,0,0,
		((LPPOINT)lpStruct)->x,((LPPOINT)lpStruct)->y);
	UNLOCK_SECTION( _MTLOCK_VIDEO);
	return 1L;
}

DWORD
DrvCursorSetClipCursor(LPARAM dwParam1, LPARAM dwParam2, LPVOID lpStruct)
{
        PRIVATEDISPLAY *dp = GETDP();
	LPRECT lprc = (LPRECT)lpStruct;

	if (lprc == NULL) {         /* freeing the cursor */
		dp->rcCursorClip.left = 0;
		dp->rcCursorClip.top = 0;
		LOCK_SECTION( _MTLOCK_VIDEO);
		dp->rcCursorClip.right = 
			XDisplayWidth(dp->display,dp->screen);
		dp->rcCursorClip.bottom = 
			XDisplayHeight(dp->display,dp->screen);
		UNLOCK_SECTION( _MTLOCK_VIDEO);
		dp->fCursorClipped = FALSE;
	}
        else {                      /* confine the cursor to given rect */
                dp->rcCursorClip.left = lprc->left;
                dp->rcCursorClip.top = lprc->top;
                dp->rcCursorClip.right = lprc->right;
                dp->rcCursorClip.bottom = lprc->bottom;
                dp->fCursorClipped = TRUE;
	}
        return 1L;
}

static DWORD
DrvCursorGetClipCursor(LPARAM dwParam1, LPARAM dwParam2, LPVOID lpStruct)
{
        PRIVATEDISPLAY *dp = GETDP();
        LPRECT lprc = (LPRECT)lpStruct;

	if (lprc) {
		lprc->left = dp->rcCursorClip.left;
		lprc->top = dp->rcCursorClip.top;
		lprc->right = dp->rcCursorClip.right;
		lprc->bottom = dp->rcCursorClip.bottom;
	}
	return 1L;
}

static DWORD
DrvCursorDoNothing(LPARAM dwParam1, LPARAM dwParam2, LPVOID lpStruct)
{
	return 1L;
}

/*------------------------- < Full Revision History > ----------------------
** $Log: DrvCursor.c,v $
** Revision 1.5  1999/11/30 17:45:15  robf
** code cleanup
**
** Revision 1.4  1999/11/30 03:48:07  robf
** use poll to retrieve cursor location
**
** Revision 1.3  1999/07/08 18:53:32  mwalsh
** Updated to Standard Header and Footer comments
**
**-------------------------------------------------------------------------*/
