/*--------------------------------------------------------------------------
**      DrvInit.c             Twin           From:  Twin/graphics/x11drv
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      Copyright 1997 Willows Software, Inc.
**------------------------ < File Content Description > --------------------
**
**  Module:	graphics/x11drv/DrvInit.c
**
**  Description:
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/graphics/x11drv/DrvInit.c,v 1.14 2000/01/28 07:55:13 robf Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/
#include <string.h>
#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/ 
#include "windows.h"
#include "Log.h"
#include "kerndef.h"
#include "ObjEngine.h"		/* DUMPHTAB */
#include "DrvDP.h"
#include "compat.h"
#include "WinConfig.h"
#include "Driver.h"
#include "DrvInit.h"
#include "DrvDC.h"
#include "thread.h"
#include "mtlock.h"

	/* enable new palette code */
#define TWIN_PALETTE

/* internal static data */
static PRIVATEDISPLAY DP[1], *dp;
static char *SysErr = "SystemError";
#define CONNECTSTR "No Display Server\nMust set DISPLAY environment variable\n"

/* internal routines */
static void ErrorHook(Display *);
static int TrapHook(Display *,XErrorEvent *);

/* this is exported to other driver subsystems, but NOT to the library */
PRIVATEDISPLAY * GETDP();
void TWIN_SystemError(UINT,DWORD,DWORD,DWORD);
TWINLIBCALLBACKPROC lpLibCallback;
int TWIN_SyncGraphics(BOOL );

int XrmUniqueQuark();

#define SYSERR_DISPLAY		1	/* error connecting to server */
#define SYSERR_SERVER		2	/* fatal error w/ server      */
#define SYSERR_XIO		3	/* fatal i/o error w/ server  */
#define SYSERR_MEMORY		4	/* fatal error memory allocation */
#define SYSERR_CONNECT		5	/* error connecting to server */
#define SYSERR_COLORS		6	/* error initializing colors */

/* wFunc has to be DSUBSYSTEM_INIT */
/* dwParam1 - 0 */
/* dwParam2 - init/exit flag */
/* lpStruct - points to the structure to fill */
DWORD
PrivateInitDriver(DWORD wFunc, LPARAM dwParam1, LPARAM dwParam2, LPVOID lpStruct)
{
    LPSTR lpszDisplayName;
    LPTWINDRVCONFIG lpDrvConfig = (LPTWINDRVCONFIG)lpStruct;

    /*------------------------------------------------------------------------
    **   JPW 04/15/99 - When we're using multithreaded systems, let's
    **                  initialize the Xlib thread control
    **----------------------------------------------------------------------*/
    #if defined(_REENTRANT)
    static int thread_initialized = 0;
    if (! thread_initialized)
    {
        thread_initialized++;
        XInitThreads();
    }
    #endif

    switch(LOWORD(wFunc)) {
    case DSUBSYSTEM_GETCAPS:
   	break; 

    case DSUBSYSTEM_INIT:

	if (dwParam2 != 0) {
		dp = &DP[0];

		lpLibCallback = lpDrvConfig->lpDrvCallback;

		lpszDisplayName = GetTwinString(WCP_DISPLAY,0,256);

    		LOCK_SECTION( _MTLOCK_VIDEO);
		if ((dp->display=XOpenDisplay(lpszDisplayName)) == NULL) {
		    ERRSTR((LF_ERROR,"PrivateInitDriver: connect failure on %s\n", 
				XDisplayName(lpszDisplayName)));
    		    UNLOCK_SECTION( _MTLOCK_VIDEO);
		    TWIN_SystemError(0,SYSERR_DISPLAY,0,
				(DWORD) XDisplayName(lpszDisplayName));
		}

		XInitThreads();
    		UNLOCK_SECTION( _MTLOCK_VIDEO);
		dp->bConnected = TRUE;


		if(GetConfigOption(WD_SYNCMODE)) {
			TWIN_SyncGraphics(1);
		}

    		LOCK_SECTION( _MTLOCK_VIDEO);
		dp->DisplayName = DisplayString(dp->display);
		dp->VendorName  = ServerVendor(dp->display);

		XSetIOErrorHandler((XIOErrorHandler)ErrorHook);
		XSetErrorHandler((XErrorHandler)TrapHook);
    		UNLOCK_SECTION( _MTLOCK_VIDEO);

		/* get the unique context value for the associated hwnd */
    		LOCK_SECTION( _MTLOCK_VIDEO);
		dp->client_hwnd  = XUniqueContext();
		dp->window_style  = XUniqueContext();
		dp->window_ex_style  = XUniqueContext();
    		UNLOCK_SECTION( _MTLOCK_VIDEO);
		
		dp->CodePage     = 437;

		DrvIPCInit(dwParam1, 1, 0);

		dp->screen = DefaultScreen(dp->display);

		DrvImagesInit(dwParam1, 1, 0);

		DrvCursorSetClipCursor(0,0,NULL);

		/* setup hardware equivalence */
		dp->HardwareEnabled = TRUE;
		dp->SwappedButtons  = FALSE;
		dp->LastClickTime   = 0;
		dp->DoubleClickTime = 250; /* set later during Registry Init */

		/* init system palette */
		dp->uSystemPaletteUse = SYSPAL_STATIC;
		dp->uSystemPaletteUseCount = ((LPUINT) 0);
		dp->hSystemPalette = (HPALETTE)NULL;
		dp->hOldSystemPalette = (HPALETTE)NULL;

		if(GetConfigOption(WD_COLORMAP)) {
    		    LOCK_SECTION( _MTLOCK_VIDEO);
		    dp->SystemPalette = XCreateColormap(dp->display,
			RootWindow(dp->display,dp->screen),
			DefaultVisual(dp->display,dp->screen),
			AllocNone);
    		    UNLOCK_SECTION( _MTLOCK_VIDEO);
		}
		else {
    		    LOCK_SECTION( _MTLOCK_VIDEO);
		    dp->SystemPalette = XDefaultColormap(dp->display,dp->screen);
    		    UNLOCK_SECTION( _MTLOCK_VIDEO);
		}

		if (!DrvColorsInit(dwParam1,1,0))
		    TWIN_SystemError(0,SYSERR_COLORS,0,0);

    		LOCK_SECTION( _MTLOCK_VIDEO);
		XSelectInput(dp->display,RootWindow(dp->display,dp->screen),
				PropertyChangeMask);
    		UNLOCK_SECTION( _MTLOCK_VIDEO);

		DrvKeyboardInit(dwParam1,1,0);

		DrvTextInit(dwParam1,1,0);

    		LOCK_SECTION( _MTLOCK_VIDEO);
		lpDrvConfig->nScreenWidth = DisplayWidth(dp->display,dp->screen);
		lpDrvConfig->nScreenHeight = DisplayHeight(dp->display,dp->screen);
    		UNLOCK_SECTION( _MTLOCK_VIDEO);
		return 1L;
	} else { 

		DrvColorsInit(0,0,0);

		/* display is a goner... */
		if (dp && dp->display && dp->bConnected) {
		    WriteProfileString(0,0,0);
    		    LOCK_SECTION( _MTLOCK_VIDEO);
		    XCloseDisplay(dp->display);
    		    UNLOCK_SECTION( _MTLOCK_VIDEO);
		    dp->bConnected = FALSE;
		}
		/* release text entries */
		DrvTextInit(0,0,0);
	}
    }
    return 1L;
}

void
TWIN_SystemError(UINT action,DWORD errorcode,DWORD wParam,DWORD lParam)
{
    int	exitcode = -1;
	
    if (dp && dp->display) {
    	LOCK_SECTION( _MTLOCK_VIDEO);
	XUngrabPointer(dp->display,0);
    	UNLOCK_SECTION( _MTLOCK_VIDEO);
    }

    switch (errorcode) {
	case SYSERR_CONNECT:
	    ERRSTR((LF_ERROR,CONNECTSTR));
	    exitcode = -errorcode;
	    break;

	case SYSERR_DISPLAY:
	    ERRSTR((LF_ERROR,"%s: Cannot connect to server %s.\n", SysErr,lParam));
	    exitcode = -errorcode;
	    break;

	case SYSERR_SERVER:
	    ERRSTR((LF_ERROR,"%s: Fatal X I/O Error %x.\n", SysErr,lParam));
	    exitcode = -errorcode;
	    break;

	case SYSERR_XIO:
	    ERRSTR((LF_ERROR,"%s: Fatal X I/O Error on %s.\n", SysErr,lParam));
	    exitcode = -errorcode;
	    break;

	case SYSERR_MEMORY:
	    ERRSTR((LF_ERROR,"%s: Memory Allocation Failure %x %x.\n",
		SysErr,wParam,lParam));
	    exitcode = -errorcode;
	    break;

	case SYSERR_COLORS:
	    ERRSTR((LF_ERROR,"Error initializing colors.\n"));
	    exitcode = -errorcode;
	    break;

	default:
	    ERRSTR((LF_ERROR,"%s: %x %x %x %x\n",
		SysErr,action,errorcode,wParam,lParam));
	    exitcode = -errorcode;
	    break;
    }
    ERRSTR((LF_CONSOLE,"Thread halted.\n"));
    for(;;);
}

static void
ErrorHook(Display *display)
{
    ERRSTR((LF_ERROR,"Fatal I/O Error on %s\n",XDisplayName(NULL)));

    TWIN_SystemError(0,SYSERR_XIO,0,(DWORD)XDisplayName(NULL));
}

static int fatalerr;
static int
TrapHook(Display *display, XErrorEvent *err)
{
    char  errbuf[512];
    char *errmsg;
    int   errflg;

    /* save the error in a globally accessable place */
    dp->ErrorCode = err->error_code;

    switch (err->error_code) {
	case Success:
	    errmsg = "Success";
	    errflg = 0;
	    break;

	case BadRequest:
	    errmsg = "BadRequest - bad request code";
	    errflg = 1;
	    break;

	case BadWindow:
	    errmsg = "BadWindow - parameter not a Window";
	    errflg = 1;
	    break;

	case BadValue:
	    errmsg = "BadValue - parameter out of range";
	    errflg = 0;
	    break;

	case BadAtom:
	    errmsg = "BadAtom - parameter not an Atom";
	    errflg = 0;
	    break;

	case BadPixmap:
	    errmsg = "BadPixmap";
	    errflg = 0;
	    break;

	case BadMatch:	
	    errmsg = "BadMatch";
	    errflg = 0;
	    break;

	case BadFont:	
	    errmsg = "BadFont";
	    errflg = 0;
	    break;

	case BadCursor:	
	    errmsg = "BadCursor";
	    errflg = 0;
	    break;

	case BadName:	
	    errmsg = "BadName - font or color name doesn't exist";
	    errflg = 0;
	    break;

	case BadAccess:	
	    errmsg = "BadAccess";
	    errflg = 0;
	    break;

	case BadDrawable:
	    errmsg = "BadDrawable";
	    errflg = 0;
	    break;

	case BadAlloc:
	    errmsg = "BadAlloc - insufficient resources ";
	    errflg = 0;
	    break;

	case BadColor:
	    errmsg = "BadColor - no such colormap ";
	    errflg = 0;
	    break;

	case BadGC:
	    errmsg = "BadGC - parameter not a GC";
	    errflg = 0;
	    break;

	default:
	    errmsg = "other";
            errflg = 1;
	    break;
    }

    sprintf(errbuf,"TrapHook: error = %d %s (%s)\n",
		err->error_code,errmsg,errflg?"FATAL":"warning");

    ERRSTR((LF_CONSOLE,"%s",errbuf));

    if (errflg) {
	if (fatalerr)
	    TWIN_SystemError(0,SYSERR_SERVER,0,(DWORD) err->error_code);
	fatalerr++;
	//FatalAppExit(0,"fatal X error");
	for(;;);
    }

    return 0;
}

PRIVATEDISPLAY *
GETDP()
{
    return dp;
}

int
TWIN_SyncGraphics(BOOL bSync)
{
    	PRIVATEDISPLAY *dp = GETDP();
	int	last;

	last = dp->bSyncronous;

    	LOCK_SECTION( _MTLOCK_VIDEO);
    	XSynchronize(dp->display,(bSync)?1:0);
    	UNLOCK_SECTION( _MTLOCK_VIDEO);

	dp->bSyncronous = bSync;

#ifdef DEBUG
	logstr(LF_CONSOLE,"%syncronous server mode.\n",
		bSync?"S":"As");
#endif

	return last;
}

Region
TSXCreateRegion()
{
    Region rgn;
    LOCK_SECTION( _MTLOCK_VIDEO);
    rgn = XCreateRegion(); 
    UNLOCK_SECTION( _MTLOCK_VIDEO);
    return rgn;
}
void
TSXDestroyRegion(Region rgn)
{
    LOCK_SECTION( _MTLOCK_VIDEO);
    XDestroyRegion(rgn); 
    UNLOCK_SECTION( _MTLOCK_VIDEO);
}

GC 
TSXCreateGC(Display *display, Drawable d, unsigned long valuemask, 
	XGCValues *values)
{
    GC gc;
    LOCK_SECTION( _MTLOCK_VIDEO);
    gc = XCreateGC(display,d,valuemask,values); 
    UNLOCK_SECTION( _MTLOCK_VIDEO);
    return gc;
}

void
TSXFreeGC(Display *display, GC gc)
{
    LOCK_SECTION( _MTLOCK_VIDEO);
    XFreeGC(display,gc);
    UNLOCK_SECTION( _MTLOCK_VIDEO);
}

void * TSXAllocDDC(int size)
{
	void * lpddc;
    
	LOCK_SECTION( _MTLOCK_VIDEO);
    	lpddc = (void *) WinMalloc(size);
    	UNLOCK_SECTION( _MTLOCK_VIDEO);
  
	return lpddc;
}

void   TSXFreeDDC(void * lpddc)
{
    LOCK_SECTION( _MTLOCK_VIDEO);
    WinFree(lpddc);
    UNLOCK_SECTION( _MTLOCK_VIDEO);
    //logstr(LF_MEM,"MEM: WinFree=%x %d\n",lpddc,--rsc_ddc);
}

/*------------------------- < Full Revision History > ----------------------
** $Log: DrvInit.c,v $
** Revision 1.14  2000/01/28 07:55:13  robf
** default to 250ms
**
** Revision 1.13  2000/01/26 19:55:25  jwhite
** This includes a wide range of changes required to get Twine to work
** with MFC on the Alpha platform.
**
** Revision 1.12  2000/01/05 22:02:56  robf
** logging
**
** Revision 1.11  1999/12/09 05:18:33  robf
** added new TWIN_SyncGraphics() declaration
**
** Revision 1.10  1999/11/30 03:49:02  robf
** modify sycnronous mode
**
** Revision 1.9  1999/07/08 18:53:32  mwalsh
** Updated to Standard Header and Footer comments
**
**-------------------------------------------------------------------------*/
