/*--------------------------------------------------------------------------
**      invoke_binary.c              Twin           From:  Twin/if1632
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      Copyright 1997 Willows Software, Inc.
**------------------------ < File Content Description > --------------------
**
**  Module:	if1632/invoke_binary.c
**
**  Description:
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/if1632/invoke_binary.c,v 1.2 1999/07/08 18:53:43 mwalsh Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "windows.h"

#undef CARRY_FLAG	/* well-coordinated work */
#undef ZERO_FLAG
#include "kerndef.h"
#include "BinTypes.h"
#include "DPMI.h"

#include "Log.h"

extern void TWIN_FreeObsoleteCatches(ENV *envp);
extern void invoke_code16(ENV *);

extern ENV *envp_global;	/* the current global pointer to the x86 environment */

extern WORD return_thunk_selector;

/***************************************************************
 *	invoke_binary()
 *	
 *	uses the current envp_global environment to call into the binary
 *	world from the native world.
 *
 ****************************************************************/

void invoke_binary()
{
	ENV new_env;
 	ENV *old_env;
	unsigned char *sp;

	/****************************************************
	Creates a new ENV structure, initializes it from the current
	ENV structure, and makes it be the current ENV structure.  The
	new ENV structure has a level field one greater than the previous,
	has a prev_env field that points to the previous, and has 0 in its
	is_catch and active fields.
	****************************************************/

	new_env = *envp_global;
 	new_env.prev_env = envp_global;
 	new_env.active = 0;
 	new_env.is_catch = 0;
	new_env.level = envp_global->level + 1;
 	envp_global = &new_env;

	/****************************************************
 	Sets up a return address by pushing a return-thunk on
 	the simulated stack.
	(the binary function will do a far-return to that 
	address, which will be intercepted by the interpreter)
  	****************************************************/

	sp = (unsigned char *)envp_global->reg.sp;
 	*(sp - 2) = return_thunk_selector & 0xff;
	*(sp - 1) = return_thunk_selector >>8;
 	*(sp - 3) = *(sp - 4) = 0;
 	envp_global->reg.sp = (REGISTER)(sp - 4);
	envp_global->reg.ax = (WORD) GetCurrentDataSelector();

	if(setjmp(envp_global->jump_buffer)) {
		/* we are doing a longjmp          */
		/* take the jump back to beginning */
		/* popping everything off stack    */
	} else { 

		if (GetSelectorType(HIWORD(envp_global->trans_addr)) == 
					TRANSFER_CODE16) 
		{
#if defined(TWIN_BIN16)
			invoke_code16(envp_global);
#else
			FatalAppExit(0,"win16 binaries not supported!\n");
#endif
	}

		if (envp_global->is_catch)
		    TWIN_FreeObsoleteCatches(envp_global);
	}

	old_env = envp_global;
 	envp_global = (ENV *)envp_global->prev_env;
	envp_global->reg.ax = old_env->reg.ax;
 	envp_global->reg.dx = old_env->reg.dx;
 	envp_global->reg.sp = old_env->reg.sp; 
}

/*------------------------- < Full Revision History > ----------------------
** $Log: invoke_binary.c,v $
** Revision 1.2  1999/07/08 18:53:43  mwalsh
** Updated to Standard Header and Footer comments
**
**-------------------------------------------------------------------------*/

