/*--------------------------------------------------------------------------
**      Log.h                Twin           From:  Twin/include
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      
**------------------------ < File Content Description > --------------------
**
**  Module:	include/Log.h
**
**  Description:
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/include/twine/Log.h,v 1.4 1999/10/25 18:10:11 jsheets Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/


#ifndef Log__h
#define Log__h	

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "windows.h"
#include "debug.h"
#include "MsgCode.h"

#ifdef __cplusplus
extern "C" {
#endif


int DEBUG_EnableLogging(int on);
void logstr(unsigned long flg, ...);

#ifdef NO_TRACE_MSGS
#define ERRSTR(x)	logstr x
#define APISTR(x)	
#define LOGSTR(x)
#define LOGMSG(h,m,w,l)
#define LOGMSGCALL(f,t,h,m,w,l)
#define LOGMSGRET(f,t,h,m,w,l)
#else
#define ERRSTR(x)	logstr x
#define APISTR(x)	logstr x
#define LOGSTR(x)	logstr x
#define LOGMSG(h,m,w,l) LogTwinMsgCall(LF_MSG,0,h,m,w,l,__FUNCTION__);
#define LOGMSGCALL(f,t,h,m,w,l) LogTwinMsgCall(f,t,h,m,w,l,__FUNCTION__);
#define LOGMSGRET(f,t,h,m,w,l) LogTwinMsgRet(f,t,h,m,w,l,__FUNCTION__);
#endif

#define LOGFLAG(cl,ch)	(dbch_##ch << 8 | dbcl_##cl)

/*
 *	CONSOLE		to stdout, as well as to log file
 *	TRACE		off by default
 *	WARN		unexpected operation
 *	FAIL		cannot recover
 *
 *	STUB		to do, not implemented
 *      FIXME		to do, partial implementation
 *	CALL
 *	RET
 */

#define LF_CONSOLE	LOGFLAG(CONSOLE,LIB)
#define LF_WARNING	LOGFLAG(WARN,LIB)
#define LF_ERROR	LOGFLAG(FAIL,LIB)
#define LF_FIXME	LOGFLAG(FIXME,LIB)

#define LF_APICALL	LOGFLAG(CALL,API)
#define LF_APIRET	LOGFLAG(RET,API)
#define LF_APIFAIL	LOGFLAG(FAIL,API)
#define LF_APISTUB	LOGFLAG(FIXME,API)

#define LF_LIBCALL	LOGFLAG(CALL,LIB)
#define LF_LIBRET	LOGFLAG(RET,LIB)
#define LF_LIBFAIL	LOGFLAG(FAIL,LIB)
#define LF_LIBSTUB	LOGFLAG(FIXME,LIB)

#define LF_BINCALL	LOGFLAG(CALL,BIN)
#define LF_BINRET	LOGFLAG(RET,BIN)

#define LF_INTCALL	LOGFLAG(CALL,INT)
#define LF_INTRET	LOGFLAG(RET,INT)
#define LF_INT86	LOGFLAG(TRACE,INT)

#define LF_API		LOGFLAG(TRACE,API)
#define LF_MSG		LOGFLAG(TRACE,MSG)
#define LF_LOGPE	LOGFLAG(TRACE,BIN)
#define LF_LOG		LOGFLAG(TRACE,LIB)
#define LF_MEM		LOGFLAG(TRACE,MEM)
#define LF_GDI		LOGFLAG(TRACE,GDI)
#define LF_USER		LOGFLAG(TRACE,USER)
#define LF_SYS		LOGFLAG(TRACE,KERNEL)

void logstr(unsigned long flg, ...);
void LogTwinMsgCall(DWORD,MSGCODE *,HWND ,int ,WPARAM ,LPARAM ,char *);
DWORD LogTwinMsgRet(DWORD,DWORD,HWND ,int ,WPARAM ,LPARAM ,char *);

#ifdef __cplusplus
}
#endif


#endif

/*------------------------- < Full Revision History > ----------------------
** $Log: Log.h,v $
** Revision 1.4  1999/10/25 18:10:11  jsheets
** Added DEBUG_EnableLogging() function to interactively turn logging on/off at runtime
**
** Revision 1.3  1999/09/28 18:52:57  jsheets
** Added logging macros for non-API library functions
**
** Revision 1.2  1999/07/31 02:40:39  jwhite
** Better protected for C++
**
** Revision 1.1  1999/07/15 17:56:55  jwhite
** Moved files that are not clearly a part of the Win32 API specification
** out of the main include directory into a 'twine' subdirectory.
** This is going to allow us to more readily integrate Wine include files.
**
** Revision 1.3  1999/07/12 21:13:43  mwalsh
** Updated comments to utilize standard header and footer
**
**-------------------------------------------------------------------------*/
