/*--------------------------------------------------------------------------
**      ModTable.h                 Twin           From:  Twin/include
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      Copyright 1997 Willows Software, Inc. 
**------------------------ < File Content Description > --------------------
**
**  Module:	include/ModTable.h
**
**  Description:
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/include/twine/ModTable.h,v 1.1 1999/07/15 17:56:55 jwhite Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/





#ifndef ModTable__h
#define ModTable__h

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "Bin32.h"

#define NO_VERSION
#define NO_LZEXPAND
#define NO_WINCOMM
#define NO_SHELL
#define NO_COMMDLG

typedef struct keyLIBTABLE
{
       void (*libentry)();
} LIBTABLE;
 

typedef struct keyMODULEDSCR
{   char *name;                 		/* module name 		*/
    struct keyENTRYTAB *entry_table;		/* entry data for module */
    struct keySEGTAB *seg_table;		/* segment data for module */
    long *resource;				/* resource table	*/
    MAPTABLE *map_table;
    LIBTABLE *lib_table;
} MODULEDSCR;

typedef MODULEDSCR *LPMODULEDSCR;

typedef struct keyMODULETAB
{   unsigned int flags;		/* flags for module */
    MODULEDSCR *dscr;		/* pointer to descriptor for module */
                                /* NULL => end of table */
} MODULETAB;

typedef MODULETAB *LPMODULETAB;

/* masks for flags field in struct hsmt_module_tab: */

#define MODULE_EXTERN      0x01    /* only module name, no descriptor */
#define MODULE_NO_LOAD     0x02    /* if module is dyn-dyn linked */
#define MODULE_SYSTEM      0x04    /* if module is system DLL	*/

typedef struct keyENTRYTAB
{   char *name;        /* entry point name */
                        /* will be NULL to indicate end of entry table */
                        /* will point to "" if only ordinal number known */
    unsigned short sel; /* selector number for entry.  will be 0 if no */
                        /* binary-world address exists for entry */
    unsigned short off; /* offset of entry point in selected segment */
    long int (*fun)();  /* pointer to C-callable native-world entry point */
                        /* will be NULL if there is none. May be pointer */
                        /* to native unit or interface to binary unit */
} ENTRYTAB;

typedef struct keySEGTAB
{	char *image;			/* pointer to image and image header */
	unsigned long size;		/* number bytes in segment image (0 => 64k) */
	unsigned short transfer;	/* transfer type of the segment	*/
					/* NULL indicates the end of the segment table */
	unsigned long alloc;	/* number of bytes to allocate for seg (0 => 64k) */
	unsigned short flags;	/* flags for the segment */
	unsigned short reserved;
} SEGTAB;

#endif	/* ModTable__h */

/*------------------------- < Full Revision History > ----------------------
** $Log: ModTable.h,v $
** Revision 1.1  1999/07/15 17:56:55  jwhite
** Moved files that are not clearly a part of the Win32 API specification
** out of the main include directory into a 'twine' subdirectory.
** This is going to allow us to more readily integrate Wine include files.
**
** Revision 1.2  1999/07/12 21:13:43  mwalsh
** Updated comments to utilize standard header and footer
**
**-------------------------------------------------------------------------*/
