/*--------------------------------------------------------------------------
**      obj_picture.h              Twine           From:  Wine/include/wine
**                                                  wine-990523
**
**              
**------------------------- < License Information > ------------------------
**
**      This file is a direct copy of the equivalent file in the
**  Wine project (www.winehq.com).  The Wine license permits
**  copying of source code, so long as all copyright and credit
**  information is preserved.  Any and all copyrights originally
**  in the Wine source code file are preserved exactly as they were
**  when this copy was made from the Wine source tree.
**  (see the 'Original Copyright(s)', below).
**  An exact copy of the Wine license is contained in WINE-LICENSE.
**  
**      For a more general description of the Wine license, and
**  for a full list of the authors who deserve credit for this
**  code, please see www.winehq.com, or WINE-AUTHORS.
**  
**      However, because this file is released with Twine, we
**  are obliged to also release it under the LGPL license.
**  In short, you may make a copy or republish this code so long as
**  all of your changes to source code are again made publicly available.
**  Further, absolutely no warranty of any kind is given or implied.
**  Refer to the LGPL description at www.fsf.org or the file LICENSE 
**  for further details.
**  
**  
**      ==========================================================
**      IF YOU MAKE CHANGES TO THIS CODE, PLEASE REFLECT
**      THOSE CHANGES IN THE ORIGINAL WINE CODE, AT WWW.WINEHQ.COM
**      ==========================================================
**
**      
**------------------------ < Original Copyright(s) > -----------------------
**
**  None in Wine source file; Wine license applies.
**
**------------------------ < File Content Description > --------------------
**
**  Module:	include/wine/obj_picture.h
**
**  Description: 
**
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/include/wine/obj_picture.h,v 1.1 1999/07/15 17:37:58 jwhite Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/
/*
 * Defines the COM interfaces and APIs related to OLE picture support.
 *
 * Depends on 'obj_base.h'.
 */

#ifndef __WINE_WINE_OBJ_PICTURE_H
#define __WINE_WINE_OBJ_PICTURE_H

#include "windows.h"
#include "windef.h"

/*****************************************************************************
 * Predeclare the structures
 */
typedef UINT OLE_HANDLE;
typedef LONG OLE_XPOS_HIMETRIC;
typedef LONG OLE_YPOS_HIMETRIC;
typedef LONG OLE_XSIZE_HIMETRIC;
typedef LONG OLE_YSIZE_HIMETRIC;



/*****************************************************************************
 * Predeclare the interfaces
 */
DEFINE_GUID(IID_IPicture, 0x7bf80980, 0xbf32, 0x101a, 0x8b, 0xbb, 0x00, 0xAA, 0x00, 0x30, 0x0C, 0xAB);
typedef struct IPicture IPicture, *LPPICTURE;

DEFINE_GUID(IID_IPictureDisp, 0x7bf80981, 0xbf32, 0x101a, 0x8b, 0xbb, 0x00, 0xAA, 0x00, 0x30, 0x0C, 0xAB);
typedef struct IPictureDisp IPictureDisp, *LPPICTUREDISP;

/*****************************************************************************
 * IPicture interface
 */
#define ICOM_INTERFACE IPicture
#define IPicture_METHODS \
  ICOM_METHOD1(HRESULT,get_Handle, OLE_HANDLE*,pHandle) \
  ICOM_METHOD1(HRESULT,get_hPal, OLE_HANDLE*,phPal) \
  ICOM_METHOD1(HRESULT,get_Type, SHORT*,pType) \
  ICOM_METHOD1(HRESULT,get_Width, OLE_XSIZE_HIMETRIC*,pWidth) \
  ICOM_METHOD1(HRESULT,get_Height, OLE_YSIZE_HIMETRIC*,pHeight) \
  ICOM_METHOD10(HRESULT,Render, HDC,hdc, LONG,x, LONG,y, LONG,cx, LONG,cy, OLE_XPOS_HIMETRIC,xSrc, OLE_YPOS_HIMETRIC,ySrc, OLE_XSIZE_HIMETRIC,cxSrc, OLE_YSIZE_HIMETRIC,cySrc, LPCRECT,pRcWBounds) \
  ICOM_METHOD1(HRESULT,set_hPal, OLE_HANDLE,hPal) \
  ICOM_METHOD1(HRESULT,get_CurDC, HDC*,phDC) \
  ICOM_METHOD3(HRESULT,SelectPicture, HDC,hDCIn, HDC*,phDCOut, OLE_HANDLE*,phBmpOut) \
  ICOM_METHOD1(HRESULT,get_KeepOriginalFormat, BOOL*,pKeep) \
  ICOM_METHOD1(HRESULT,put_KeepOriginalFormat, BOOL,Keep) \
  ICOM_METHOD (HRESULT,PictureChanged) \
  ICOM_METHOD3(HRESULT,SaveAsFile, LPSTREAM,pStream, BOOL,fSaveMemCopy, LONG*,pCbSize) \
  ICOM_METHOD1(HRESULT,get_Attributes, DWORD*,pDwAttr) 
#define IPicture_IMETHODS \
	IUnknown_IMETHODS \
	IPicture_METHODS
ICOM_DEFINE(IPicture,IUnknown)
#undef ICOM_INTERFACE

#ifdef ICOM_CINTERFACE
/*** IUnknown methods ***/
#define IPicture_QueryInterface(p,a,b)         ICOM_CALL2(QueryInterface,p,a,b)
#define IPicture_AddRef(p)                     ICOM_CALL (AddRef,p)
#define IPicture_Release(p)                    ICOM_CALL (Release,p)
/*** IPicture methods ***/
#define IPicture_get_Handle(p,a)               ICOM_CALL1(get_Handle,p,a)
#define IPicture_get_hPal(p,a)                 ICOM_CALL1(get_hPal,p,a)
#define IPicture_get_Type(p,a)                 ICOM_CALL1(get_Type,p,a)
#define IPicture_get_Width(p,a)                ICOM_CALL1(get_Width,p,a)
#define IPicture_get_Height(p,a)               ICOM_CALL1(get_Height,p,a)
#define IPicture_Render(p,a,b,c,d,e,f,g,h,i,j) ICOM_CALL10(Render,p,a,b,c,d,e,f,g,h,i,j)
#define IPicture_set_hPal(p,a)                 ICOM_CALL1(set_hPal,p,a)
#define IPicture_get_CurDC(p,a)                ICOM_CALL1(get_CurDC,p,a)
#define IPicture_SelectPicture(p,a,b,c)        ICOM_CALL3(SelectPicture,p,a,b,c)
#define IPicture_get_KeepOriginalFormat(p,a)   ICOM_CALL1(get_KeepOriginalFormat,p,a)
#define IPicture_put_KeepOriginalFormat(p,a)   ICOM_CALL1(put_KeepOriginalFormat,p,a)
#define IPicture_PictureChanged(p)             ICOM_CALL (PictureChanged,p)
#define IPicture_SaveAsFile(p,a,b,c)           ICOM_CALL3(SaveAsFile,p,a,b,c)
#define IPicture_get_Attributes(p,a)           ICOM_CALL1(get_Attributes,p,a)
#endif


/*****************************************************************************
 * IPictureDisp interface
 */
#define ICOM_INTERFACE IPictureDisp
#define IPictureDisp_METHODS 
#define IPictureDisp_IMETHODS \
				IDispatch_IMETHODS \
				IPictureDisp_METHODS
ICOM_DEFINE(IPictureDisp,IDispatch)
#undef ICOM_INTERFACE

#ifdef ICOM_CINTERFACE
/*** IUnknown methods ***/
#define IPictureDisp_QueryInterface(p,a,b)      ICOM_CALL2(QueryInterface,p,a,b)
#define IPictureDisp_AddRef(p)                  ICOM_CALL (AddRef,p)
#define IPictureDisp_Release(p)                 ICOM_CALL (Release,p)
/*** IDispatch methods ***/
#define IPictureDisp_GetTypeInfoCount(p,a)      ICOM_CALL1 (GetTypeInfoCount,p,a)
#define IPictureDisp_GetTypeInfo(p,a,b,c)       ICOM_CALL3 (GetTypeInfo,p,b,c)
#define IPictureDisp_GetIDsOfNames(p,a,b,c,d,e) ICOM_CALL5 (GetIDsOfNames,p,a,b,c,d,e)
#define IPictureDisp_Invoke(p,a,b,c,d,e,f,g,h)  ICOM_CALL8 (Invoke,p,a,b,c,d,e,f,g,h)
/*** IPictureDisp methods ***/
#endif



#endif /* __WINE_WINE_OBJ_PICTURE_H */


/*------------------------- < Full Revision History > ----------------------
** $Log: obj_picture.h,v $
** Revision 1.1  1999/07/15 17:37:58  jwhite
** Added files from Wine for COM/OLE support.
**
**
**-------------------------------------------------------------------------*/


