/*--------------------------------------------------------------------------
**      mfs_regexp.c            Twin           From:  Twin/msdos
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**    Copyright 1997 Willows Software, Inc.  
**------------------------ < File Content Description > --------------------
**
**  Module:	msdos/mfs_regexp.c
**
**  Description: mfs file comparison function using regexp style compare
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/msdos/mfs_regexp.c,v 1.2 1999/07/08 21:30:39 mwalsh Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>

/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "platform.h"
#include "windows.h"
#include "mfs_config.h"
#include "mfs_core.h"

static int charicmp(char, char);

/************************************************************************/
static int 
charicmp (
        char                    char1,
        char                    char2 )
 
{
 
        char                    Char1 = ( 'a' <= char1 ) && ( char1 <= 'z' ) ?
                                        char1 - 'a' + 'A' : char1;
        char                    Char2 = ( 'a' <= char2 ) && ( char2 <= 'z' ) ?
                                        char2 - 'a' + 'A' : char2;
 
        return ( Char2 - Char1 );
 
}

/************************************************************************/
DWORD
mfs_regexp(DWORD lpFileName, DWORD lpFilter, DWORD p3, DWORD p4)
{
	/*   The following code is provided by Tarang and I trust him...
	*/
        LPSTR                   lpTempFileName = (LPSTR)lpFileName;
        LPSTR                   lpTempFilter   = (LPSTR)lpFilter;
        char                    TempToken [ 2 ];
	char			TempFilter [ 2 ];
        BOOL                    Matched = FALSE;

        if ( ( ! (LPSTR)lpFileName ) || ( ! *(LPSTR)lpFileName ) ||
             ( ! (LPSTR)lpFilter ) || ( ! *(LPSTR)lpFilter ) )
                return 0L;

	if ( ! lstrcmp ( ( LPSTR )lpFilter, "*.*" ) )
	{
		wsprintf ( TempFilter, "*" );
		lpTempFilter = TempFilter;
		lpFilter     = ( DWORD )TempFilter;
	}

        while ( ( lpTempFilter ) && ( *lpTempFilter ) && ( ! Matched ) )
        {
                memset ( TempToken, 0, sizeof ( TempToken ) );
                switch ( *lpTempFilter )
                {
                        default:
                                if ( charicmp ( *lpTempFileName, *lpTempFilter ) )
                                {
                                        lpTempFileName = (LPSTR)lpFileName;
                                        lpTempFilter = strpbrk ( lpTempFilter, " ,;" );
                                        if ( lpTempFilter )
                                                lpTempFilter++;
                                }
                                else
                                {
                                        lpTempFilter++;
                                        lpTempFileName++;
                                        switch ( *lpTempFilter )
                                        {
                                                default:
                                                        break;

                                                case '\0':
                                                case ' ':
                                                case ',':
                                                case ';':
                                                        if ( ! *lpTempFileName )
                                                                Matched = TRUE;
                                                        break;
                                        }
                                }
                                break;

                        case '?':
                                lpTempFilter++;
                                lpTempFileName++;
                                break;

                        case '*':
                                if ( ! ( TempToken [ 0 ] = *( ++lpTempFilter ) ) )
                                        Matched = TRUE;
                                else
                                {
                                        lpTempFilter++;
                                        while ( ( lpTempFileName = strpbrk ( lpTempFileName, TempToken ) ) &&
                                                ( ! Matched ) )
                                                Matched = mfs_regexp ( (DWORD)++lpTempFileName, (DWORD)lpTempFilter, 0L, 0L );
                                        if ( ( ! lpTempFileName ) && ( ! Matched ) )
                                        {
                                                lpTempFileName = (LPSTR)lpFileName;
                                                lpTempFilter = strpbrk ( lpTempFilter, " ,;" );
                                                if ( lpTempFilter )
                                                        lpTempFilter++;
                                        }
                                }
                                break;

                        case '\0':
                        case ' ':
                        case ',':
                        case ';':
                                Matched = TRUE;
                                break;
                }
        }

        return (DWORD)Matched;

#ifdef OLDSTUFF
    char	*s;       /* p1 */
    char        *pattern; /* p2 */
    int 	 last;
    int 	 matched;
    int 	 reverse;
    char         pp[128], *p;

    s = (char *)p1;
    pattern = (char *)p2;

    strcpy(pp, pattern);
    p = (char *) strtok(pp, " ,;");
    while ( p ) {
    	for ( ; *p; s++, p++)
	    switch (*p) {
	    case '\\':
		/* Literal match with following character; fall through. */
		p++;
	    default:
		if ((toupper(*s) != *p) && (tolower(*s) != *p))
		    return(FALSE);
		continue;

	    case '?':
		/* Match anything. */
		if (*s == '\0')
		    return(FALSE);
		continue;

	    case '*':
		/* Trailing star matches everything. */
		return(*++p ? Star(s, p) : TRUE);

	    case '[':
		/* [^....] means inverse character class. */
		if ((reverse = p[1]) == '^')
		    p++;
		for (last = 0400, matched = FALSE; *++p && *p != ']'; last = *p)
		    /* This next line requires a good C compiler. */
		    if (*p == '-' ? *s <= *++p && *s >= last : *s == *p)
			matched = TRUE;
		if (matched == reverse)
		    return(FALSE);
		continue;
	    }
	if ( *s == '\0'  ||  *s == '/' )
		return TRUE;
	p = (char *) strtok(NULL, " ,;");
    }
    return FALSE;
#endif
}

/*------------------------- < Full Revision History > ----------------------
** $Log: mfs_regexp.c,v $
** Revision 1.2  1999/07/08 21:30:39  mwalsh
** Updated to new standard comment header and footer
**
**-------------------------------------------------------------------------*/

