/*--------------------------------------------------------------------------
**      system.c                   Twine           
**
**              
**------------------------- < License Information > ------------------------
**
**  
**      This file is released under terms of an LGPL.  Readers of 
**  this file may modify or use this source code so long as they make all
**  modifications publically available.  Please refer to TWINE-LICENSE 
**  for further details.
**  
**
**      The original Copyright is in 1999, by CodeWeavers, Inc.
**
**      
**------------------------ < File Content Description > --------------------
**
**  Module:   	osdll/system.c      
**
**  Description:
**     		provides system dependent functions to load/unload
**		shared libraries 
**
**  Functions defined:
**		osdll_library_open(char *lpszLibFileName)
**		osdll_library_close(DWORD hSO)
**		osdll_memory_sync(LPVOID lpData,DWORD dwLength)
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/osdll/system.c,v 1.3 1999/07/13 20:17:17 mwalsh Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/



/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "windows.h"
#include "Log.h"

/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/

#if defined(HAVE_DLFCN_H)
#include <dlfcn.h>
#else
#include "twindlfcn.h"
#endif

/*------------------------- < Full Revision History > ----------------------
** $Log: system.c,v $
** Revision 1.3  1999/07/13 20:17:17  mwalsh
** Updated comments to use standard header and footer.
**
** Revision 1.2  1999/05/07 08:13:18  robf
** remove RTLD_GLOBAL
**
** Revision 1.1.1.1  1999/03/17 20:48:18  jgraham
** First Release
**
**-------------------------------------------------------------------------*/


DWORD 
osdll_library_open(char *lpszLibFileName)
{
    char      LibraryPath[256];
    char      LibraryFile[256];
    char      LibraryFullName[256];
    char      EntryName[128];
    void     *hSO;
    char      *s;

    /*
     *  Split the file into path and filename components.  The
     *  path might be null at this point.
     */
    s = strrchr (lpszLibFileName, '/');
    if (s == NULL)
    {
	/*
	 *  No path info, entire string is the filename.
	 */
	strcpy(LibraryPath, "");
	strcpy(LibraryFile, lpszLibFileName);
    }
    else
    {
	/*
	 *  Copy the info in as the path, and set the null terminator
	 *  immediately following the found '/'.  Set the filename
	 *  portion to be the part after the '/'.
	 */
	strcpy(LibraryPath, lpszLibFileName);
	LibraryPath[(s - (char *)lpszLibFileName) + 1] = 0;
	strcpy(LibraryFile, s+1);
    }

    /*
     *  Look for an extension.  We only munge the filename if the
     *  extension is .dll, or there is no extension.  Other names
     *  are left untouched.
     */
    s = strrchr (LibraryFile, '.');
    if (s)
    {
	if (strcmp(s, ".dll") == 0)
	{
	    /*
	     *  A .dll extension has been found, so remove it.  Set
	     *  the pointer to NULL to indicate that there is no longer
	     *  an extension.
	     */
	    *s = 0;
	    s = NULL;
	}
    }

    /*
     *  If there is still an extension on the file, then we do not
     *  munge the name --- revert to the original for the full name.
     *  Otherwise, we prepend "lib" to the filename, append "32" for
     *  the 32-bit version of the library, and append the system-specific
     *  shared-library extension.
     */
    if (s)
    {
	strcpy(LibraryFullName, lpszLibFileName);
	strcpy(EntryName, LibraryFile);
    }
    else
    {
	strcpy(LibraryFullName, LibraryPath);
	strcat(LibraryFullName, "lib");
	strcat(LibraryFullName, LibraryFile);
	strcat(LibraryFullName, ".so");
	
	for(s=LibraryFullName;*s;s++)
		*s = tolower(*s);

        strcpy(EntryName, "TWIN_LibEntry_lib");
	strcat(EntryName, LibraryFile);
    }

    hSO = dlopen(LibraryFullName,RTLD_LAZY);

    if (hSO == 0)
    {
	return (0);
    }

#ifdef TWIN_DRVINIT
    {
	    FARPROC   fn;
	    fn = (FARPROC) dlsym(hSO,EntryName);    

	    if(fn)
	      (*fn)();
    }
#endif

    /* LoadLibrary should continue to initialize  */
    /* and not try to load a filename.dll         */
    return (DWORD) hSO;
}

DWORD 
osdll_library_close(DWORD hSO)
{
	return (DWORD) dlclose((void *) hSO);
}

DWORD
osdll_memory_sync(LPVOID lpData,DWORD dwLength)
{
#ifdef TWIN_RISC_CPU
    	SYNC_CODE_DATA;
#endif
	return 0;
}

/*------------------------- < Full Revision History > ----------------------
** $Log: system.c,v $
** Revision 1.3  1999/07/13 20:17:17  mwalsh
** Updated comments to use standard header and footer.
**
**-------------------------------------------------------------------------*/
