/*--------------------------------------------------------------------------
**      prndc.c                 Twin           From:  Twin/programs/prnsetup
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      Hunter Systems - Apr 1994
**------------------------ < File Content Description > --------------------
**
**  Module:	programs/prnsetup/prndc.c
**
**  Description:
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/programs/prnsetup/prndc.c,v 1.2 1999/07/13 20:17:19 mwalsh Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "windows.h"





/*============================================================================
	prndc.c

	GetDevDrvPort sets 3 pointers (device name, driver name, port name
	from win.ini).
	GetPrinterDC creates default printer DC, using the settings in win.ini.
	The DC can be initialized with data pointed by lpData parameter.
  ============================================================================*/
int
GetDevDrvPort(char **pDev, char **pDrv, char **pPort)
{
	static char	*p1 = 0, *p2 = 0, *p3 = 0;
	static char	szPrn[128] = { '\0' };

	if ( !szPrn[0] )
		if ( !GetProfileString ("windows","device",",,,",szPrn,sizeof(szPrn)) )
			return FALSE;

	if ( !p3 ) {
		p1 = (char*)strtok(szPrn, ",");
		p2 = (char*)strtok(NULL,  ", ");
		p3 = (char*)strtok(NULL,  ", ");
	}
	if ( !p1  ||  !p2  ||  !p3  ||  !*p1  ||  !*p2  ||  !*p3 )
		return FALSE;

	if ( pDev )
		*pDev = p1;
	if ( pDrv )
		*pDrv = p2;
	if ( pPort )
		*pPort = p3;
	return TRUE;
}


HDC
GetPrinterDC(LPVOID lpData)
{
	char	*pDev, *pDrv, *pPort;

	if ( !GetDevDrvPort(&pDrv, &pDev, &pPort) )
		return (HDC)0;

	return CreateDC(pDev, pDrv, pPort, lpData);
}

/*------------------------- < Full Revision History > ----------------------
** $Log: prndc.c,v $
** Revision 1.2  1999/07/13 20:17:19  mwalsh
** Updated comments to use standard header and footer.
**
**-------------------------------------------------------------------------*/