/*--------------------------------------------------------------------------
**      setup.c                   Twine           
**              
**------------------------- < License Information > ------------------------
**
**  
**      This file is released under terms of an LGPL.  Readers of 
**  this file may modify or use this source code so long as they make all
**  modifications publically available.  Please refer to TWINE-LICENSE 
**  for further details.
**  
**
**      The original Copyright is in 1999, by CodeWeavers, Inc.
**
**      
**------------------------ < File Content Description > --------------------
**
**  Module:	programs/twinerc/setup.c
**	
**  Description:
**		
**
**  Functions defined:
**
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/programs/twinerc/setup.c,v 1.2 1999/07/13 20:17:24 mwalsh Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "stdio.h"
#include "windows.h"
#include "setup.h"
#include "compat.h"

extern int __argc;
extern char **__argv;

typedef struct tagOPTIONS {
	char *arg;
	int   flag;
	int   invert;
	char *opt;
} OPTIONS;

char * GetTwinFilename(void);

OPTIONS  options[] = {
	{ "verbose",0,0 ,"verbose output"},
	{ "dither",WD_NODITHERPURE,0, "dither no-pure colors" },
	{ "colormap",WD_COLORMAP,0,"allocate private x11 colormap" },
	{ "hiresfonts",WD_HIRESFONTS,0, "use hi-res fonts" },
	{ "vgamode",WD_VGAMODE,0,"configure vga display mode" },
 
 
	{ "debugger",WD_DEBUGGER,1,"enable debugger"},
	{ "fatal",WD_FATALERR,0,"break into debugger on fatal errors"},
	{ "errors",WD_HANDLERR,0, "break on HANDLE errors" },
	{ "stubs",WD_APISTUBS,0,"show messagebox on apistubs"},

	{ "paging",WD_NOPAGING,0, "demand pages win16 applications" },
	{ "changecase",WD_NOCHANGECASE,1, "fold filenames to lowercase"},
	{ "mapdirectory",WD_MAPCURDIR,0 ,"map current directory as dos drive"},
	{ 0,0,0,0 }
};

BOOL PASCAL 
WinMain(HANDLE hInst, HANDLE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	int i;
	char *p;
	int n;
	int control,flag,invert;
	char string[80];
	int verbose = 0;

	for(i=0;i<__argc;i++) {
		p = __argv[i];
		invert = 0;
		flag   = 0;
		
		if(*p == '-') {
			p++;
			if(*p == '-')
			 p++;
		}
		if(*p == 'n' && *(p+1) == 'o' ) {
			invert = 1;
			p += 2;
		}
		for(n=0;options[n].arg;n++) {
			if(strncasecmp(p,options[n].arg,strlen(p)) == 0) {
				flag = options[n].flag;
				if(flag == 0)
					verbose++;
				if(options[n].invert)
					invert = !invert;
				if(invert)
					control &= ~flag;
				else
					control |= flag;
				break;
			}
		}
	}
			
	sprintf(string,"0x%4.4x",control);
	WritePrivateProfileString("boot","control",string,
		GetTwinFilename());

	for(n=0;options[n].arg;n++) {
		if(options[n].flag == 0)
			continue;
		if(options[n].flag & control) {
			printf("0x%4.4x\t%s%s\n",
				options[n].flag,
				options[n].invert?"do not ":"",
				options[n].opt);
		} else {
			if(verbose)
			        printf("0x%4.4x\t%s%s\n",
					options[n].flag,
					options[n].invert?"":"do not ",
					options[n].opt);

		}
	}
	return 0;
}

#if 0
	{ "dither",WD_NODITHERPURE,0, "dither no-pure colors" },
	{ "colormap",WD_COLORMAP,0,"allocate private x11 colormap" },
	{ "hiresfonts",WD_HIRESFONTS,0, "use hi-res fonts" },
	{ "vgamode",WD_VGAMODE,0,"configure vga display mode" },
	{ "debugger",WD_DEBUGGER,1,"enable debugger"},
	{ "fatal",WD_FATALERR,0,"break into debugger on fatal errors"},
	{ "errors",WD_HANDLERR,0, "break on HANDLE errors" },
	{ "stubs",WD_APISTUBS,0,"show messagebox on apistubs"},
	{ "paging",WD_NOPAGING,0, "demand pages win16 applications" },
	{ "changecase",WD_NOCHANGECASE,1, "fold filenames to lowercase"},
	{ "mapdirectory",WD_MAPCURDIR,0 ,"map current directory as dos drive"},

verbose
	"#noditherpure=1  how to handle dithering "
	"#nothreads=1	don't allow threads		"
	"#fatalerr=1	break into debugger 		"
	"#handlerr=1	break on handleobj errors 	"
	"#noapistubs=1	NO msgbox for STUB 		"
	"#nopaging=1 	disable demand page loading  	"
	"#nochangecase=1  don't change case of filenames 	"
	"#mapcurdir=1   	xdos map current directory   	"
	"#colormask=1  	use rgb colormask to convert mono"
#endif

/*------------------------- < Full Revision History > ----------------------
** $Log: setup.c,v $
** Revision 1.2  1999/07/13 20:17:24  mwalsh
** Updated comments to use standard header and footer.
**
**-------------------------------------------------------------------------*/