/*--------------------------------------------------------------------------
**      liblist.c                 Twin           From:  Twin/programs/twinview/tvlib
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      
**------------------------ < File Content Description > --------------------
**
**  Module:	programs/twinview/tvlib/liblist.c
**
**  Description:Library List Functions
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/programs/twinview/tvlib/liblist.c,v 1.2 1999/07/13 20:17:26 mwalsh Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/
#include <windows.h>

/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "tvlibprv.h"





/*************************************

*

*  Global Variables

*

**************************************/

HTVLIBLIST  hTVLibList = 0;





/*************************************

*

*  Create a library list.

*

**************************************/

BOOL __far __pascal __export CreateTVLibList

( 

  char __far * szLibWildName,        /* File specification for a library file. */

  BOOL Yield                         /* Yeild while looking for library files. */

)

{

  FPTVLIBLIST fpTVLibList;          /* Pointer to the allocated library list. */



  if( ! GlobalAllocMem( NULL, &hTVLibList, sizeof(TVLIBLIST) ) ) return FALSE;

  

  /* Lock it. */

  fpTVLibList = (FPTVLIBLIST) GlobalLock( hTVLibList ) ;



  /* Initialize structure. */

  fpTVLibList->nLibCount   = 0;

  fpTVLibList->hTVLibInfo = 0;

  

  /* Unlock it. */

  GlobalUnlock( hTVLibList ) ;



  /* Find all library file names and pass them to the callback function InsertLibInfo(). */

  if( !EnumTVLibFiles( NULL, szLibWildName, InsertLibInfo, NULL, Yield ) ) return FALSE;



  /* Return it. */

  return TRUE;

}









/*************************************

*

*  Destroy a library list.

*

**************************************/

void __far __pascal __export DestroyTVLibList( void )

{

  FPTVLIBLIST fpTVLibList;         /* Pointer to the library list. */

  FPTVLIBINFO fpTVLibInfo;         /* Pointer to the top of the lib. info list. */

  int Counter;                     /* Incrementer. */

 



  /* Error - Bad list handle. */

  if( hTVLibList == 0 ) return;



  /* Lock the TVLIBLIST handle. */

  fpTVLibList = (FPTVLIBLIST) GlobalLock( hTVLibList );



  /* Error - Bad list handle. */

  if( fpTVLibList == NULL ) return;



  /* No lib info in the list. */

  if( fpTVLibList->nLibCount == 0 ) 

  {

    GlobalUnlock( hTVLibList );

    return;

  }

  

  /* Lock the lib info list. */ 

  fpTVLibInfo = (FPTVLIBINFO) GlobalLock( fpTVLibList->hTVLibInfo );



  /* For each TVLIBINFO structure in the list. */

  for( Counter = 0; Counter < fpTVLibList->nLibCount; Counter++ )

  {

    /* Delete data inside of the TVLIBINFO. */

    DeleteLibInfoData( fpTVLibInfo );

    

    /* Move to the next TVLIBINFO in the list. */

    fpTVLibInfo = fpTVLibInfo + 1;

  }



  /* Free the TVLIBINFO handle. */

  GlobalUnlock( fpTVLibList->hTVLibInfo );

  GlobalFree( fpTVLibList->hTVLibInfo );



  /* Free the TVLIBLIST handle. */

  GlobalUnlock( hTVLibList );

  GlobalFree( hTVLibList );

}


/*------------------------- < Full Revision History > ----------------------
** $Log: liblist.c,v $
** Revision 1.2  1999/07/13 20:17:26  mwalsh
** Updated comments to use standard header and footer.
**
**-------------------------------------------------------------------------*/






  

