/*--------------------------------------------------------------------------
**      filebuff.h                 Twin           From:  Twin/programs/twinview/twinhelp
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      Copyright 1997 Willows Software, Inc.
**------------------------ < File Content Description > --------------------
**
**  Module:	programs/twinview/twinhelp/filebuff.h
**
**  Description:
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/programs/twinview/twinhelp/filebuff.h,v 1.2 1999/07/13 16:39:43 mwalsh Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/


#ifndef FILEBUFF_H

#define FILEBUFF_H


/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/
#include <windows.h>
#include <stdlib.h>

/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "hlpfile.h"





/***********************************

**

**  Shared Defines

**

***********************************/



/* Record in the help file data buffer. */

typedef struct tagHLPFILEBUFFREC

{

  BOOL     bValid;           /* Is data in record valid. */



  DWORD    dwTickCount;     /* When file was opened. Used to removed the oldest

                               help file from the help file buffer. */

                               

  /* Key data for record - complete path to the help file. */

  char     szHelpFile[ _MAX_PATH ]; 



  HGLOBAL  hHelpFileInfo;   /* Handle to a HLPFILEINFO structure. */



  HGLOBAL  hTopicBuffer;    /* Handle to a buffer of TOPICBUFFREC structures. */

  

}

HLPFILEBUFFREC;

typedef HLPFILEBUFFREC __far * FPHLPFILEBUFFREC;





/* 

** Record in hTopicBuffer buffer. 

**

** Holds processed data for one 

** topic in a help file. 

*/

typedef struct tagTOPICBUFFREC

{

  BOOL          bValid;           /* Is data in record valid. */



  DWORD         dwUsage;          /* How many windows are using this topic. */

  

  DWORD         dwTickCount;      /* When topic was last used. Used to removed the oldest

                                     topic from the buffer. */



  /* dwTopicCharOffset is key data unique for each record. */

  DWORD         dwTopicCharOffset; /* Character offset for the topic's data in the

                                      |TOPIC file. */



  HGLOBAL       hScrollElements;      /* Display element data for the non-scrollable part of 

                                         topic. */

  HGLOBAL       hNonScrollElements;   /* Display element data for the non-scrollable part of 

                                         topic. */

  

  long          NextTopicCharOffset; /* Character offset for the next browse topic in the

                                        |TOPIC file. */

  

  long          PrevTopicCharOffset; /* Character offset for the prev. browse topic in the

                                        |TOPIC file. */

}

TOPICBUFFREC;

typedef TOPICBUFFREC __far * FPTOPICBUFFREC;





/* 

** All available topic information.

*/

typedef struct tagTOPICDATA

{

  /* 

  ** The dwBlock, wOffset, and the hHelpFileInfo->szFileName 

  ** can be used to identify which file and topic's data is stored

  ** in this structure.

  */

  DWORD     dwTopicCharOffset;    /* Character offset for the topic's data in the

                                     |TOPIC file. */



  char      szHelpFile[ _MAX_PATH ];  /* Name of help file. */



  HGLOBAL   hHelpFileInfo;       /* Handle to a HLPFILEINFO structure.     */

  HGLOBAL   hNonScrollElements;  /* Nonscrollable elements for the topic.  */ 

  HGLOBAL   hScrollElements;     /* Scrollable elements for the topic.     */ 

            

  long int  NextTopicCharOffset; /* Character offset for the next browse topic in the

                                    |TOPIC file. */

  

  long int  PrevTopicCharOffset; /* Character offset for the prev. browse topic in the

                                    |TOPIC file. */

}

TOPICDATA;

typedef TOPICDATA __far * FPTOPICDATA;

typedef HGLOBAL HTOPICDATA;               





/**********************************

**

**  Shared Function Prototypes

**

***********************************/



/***********************************

**

**  Shared Function Prototypes

**

***********************************/



HGLOBAL __far _pascal OpenHelpFile( HWND hMainWnd, char __far * szHelpFilePath );



BOOL __far __pascal AllocHelpFileBuffer( HWND hMainWnd );

void __far __pascal FreeHelpFileBuffer( HWND hMainWnd  );



HGLOBAL __far _pascal LockTopicData( HWND hMainWnd, char __far * szHelpFilePath, DWORD dwTopicCharOffset );

void __far _pascal UnlockTopicData( HWND hMainWnd, HGLOBAL hTopicData );



#endif



/*------------------------- < Full Revision History > ----------------------
** $Log: filebuff.h,v $
** Revision 1.2  1999/07/13 16:39:43  mwalsh
** Updated comments to standard header and footer
**
**-------------------------------------------------------------------------*/



