/*--------------------------------------------------------------------------
**      macroeng.h                 Twin           From:  Twin/programs/twinview/twinhelp
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      Copyright 1997 Willows Software, Inc.
**------------------------ < File Content Description > --------------------
**
**  Module:	programs/twinview/twinhelp/macroeng.h
**
**  Description:
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/programs/twinview/twinhelp/macroeng.h,v 1.2 1999/07/13 16:39:44 mwalsh Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/


#ifndef MACROENG_H

#define MACROENG_H


/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/
#include <windows.h>

/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/





/***********************************

**

**  Shared Defines

**

***********************************/



/* Hnadle to an instance of the macro engine. */

typedef HGLOBAL HMACROENGINE;



/* Return value of a macro. */

typedef long int MACRORETVAL;

 

/* Macro implementation callback function. */

typedef BOOL ( CALLBACK * MACROFUNCPTR )( HMACROENGINE hMacroEngine, HGLOBAL hAppData, HGLOBAL hMacroData, MACRORETVAL __far * MacroRetValPtr );



/* 

** A macro table entry contains information

** for one macro that is in a macro table.

*/

typedef struct tagMACROTABLEENTRY

{

  char __far * MacroNamePtr;

  MACROFUNCPTR MacroFuncPtr;

  HGLOBAL      hMacroData;

} 

MACROTABLEENTRY;

typedef MACROTABLEENTRY __far * MACROTABLEENTRYPTR;



typedef enum 

{

  MACRO_NOCONVERT,            /* Argument is a string without quotes - return all. */

  MACRO_STRING,               /* Argument is a quoted string - quotes will be removed. */

  MACRO_UNSIGNEDSHORTINT,     /* UINT, WORD, WPARAM */

  MACRO_UNSIGNEDLONGINT,      /* DWORD */

  MACRO_SIGNEDSHORTINT,       /* BOOL */

  MACRO_SIGNEDLONGINT,        /* LONG, LPARAM, LRESULT */

  MACRO_BOOL                  /* Value of 1 or 0. */

}

MACROARGTYPE;





/* 

** Maximum length of a macro string 

** including the terminating NULL character.

*/

#define MAX_MACRO_STRING    513



/* 

** Maximum length of a macro table's name 

** including the terminating NULL character. 

*/

#define MAX_TABLE_NAME      30



/* This table name is reserved and cannot be used. */

#define RESERVED_TABLE_NAME  "RESERVED_TABLE_NAME"



/* 

** This table name can be registered and loaded and

** contains default system macros. 

*/

#define SYSTEM_TABLE_NAME  "SYSTEM_TABLE_NAME"





/***********************************

**

**  Shared Function Prototypes

**

***********************************/



HMACROENGINE __far __pascal __export StartMacroEngine( HWND hErrorWnd, HGLOBAL hAppData );

void __far __pascal __export StopMacroEngine( HMACROENGINE __far * hMacroEnginePtr );

HGLOBAL __far __pascal __export GetMacroEngineAppData( HMACROENGINE hMacroEngine );



BOOL __far __pascal __export RegisterMacroTable

( 

  HWND hErrorWnd,

  HMACROENGINE hMacroEngine,

  char __far * szTableName, 

  WORD wNumMacros,

  MACROTABLEENTRYPTR MacroTableEntryPtr

);



BOOL __far __pascal __export UnregisterMacroTable

( 

  HMACROENGINE hMacroEngine,

  char __far * szTableName

);





BOOL __far __pascal __export LoadMacroTable

( 

  HWND hErrorWnd,

  HMACROENGINE hMacroEngine,

  char __far * szTableName 

);



BOOL __far __pascal __export UnloadMacroTable

( 

  HMACROENGINE hMacroEngine,

  char __far * szTableName 

);



BOOL __far __pascal __export ExecuteMacro

( 

  HWND hErrorWnd,

  HMACROENGINE hMacroEngine,

  char __far * MacroStringPtr,

  MACRORETVAL __far * MacroRetValPtr

);



BOOL __far __pascal __export GetNextMacroArg 

(

  HWND hErrorWnd,                /* Parent window for error message box. */

  HMACROENGINE hMacroEngine,     /* Handle to engine. */

  void __far * DataPtr,          /* Place in which to save data. */

  MACROARGTYPE MacroArgType      /* Type of data that it is. */

);





#endif




/*------------------------- < Full Revision History > ----------------------
** $Log: macroeng.h,v $
** Revision 1.2  1999/07/13 16:39:44  mwalsh
** Updated comments to standard header and footer
**
**-------------------------------------------------------------------------*/




