/*--------------------------------------------------------------------------
**      winmem.h                 Twin           From:  Twin/programs/twinview/twinhelp
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      Copyright 1997 Willows Software, Inc.
**------------------------ < File Content Description > --------------------
**
**  Module:	programs/twinview/twinhelp/winmem.h
**
**  Description:Memory Allocation Functions
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/programs/twinview/twinhelp/winmem.h,v 1.2 1999/07/13 16:39:45 mwalsh Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/




#ifndef WINMEM_H

#define WINMEM_H


/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/
#include <windows.h>

/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/






/***********************************

**

**  Shared Defines

**

***********************************/



/* Element list and list info. */

typedef struct tagMEMORYPAGE

{

  HGLOBAL hGlobal;             /* Handle to page's memory space. */

  void __far * GlobalPtr;      /* Locked page's memory space. */

  UINT    Segment;             /* SELECTOROF( GlobalPtr ). */

}

MEMORYPAGE;

typedef MEMORYPAGE __far * FPMEMORYPAGE;



/* Memory list and list info. */

typedef struct tagMEMORYLIST

{

  WORD wNumPages;              /* Number of page's in list (used and  not used). */

  WORD wUsedPages;             /* Number of page's in list that are being used. */

  HGLOBAL hPageList;           /* Handle to the list of pages. */

  FPMEMORYPAGE TopPagePtr;     /* Pointer to the first page in the list. */

}

MEMORYLIST;

typedef MEMORYLIST __far * FPMEMORYLIST;





/*************************************

*

*  Allocate global memory of a certain size and trap error.

*

**************************************/

BOOL __far __pascal GlobalAllocMem( HWND hWnd, HGLOBAL __far * hGlobal, DWORD dwSize  );





/*************************************

*

*  Reallocate global memory of a certain size and trap error.

*

**************************************/

BOOL __far __pascal GlobalReAllocMem( HWND hWnd, HGLOBAL __far * hGlobal, DWORD dwNewSize  );





/*****************************************

**

**  Allocates a buffer and copies string's

**  data into the buffer.

**

*****************************************/

HGLOBAL __far __pascal CopyString( HWND hErrorWnd, char __far * fpString );







/****************************** Local Heap Functions ******************************/



/***************************************************

*

* Creates a MEMORYLIST structure that contains a 

* list of memory pages. 

*

****************************************************/

BOOL AllocMemoryList( HGLOBAL __far * hMemoryList );





/***************************************************

*

* Free a MEMORYLIST structure that contains a 

* list of memory pages. 

*

****************************************************/

void FreeMemoryList( HGLOBAL __far * hMemoryList ); 





/***************************************************

*

* Allocates memory inside of a page of memory. 

*

****************************************************/

LPVOID MyAlloc( FPMEMORYLIST fpMemoryList, WORD wSize );





#endif

/*------------------------- < Full Revision History > ----------------------
** $Log: winmem.h,v $
** Revision 1.2  1999/07/13 16:39:45  mwalsh
** Updated comments to standard header and footer
**
**-------------------------------------------------------------------------*/