/*--------------------------------------------------------------------------
**      Threads.c                 Twin           From:  Twin/scheduler
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      Copyright 1997 Willows Software, Inc.
**------------------------ < File Content Description > --------------------
**
**  Module:	scheduler/Threads.c
**
**  Description:
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/scheduler/Threads.c,v 1.10 1999/11/30 04:13:11 robf Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "windows.h"
#include "windowsx.h"
#include "Log.h"
#include "kerndef.h"
#include "Resources.h"
#include "Module.h"
#include "Kernel.h"
#include "Log.h"
#include "osdll.h"
#include "Segment.h"
#include "DPMI.h"
#include "mfs_config.h"
#include "Log.h"

static HANDLE AllocProcessHandle(PDB *,K32OBJ * );
static BOOL   DeleteProcessHandle(PDB *,HANDLE );
//static HANDLE FindProcessHandle(PDB *,K32OBJ_TYPE ,K32OBJ *);
static void   InternalExitThread(THDB *);

/* (WIN32) Threads ********************************************************* */

BOOL WINAPI EnumThreadWindows(DWORD dwThreadId, WNDENUMPROC lpfn, 
			      LPARAM lParam)
{
    return EnumTaskWindows((HTASK) dwThreadId, lpfn, lParam);
}

DWORD WINAPI GetCurrentProcessId(void)
{
    return (DWORD) GetCurrentTask();
}


BOOL WINAPI PostThreadMessage(DWORD idThread, UINT Msg, 
			      WPARAM wParam, LPARAM lParam)
{
    return FALSE;
}

LPQUEUE
AllocQueue(int cMsg)
{
     LPQUEUE    lpQueue;
     LPQUEUEMSG lpQueueMsg;
     int nCnt;

    nCnt = sizeof(QUEUE);
    nCnt += sizeof(QUEUEMSG) * cMsg;

    if ((lpQueue = (LPQUEUE) WinMalloc(nCnt))) {

	    lpQueue->header.type     = K32OBJ_QUEUE;
	    lpQueue->header.refcount = 1;
	    /* initialize queue flags and pointers */
	    lpQueue->wQueueFlags = 0;
	    lpQueue->lpQueueFree = (LPQUEUEMSG)(lpQueue+1);
	    lpQueue->lpQueueNext = 0;

	    /* initialize next pointers in the free list*/
	    for (nCnt = 0,lpQueueMsg = lpQueue->lpQueueFree;
			nCnt < (cMsg-1); 
			nCnt++, lpQueueMsg++)

		lpQueueMsg->lpNextMsg = lpQueueMsg + 1;

	    /* set the last one to NULL */
	    lpQueueMsg->lpNextMsg = 0;
    }
    
    return lpQueue;
}
BOOL  WINAPI
SetMessageQueue(int cMsg)
{
    BOOL       bResult;
    THDB      *tdb;
    HANDLE     hQueue;

    tdb = osdll_thread_gettdb();

    if((hQueue = tdb->teb.queue)) {
	DeleteProcessHandle(tdb->process, hQueue);
    }

    hQueue = AllocProcessHandle(tdb->process,(K32OBJ *) AllocQueue(cMsg));
    bResult = (BOOL) ((tdb->teb.queue = hQueue) != 0);
    return bResult;
}

/*************************************************************/

int
GetThreadPriority(HANDLE hThread)
{
	APISTR((LF_API, "GetThreadPriority()\n"));
	return 0;
}

BOOL
SetThreadPriority(HANDLE hThread,int priority)
{
	APISTR((LF_API, "SetThreadPriority(HANDLE=%x,int=%d)\n",
		hThread,priority));
	return TRUE;
}

HANDLE 
GetCurrentThread()
{
	APISTR((LF_APISTUB, "GetCurrentThread()\n"));
	return ((HANDLE) CURRENT_THREAD_PSEUDOHANDLE);
}

DWORD GetProcessVersion(DWORD ProcessId)
{
	APISTR((LF_APISTUB, "GetProcessVersion()\n"));
	return 0;
}

HANDLE WINAPI
GetCurrentProcess(void)
{
	APISTR((LF_API, "GetCurrentProcess()\n"));
	return ((HANDLE)0x7fffffff);
}

LCID WINAPI
GetThreadLocale(void)
{
	APISTR((LF_APICALL, "GetThreadLocale()\n"));
	APISTR((LF_APIRET, "GetThreadLocale returns 0x409\n"));
	return 0x409;
}

static void *
GetProcessHandle(PDB *pdb,HANDLE handle)
{
	int index = (int) handle;
	if(index < 0) {
		// later handle PDB, THDB for current process
		return 0;
	}
	if(pdb == (PDB *) 0)
		return 0;
	if(pdb->handletable == 0)
		return 0;
	if(index == 0)
		return pdb;

	if(index > pdb->handletable[0])
		return 0;

	return (void *) pdb->handletable[index];
}

#if 0
static HANDLE
FindProcessHandle(PDB *pdb,K32OBJ_TYPE k32obj_type,K32OBJ *p)
{
	int i;
	if(p->type != k32obj_type)
		return 0;

	for(i=1;i<pdb->handletable[0];i++) {
		if(pdb->handletable[i] == (HANDLE) p ) {
			return i;
		}
	}
	return 0;
}
#endif

BOOL DuplicateHandle(
    HANDLE hSourceProcessHandle,// handle to process with handle to duplicate 
    HANDLE hSourceHandle,	// handle to duplicate 
    HANDLE hTargetProcessHandle,// handle to process to duplicate to 
    LPHANDLE lpTargetHandle,	// pointer to duplicate handle 
    DWORD dwDesiredAccess,	// access for duplicate handle 
    BOOL bInheritHandle,	// handle inheritance flag
    DWORD dwOptions 		// optional actions 
   )
{
	APISTR((LF_APISTUB,"DuplicateHandle(%d,%d,%d,%p,%d,%d,%d)\n",
	    hSourceProcessHandle,
	    hSourceHandle,	
	    hTargetProcessHandle,
	    lpTargetHandle,	
	    dwDesiredAccess,	
	    bInheritHandle,	
	    dwOptions));
		
	return TRUE;
}

 
static BOOL
DeleteProcessHandle(PDB *pdb,HANDLE handle)
{
	K32OBJ *p;
	if(pdb->handletable == 0)
		return FALSE;

	if(handle <= 0 || handle > pdb->handletable[0])
		return FALSE;

	if((p = (K32OBJ *)pdb->handletable[handle])) {
		/* TODO validate that p is of type K32OBJ */
		p->refcount--;

		/* TODO do type specific delete */
		if(p->refcount == 0) {
			WinFree(p);
		}

		pdb->handletable[handle] = 0;
	}	
	return FALSE;
}

static HANDLE
AllocProcessHandle(PDB *pdb,K32OBJ * handle)
{
	int i;

	if(pdb->handletable == 0) {
		pdb->handletable = (HANDLE *)WinMalloc(sizeof(HANDLE)*32);
		memset(pdb->handletable,0,sizeof(HANDLE)*32);
		pdb->handletable[0] = 32;

	}
	for(i=1;i<pdb->handletable[0];i++) {
		if(pdb->handletable[i] == 0) {
			break;
		}
	}
	if(i == pdb->handletable[0] ) {
		int n;
		pdb->handletable[0] += 32;
		pdb->handletable = WinRealloc(pdb->handletable,
			sizeof(HANDLE)*pdb->handletable[0]);
		for(n=i;n<pdb->handletable[0];n++)
			pdb->handletable[n] = 0;
	}

	pdb->handletable[i] = (HANDLE) handle;
	return (HANDLE) i;
}

BOOL
CloseHandle(HANDLE hObject)
{
    THDB *tdb = osdll_thread_gettdb();
    PDB *pdb = tdb?tdb->process:0;
    K32OBJ *k32obj;
    BOOL    bClosed = FALSE;

   
   LOGSTR((LF_APICALL,"CloseHandle(%x)\n",hObject));
   if(hObject == CURRENT_THREAD_PSEUDOHANDLE) {
        LOGSTR((LF_APIRET,"CloseHandle returns TRUE (1)\n"));
	return TRUE;
   }
 
   if(pdb == 0) {
        LOGSTR((LF_APIRET,"CloseHandle returns FALSE (2)\n"));
	return FALSE;
   }

   k32obj = (K32OBJ *) GetProcessHandle(pdb,hObject);
   if(k32obj)  {
   	LOGSTR((LF_API,"CloseHandle K32OBJ=%p %d\n",k32obj,k32obj->type));
   	switch(k32obj->type) {
	case K32OBJ_PROCESS:
		bClosed = TRUE;
		break;
	case K32OBJ_THREAD:
		bClosed = TRUE;
		break;
	default:
	
   	}
   }
   LOGSTR((LF_APIRET,"CloseHandle returns %d (3)\n",bClosed));
   return bClosed;
}

static void
InternalStartup(THDB *tdb)
{
   	LPTHREAD_START_ROUTINE lpfn;
	int rc;

	APISTR((LF_LOG, "internalstartup...%p\n",tdb));

 	lpfn = (LPTHREAD_START_ROUTINE)tdb->entry_point;

    	//MODULE_InitializeDLLs( thdb->process, 0, DLL_THREAD_ATTACH, NULL );

        //osdll_thread_init(tdb);

	// TBD wrap this with setjmp/longjmp
	// if CANCELLED, lets us cleanup... 
	rc = (lpfn)(tdb->entry_arg);
	tdb->dwExit = rc;

	// free up resources other than THDB
	InternalExitThread(tdb);
}

static HANDLE SystemHeap;

HANDLE WINAPI HeapCreate( DWORD ,DWORD, DWORD);

/***********************************************************************
 *           GetProcessHeap    (KERNEL32.259)
 */
HANDLE WINAPI GetProcessHeap(void)
{
    THDB *tdb = osdll_thread_gettdb();
    PDB *pdb = tdb?tdb->process:0;
    HANDLE heap = SystemHeap;

    /* If we have a process */
    if(pdb) {
	
	/* if we never allocated a heap yet */
	if(pdb->heap == 0 && pdb->heapsize) {

		/* allocate one now */	
    		pdb->heap = HeapCreate(0,pdb->heapsize,0);
	}

	/* does this process have a heap? */
	if(pdb->heap)
		heap = pdb->heap;
    }

    /* either private or system heap... */
    return heap;
}


/*
 * this is both thread and process
 */
THDB *
TWIN_AllocTHDB(HTASK hTask,DWORD stack_size,
	LPTHREAD_START_ROUTINE start_addr,LPVOID cmdline)
{
    THDB *initthread;
    HGLOBAL hProcess,hThread;
    PDB *pdb = 0;

    initthread = osdll_thread_gettdb();
    if(initthread)
	pdb = initthread->process;

    /* allocate a block for this one */
    hThread = GlobalAlloc(GHND,sizeof(THDB));
    initthread = (THDB *) GlobalLock(hThread);

    memset(initthread,0,sizeof(THDB));
    initthread->header.type     = K32OBJ_THREAD;
    initthread->header.refcount = 1;

    if(pdb == 0) {
	    hProcess = GlobalAlloc(GHND,sizeof(PDB));
	    pdb = (PDB *) GlobalLock(hProcess);
	    memset(pdb,0,sizeof(PDB));

	    pdb->hTask = hTask;
	    pdb->header.type     = K32OBJ_PROCESS;
	    pdb->header.refcount = 1; 

	    pdb->heapsize = 64 * 1024;

	    InitializeCriticalSection(&pdb->crit_section);

	    AllocProcessHandle(pdb,(K32OBJ *) pdb);
	    AllocProcessHandle(pdb,(K32OBJ *) initthread);
    }

    /* this thread points to our process, points to this module */
    initthread->process = pdb;
   
    initthread->teb_sel = AssignSelector((LPBYTE) &initthread->teb, 0, 
			TRANSFER_DATA32, sizeof(THDB));
    initthread->teb.self    = &initthread->teb;
    initthread->teb.process = pdb;
    initthread->teb.except  = (void *) -1;

    /* put something there for now... */
    initthread->teb.stack_top = &hTask;
    initthread->teb.stack_low = &hTask - 0x100000;

    initthread->teb.queue   = AllocProcessHandle(pdb, (K32OBJ *)AllocQueue(32));
    initthread->teb.tls_ptr = initthread->tls_array;
    initthread->entry_point = start_addr;
    initthread->entry_arg   = cmdline;

    return initthread;
}


HANDLE CreateThread(
  LPSECURITY_ATTRIBUTES  lpattr,  // pointer to security attributes
  DWORD                  dwstack, // initial thread stack size
  LPTHREAD_START_ROUTINE start,   // pointer to thread function
  LPVOID	         arg,     // argument for new thread
  DWORD			 cflags,  // creation flags
  LPDWORD 		 lptid    // pointer to receive thread ID
)
{
	HANDLE hThread = 0;
	HTASK  hTask;
	THDB  *tdb;
	TLS   *tls;

	APISTR((LF_APICALL, "CreateThread(%p,stack=%x,arg=%p,flags=%x,lptid=%p)\n",
		lpattr,dwstack,start,arg,cflags,lptid));

	// allocate THDB
	hTask = GetCurrentTask();
	tdb = TWIN_AllocTHDB(hTask,dwstack,start,arg);

	/* this THDB needs a stack */
	if (dwstack < 64*1024)
	   dwstack = 64*1024;

	tdb->stack_base = WinMalloc(dwstack);
	tdb->teb.stack_top = (char *)tdb->stack_base + dwstack;
	tdb->teb.stack_low = tdb->stack_base;
	tdb->exit_stack    = tdb->teb.stack_top;

	// wrappers for new thread
	tls = (TLS *) tdb->stack_base;
	tls->init = InternalStartup; 
	tls->tdb  = tdb;	

	/* launch a new thread, and start our startup code */
	if(osdll_thread_create(tdb) == 0) {
		WinFree(tdb->stack_base);
		WinFree(tdb);
		APISTR((LF_APIRET, "CreateThread returns 0\n"));
		return 0;
	}

	hThread = AllocProcessHandle(tdb->process,(K32OBJ *)tdb);
	if(lptid)
		*lptid = THDB_TO_THREAD_ID(tdb); 

	APISTR((LF_APIRET, "CreateThread returns %x\n",hThread));
	return hThread;
}

DWORD
ResumeThread(HANDLE handle)
{
	DWORD suspend_count = 0;
	THDB *tdb = (THDB *) osdll_thread_gettdb();

	if(tdb) {
		PDB *pdb;
		pdb = tdb->process;
		tdb = (THDB *) GetProcessHandle(pdb,handle);
	
	}		
		
	if(tdb) {
		suspend_count = (DWORD)tdb->suspend_count;

		if(tdb->suspend_count > 0) {
			tdb->suspend_count--;

			if(tdb->suspend_count == 0) {
				osdll_thread_ops(tdb,OSDLL_THREAD_RESUME);
			}
		}
	
	}
	return suspend_count;
}

DWORD
SuspendThread(HANDLE handle)
{
	DWORD suspend_count = 0;
	THDB *tdb = osdll_thread_gettdb();

	if(tdb) {
		PDB *pdb;
		pdb = tdb->process;
		tdb = (THDB *) GetProcessHandle(pdb,handle);
	}		
		
	if(tdb) {
		suspend_count = (DWORD)tdb->suspend_count;

		tdb->suspend_count++;

		if(tdb->suspend_count == 1) {
			osdll_thread_ops(tdb,OSDLL_THREAD_SUSPEND);
		}
	}	
	return suspend_count;
}

static void
InternalExitThread(THDB *tdb)
{
	if(tdb) {
		/* close the message queue */
    		CloseHandle(tdb->teb.queue);
		tdb->teb.queue = 0;

		/* release the selector */
    		//FreeSelector(tdb->teb_sel);

		/* release the stack */
		WinFree(tdb->stack_base);
	}	
	exit(0);
}

void
ExitThread(DWORD dwExit)
{
	THDB *tdb = osdll_thread_gettdb();

	// set the exit code of the handle...
	tdb->dwExit = dwExit;

	InternalExitThread(tdb);
}

void
TerminateThread(HANDLE handle,DWORD dwExit)
{
	THDB *tdb = osdll_thread_gettdb();
	PDB *pdb  = 0;

	// our own tdb to get our pdb
	if(tdb) {
		pdb = tdb->process;
		tdb = (THDB *) GetProcessHandle(pdb,handle);
	}		
		
	// the tdb of handle, in our process
	if(pdb && tdb) {

		// set the exit code of the handle...
		tdb->dwExit = dwExit;

		// now kill the thread
		osdll_thread_ops(tdb,OSDLL_THREAD_CANCEL);
	}	
	return;
}

DWORD WINAPI
GetCurrentThreadId(void)
{
	THDB *tdb = osdll_thread_gettdb();
	//TWIN_ASSERT(tdb->unix_pid == getpid());

	return THDB_TO_THREAD_ID(tdb); 
}

/*------------------------- < Full Revision History > ----------------------
** $Log: Threads.c,v $
** Revision 1.10  1999/11/30 04:13:11  robf
** api call/ret and initialized stack
**
** Revision 1.8  1999/07/13 20:17:37  mwalsh
** Updated comments to use standard header and footer.
**
**-------------------------------------------------------------------------*/
