#!/bin/sh
#--------------------------------------------------------------------------
#      update.sh                   Twine           
#              
#------------------------- < License Information > ------------------------
#
#  
#      This file is released under terms of an LGPL.  Readers of 
#  this file may modify or use this source code so long as they make all
#  modifications publically available.  Please refer to TWINE-LICENSE 
#  for further details.
#  
#
#      The original Copyright is in 1999, by CodeWeavers, Inc.
#
#      
#------------------------ < File Content Description > --------------------
#
#  Module:	tools/update.sh
#	
#  Description:
#     This script will find any patches in the current directory
#  and apply them.  See the usage() function for further information.
#
#  Notes:
#      1.  This script relies on Twine patch files containing
#          a description at their top.
#		
#    
#------------------------- < Revision Information > -----------------------
# $Header: /twinecvs/src/tools/update.sh,v 1.2 1999/03/30 20:08:04 jwhite Exp $
#
#--------------------------------------------------------------------------



#-----------< Function >---------------------------------------------------
# usage
#
#     This function describes the usage of update.sh
#
#--------------------------------------------------------------------------
usage()
{
    echo "update.sh directory [patch-arguments]"
    echo "    Apply a Twine patch to update an older version of Twine to a new one."
    echo ""
    echo "This script will iterate through all twine*.patch files it finds in the"
    echo "current directory, and then optionally apply each one to the directory"
    echo "specified on the command line."
    echo ""
    echo "You can specify additional arguments to the patch command after the directory"
    echo ""
    echo "Examples:"
    echo "    update.sh . --dry-run"
    echo "applies patches in the current directory, but only does a dry run (makes no changes)"
    echo ""
    echo "    update.sh twine-1.1"
    echo "applies patches in the current directory."
}



#-------------------------------------------------------------------------
# mainline
#
#     The script starts here.  Start by validating parameters.
#
#--------------------------------------------------------------------------
directory=$1
if [ x$1 = x ] ; then
    usage
    exit
fi

if [ ! -d $directory ]; then
    echo "Directory $directory not found"
    usage
    exit
fi

shift



#-------------------------------------------------------------------------
# Step 2:  Identify patches
#--------------------------------------------------------------------------
for pfile in twine*.patch; do
    echo $pfile:
    cat $pfile | awk -- '{ if (match($0, "^END")) exit; if (printing) print $0; if (match($0, "^START")) printing = 1;   }'
    echo "Apply this patch?  Enter 'yes' to proceed"
    read answer
    if [ $answer != "yes" ] ; then
        continue;
    fi
    echo Executing...  patch -d $directory -p 1 $* \<$pfile
    patch -d $directory -p 1 $* <$pfile
done



#------------------------- < Full Revision History > ----------------------
# $Log: update.sh,v $
# Revision 1.2  1999/03/30 20:08:04  jwhite
# Bug fix for non Linux systems.
#
# Revision 1.1  1999/03/29 20:23:37  jwhite
# Added update.sh, a utility to insert patches.
#
#
#
#--------------------------------------------------------------------------



