/*--------------------------------------------------------------------------
**      Hook.c                 Twin           From:  Twin/windows
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      Copyright 1997 Willows Software, Inc.
**------------------------ < File Content Description > --------------------
**
**  Module:	windows/Hook.c
**
**  Description:
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/windows/Hook.c,v 1.4 1999/10/25 16:53:23 jsheets Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "windows.h"
#include "Log.h"
#include "Hook.h"
#include "kerndef.h"

HOOKSTRUCT *lpHookList[12];

#ifdef STRICT
HHOOK WINAPI
#else
HOOKPROC WINAPI
#endif
SetWindowsHook(int idHook, HOOKPROC hkprc)
{
    HOOKPROC rc;

    APISTR((LF_APICALL,"SetWindowsHook(id:%x,proc=%x)\n",idHook,hkprc));
    rc = (HOOKPROC)SetWindowsHookEx(idHook,hkprc,(HINSTANCE)0,
			GetCurrentTask());
    APISTR((LF_APIRET,"SetWindowsHook: returns HOOKPROC %x\n", rc));

    return rc;
}

HHOOK WINAPI
SetWindowsHookEx(int idHook, HOOKPROC hkprc, HINSTANCE hInstance,
		HTASK hTask)
{
    LPHOOKSTRUCT lpHookStruct;

    APISTR((LF_APICALL,
	"SetWindowsHookEx: id %x proc %x inst %x task %x\n",
	idHook, hkprc, hInstance, hTask));

    if (!hkprc)
    {
        APISTR((LF_APIRET,"SetWindowsHookEx: returns NULL HHOOK\n"));
	return (HHOOK)0;
    }

    lpHookStruct = (LPHOOKSTRUCT)WinMalloc(sizeof(HOOKSTRUCT));
    lpHookStruct->lpfnHookProc = hkprc;
    lpHookStruct->idHook = idHook;
    lpHookStruct->hInst = hInstance;
    lpHookStruct->hTask = hTask;
    lpHookStruct->lpHookNext = lpHookList[idHook+1];
    lpHookList[idHook+1] = lpHookStruct;

    APISTR((LF_APIRET,"SetWindowsHookEx: returns HHOOK %x\n",
	    (HHOOK)lpHookStruct));
    return (HHOOK)lpHookStruct;
}

BOOL
UnhookWindowsHook(int idHook, HOOKPROC hkprc)
{
    LPHOOKSTRUCT lpHookStruct;
    BOOL rc;

    APISTR((LF_APICALL,"UnhookWindowsHook(idHook:%x,proc:%x)\n",
				idHook, hkprc));
    for (lpHookStruct = lpHookList[idHook+1]; lpHookStruct;
		lpHookStruct = lpHookStruct->lpHookNext)
	if (lpHookStruct->lpfnHookProc == hkprc)
	    break;
    if (lpHookStruct == NULL)
    {
        APISTR((LF_APIRET,"UnhookWindowsHook: returns BOOL FALSE\n"));
	return FALSE;
    }

    rc = UnhookWindowsHookEx((HHOOK)lpHookStruct);
    APISTR((LF_APIRET,"UnhookWindowsHook: returns BOOL %d\n", rc));
    return rc;
}

BOOL WINAPI
UnhookWindowsHookEx(HHOOK hHook)
{
    LPHOOKSTRUCT lpHookStruct;
    int idHook;

    APISTR((LF_APICALL,"UnhookWindowsHookEx: hHook %x\n",hHook));

    idHook = ((LPHOOKSTRUCT)hHook)->idHook;
    for(lpHookStruct = lpHookList[idHook+1];lpHookStruct; 
                lpHookStruct = lpHookStruct->lpHookNext)
    {
	if (lpHookStruct->lpHookNext == (LPHOOKSTRUCT)hHook)
	    break;
    }

    if (lpHookStruct == NULL)
    {
	if (lpHookList[idHook+1] == (LPHOOKSTRUCT)hHook) 
	    lpHookList[idHook+1] = NULL;
	else
	{
	    APISTR((LF_APIRET,"UnhookWindowsHookEx: returns BOOL TRUE\n"));
	    return FALSE;
	}
    }
    else 
	lpHookStruct->lpHookNext = ((LPHOOKSTRUCT)hHook)->lpHookNext;
    WinFree((LPSTR)hHook);

    APISTR((LF_APIRET,"UnhookWindowsHookEx: returns BOOL TRUE\n"));
    return TRUE;
}

LRESULT WINAPI
#ifdef STRICT
DefHookProc(int nCode, WPARAM wParam, LPARAM lParam, HHOOK *lphhook)
#else
DefHookProc(int nCode, WPARAM wParam, LPARAM lParam, HOOKPROC *lphhook)
#endif
{
    LRESULT rc;

    APISTR((LF_APICALL,"DefHookProc: hHook %x\n",*lphhook));
    rc = (DWORD)CallNextHookEx((HHOOK)(*(HHOOK *)lphhook),
			nCode,wParam,lParam);
    APISTR((LF_APIRET,"DefHookProc: returns LRESULT %ld\n", rc));

    return rc;
}

LRESULT WINAPI
CallNextHookEx(HHOOK hHook, int nCode, WPARAM wParam, LPARAM lParam)
{
    LPHOOKSTRUCT lpHookStruct;
    LRESULT      rc;

    APISTR((LF_APICALL,"CallNextHookEx: hHook %x\n",hHook));

    lpHookStruct = (LPHOOKSTRUCT)hHook;
    if (!lpHookStruct)
	return (LRESULT)NULL;

    while (lpHookStruct->lpHookNext) {
	if (lpHookStruct->lpHookNext->hTask == GetCurrentTask()) {
	    rc = lpHookStruct->lpHookNext->lpfnHookProc(nCode,wParam,lParam); 
    	    APISTR((LF_APIRET,"CallNextHookEx: returns LRESULT %x\n",rc));
	    return rc;
	}
	lpHookStruct = lpHookStruct->lpHookNext;
    }
    APISTR((LF_APIRET,"CallNextHookEx: returns LRESULT %x\n",NULL));
    return (LRESULT)NULL;
}

BOOL WINAPI
CallMsgFilter(LPMSG lpmsg, int nCode)
{
    LPHOOKSTRUCT lpMsgFilter,lpSysMsgFilter;
    LRESULT rc = 0;
    
    APISTR((LF_APICALL,"CallMsgFilter(hwnd=%x,msg=%x,code:%x)\n",
	lpmsg->hwnd,lpmsg->message,nCode));

    if ((lpSysMsgFilter = lpHookList[WH_SYSMSGFILTER+1]))
    {
	if ((rc = lpSysMsgFilter->lpfnHookProc(nCode,0,(LPARAM)lpmsg)))
	{
	    APISTR((LF_APIRET,"CallMsgFilter: returns BOOL TRUE\n"));
	    return TRUE;
	}
    }
    if ((lpMsgFilter = lpHookList[WH_MSGFILTER+1]))
    {
	if (lpMsgFilter->hTask == GetWindowTask(lpmsg->hwnd))
	    rc = lpMsgFilter->lpfnHookProc(nCode,0,(LPARAM)lpmsg);
	APISTR((LF_APIRET,"CallMsgFilter: returns BOOL %s\n",
		(rc) ? "TRUE" : "FALSE"));
	return (rc)?TRUE:FALSE;
    }
    APISTR((LF_APIRET,"CallMsgFilter: returns BOOL TRUE\n"));
    return FALSE;
}

/*------------------------- < Full Revision History > ----------------------
** $Log: Hook.c,v $
** Revision 1.4  1999/10/25 16:53:23  jsheets
** Cleaned up lots of missing CALL/RET logging pairs
**
** Revision 1.3  1999/09/28 19:28:58  jsheets
** Fixed logging string
**
** Revision 1.2  1999/07/09 21:55:41  mwalsh
** Updated comments to standard header and footer
**
**-------------------------------------------------------------------------*/
