/*--------------------------------------------------------------------------
**      Kernel.h                 Twin           From:  Twin/windows
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      Copyright 1997 Willows Software, Inc.
**------------------------ < File Content Description > --------------------
**
**  Module:	windows/Kernel.h
**
**  Description:
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/windows/Kernel.h,v 1.3 1999/07/09 21:55:41 mwalsh Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

#ifndef Kernel__h
#define Kernel__h

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/
#include <setjmp.h>

/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "ObjEngine.h"
#include "BinTypes.h"

/* ----- KERNEL OBJECTS -----*/

typedef struct tagMEMORYINFO
{
        OBJHEAD	ObjHead;	/* generic object header */
	struct  tagMEMORYINFO *lpNext;
	HMODULE hModule;	/* owning module */
	HTASK	hTask;		/* owning task */
	WORD	wFlags;		/* flags passed/set on handle */
	WORD	wType;		/* MT_ kind */
	DWORD	dwSize;		/* global object size */
	LPSTR   lpCore;		/* this points to native data */
	HANDLE	hMemory;	/* this is the memory handle  */
	WORD	lockcount;	/* reference count */
	LPSTR	lpData;		/* points to resource data in 86-format */
	DWORD	dwBinSize;	/* size of object in lpData	*/
	WORD	wIndex;		/* resource type */

} MEMORYINFO;
typedef MEMORYINFO *LPMEMORYINFO;

typedef MEMORYINFO HANDLEINFO;
typedef HANDLEINFO *LPHANDLEINFO;

#define HSWMC_DEST_MASK             0x0003
#define HSWMC_DEST_WINDOW_PROC      0x0000
#define HSWMC_DEST_CALLWNDPROC      0x0002
#define HSWMC_DEST_FILTER           0x0001
#define HSWMC_DEST_DEF_FRAME	    0x0003
#define HSWMC_NATMSG                0x4000
#define HSWMC_BINMSG                0x2000
#define HSWMC_SIMSTACK              0x0400
#define HSWMC_ALLOC                 0x0200

typedef struct tagWMCONVERT {
	LPARAM  lParam;
	WPARAM  wParam;
	UINT    uMsg;
	LONGPROC targ;          /* only set if not HSWMC_HOOK_WNDPROC */
	int     hook_code;      /* only set if not HSWMC_HOOK_NONE  */
	WPARAM  hook_wParam;    /* only set if HSWMC_HOOK_CALLWNDPROC */
} WMCONVERT;

/* Task queue entry */
typedef struct tagQUEUEMSG
{
    struct tagQUEUEMSG  *lpNextMsg;		/* ptr to next */
    MSG			 Message;		/* message data */
    WMCONVERT		wmc;			/* WMCONVERT message */
} QUEUEMSG;

typedef QUEUEMSG *LPQUEUEMSG;

typedef struct tagQUEUE
{
    K32OBJ	header;
    WORD	wQueueFlags;
    LPQUEUEMSG	lpQueueFree;		/* pointer to free message list */
    LPQUEUEMSG	lpQueueNext;		/* pointer to message in queue */
} QUEUE;

typedef QUEUE *LPQUEUE;

#define TASK_READY		0x0001
#define TASK_DELETE_PENDING	0x0002

/* Task information structure */
typedef struct tagTASKINFO
{
    OBJHEAD	ObjHead;		/* generic object header */
    HANDLE	hInst;			/* instance handle */
    WORD	wTDBSelector;		/* selector of TDB segment */
    LPSTR	lpCmdLine;		/* command line */
    //LPQUEUE	lpQueue;		/* application message queue */
    WORD	wStateFlags;		/* task state flags */

    UINT	uiHeapSize;		/* initial heap size */
    UINT	uiStackSize;		/* initial stack size */
    UINT	uiAutodataSize;		/* initial bss size */

    WORD	wDGROUP;		/* the dgroup selector for this task */

    ENV	       *GlobalEnvp;		/* envp_global at last switch */
    
} TASKINFO;

typedef TASKINFO *LPTASKINFO;

/* Intertask Sendmessage structure */
typedef struct tagITSMINFO ITSMINFO;

struct tagITSMINFO
{
    ITSMINFO   *lpPrev;
    HTASK	hSendingTask;
    HTASK	hReceivingTask;
    MSG		msg;
    LRESULT	lResult;
    BOOL	bSendReceived;
    BOOL	bSendCompleted;
};    

/* Global Variables */
extern BOOL bTaskingEnabled;

/* Prototypes */

HMODULE CreateModuleInstance(LPSTR);
void FreeModuleInstance(HMODULE);
void InitializeNativeTask(LPSTR);
HTASK CreateTask(void);
void InitializeTask(HTASK hTask, ENV *envp, HINSTANCE hInst,
		    UINT heap_size, UINT stack_size, UINT bss_size);
HTASK ScheduleTask(void);
void ReadyTask(HTASK);
void DeleteTask(HTASK);
BOOL CreateDataInstance(HINSTANCE, HMODULE, HTASK);
BOOL ActivateTask(HTASK);
HMODULE GetModuleFromInstance(HINSTANCE);
HTASK GetTaskFromInstance(HINSTANCE);
HINSTANCE GetInstanceFromModule(HMODULE);
BOOL InternalYield(void);
void InternalDirectedYield(HTASK);
void WINAPI Sleep(DWORD);

BOOL QueueAddMsg(DWORD, WORD,LPMSG);
BOOL QueueGetMsg(LPQUEUE, LPMSG, HWND, UINT, UINT, UINT);
LPQUEUE QueueGetPtr(DWORD,int);
void QueueSetFlags(DWORD, WORD);
void QueueClearFlags(DWORD, WORD);

#endif /* Kernel__h */

/*------------------------- < Full Revision History > ----------------------
** $Log: Kernel.h,v $
** Revision 1.3  1999/07/09 21:55:41  mwalsh
** Updated comments to standard header and footer
**
**-------------------------------------------------------------------------*/
