/*--------------------------------------------------------------------------
**      LSDevices.c                 Twin           From:  Twin/windows
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      Copyright 1997 Willows Software, Inc.
**------------------------ < File Content Description > --------------------
**
**  Module:	windows/LSDevices.c
**
**  Description:
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/windows/LSDevices.c,v 1.2 1999/07/09 21:55:41 mwalsh Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/
#include <string.h>

/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "windows.h"
#include "windowsx.h"
#include "Log.h"
#include "GdiDC.h"
#include "kerndef.h"

extern DWORD DCDisplayProc(WORD,HDC32,DWORD,LPLSDS_PARAMS);
extern DWORD DCPrinterProc(WORD,HDC32,DWORD,LPLSDS_PARAMS);
extern DWORD DCMapModeProc(WORD,HDC32,DWORD,LPLSDS_PARAMS);
extern DWORD DCBoundsRectProc(WORD,HDC32,DWORD,LPLSDS_PARAMS);
extern DWORD DCMetaProc(WORD,HDC32,DWORD,LPLSDS_PARAMS);

#define INIT_DRIVER_SLOTS	3	/* Number of slots to start with. */
#define EXPAND_DRIVER_SLOTS	3	/* Number to add when we need more. */

/* exported functions */
BOOL AddDriverEntry(LPSTR,GDIPROC,GDIPROC,GDIPROC);
LSDENTRY *GetDeviceEntry(LPCSTR lpszDriver);
void TWIN_InitLSD();

/* static functions */
static BOOL ExpandDriverTable(void);

/* Pointer to an expandable array of LSDENTRIES */
static LSDENTRY *DriverTable = NULL;
static int DriverTableSlots = 0;		/* Slots in allocated array. */
static int DriverTableUsed = 0;			/* Number of slots used. */

/* this is used for speedy referencing of DISPLAY tables in GetDC */
LSDENTRY *DisplayDeviceEntry = 0;

/* Increase the size of the driver table array. */
static BOOL
ExpandDriverTable(void)
{
	/* Initial allocation? */
	if(DriverTableSlots == 0) {
	    DriverTable = (LSDENTRY *)
		WinMalloc(INIT_DRIVER_SLOTS * sizeof(LSDENTRY));
	    DriverTableSlots = INIT_DRIVER_SLOTS;
	    return TRUE;
	}
	else {	/* Expand the existing array. */
	    LSDENTRY *NewDriverTable;
	    int NewSlots;

	    NewSlots = DriverTableSlots + EXPAND_DRIVER_SLOTS;
	    NewDriverTable = (LSDENTRY *)
		WinRealloc((char *)DriverTable,NewSlots * sizeof(LSDENTRY));

	    DriverTable = NewDriverTable;
	    DriverTableSlots = NewSlots;

	    return TRUE;
	}
}

BOOL
AddDriverEntry(LPSTR lpszName, GDIPROC lpfnOutput,
		GDIPROC lpfnMapMode, GDIPROC lpfnBoundsRect)
{
	LSDENTRY *NewDriver;

	/* Do we need to expand the driver table array? */
	if (DriverTableUsed == DriverTableSlots) {
	    if(!ExpandDriverTable())
		return FALSE;		/* Sorry! */
	}

	NewDriver = &DriverTable[DriverTableUsed++];

	strncpy(NewDriver->Name, lpszName, MAX_DEVICE_NAME - 1);
	NewDriver->Name[MAX_DEVICE_NAME - 1] = '\0';

	NewDriver->OutputProc = lpfnOutput;
	NewDriver->MapModeProc = lpfnMapMode;
	NewDriver->BoundsRectProc = lpfnBoundsRect;

	if (!(strcasecmp(lpszName,"DISPLAY")))
	    DisplayDeviceEntry = NewDriver;

	return TRUE;
}

LSDENTRY *
GetDeviceEntry(LPCSTR lpszDriver)
{
    int i;

    for(i = 0; i < DriverTableUsed; ++i)
	if (!strncasecmp(lpszDriver, DriverTable[i].Name,MAX_DEVICE_NAME-1))
	    return &DriverTable[i];
    return (LSDENTRY *)0;
}

void
TWIN_InitLSD()
{
    AddDriverEntry("DISPLAY", &DCDisplayProc, &DCMapModeProc, &DCBoundsRectProc);
    AddDriverEntry("PSCRIPT", &DCPrinterProc, &DCMapModeProc, &DCBoundsRectProc);
    AddDriverEntry("META", &DCMetaProc, &DCMetaProc, &DCBoundsRectProc);
}

/*------------------------- < Full Revision History > ----------------------
** $Log: LSDevices.c,v $
** Revision 1.2  1999/07/09 21:55:41  mwalsh
** Updated comments to standard header and footer
**
**-------------------------------------------------------------------------*/

