/*--------------------------------------------------------------------------
**      Module.c                 Twin           From:  Twin/windows
**
**              
**------------------------- < License Information > ------------------------
**
**      This file was originally a part of Willows TWIN.  Willows
**  TWIN was released under a Library GPL (LGPL).  This permits
**  redistribution of this source code, provided that the full
**  TWIN license is in effect, and provided that all modifications
**  to this source code are made publicly available.
**  Please refer to Willows software (www.willows.com) or
**  LICENSE for full information.
**  
**      Under Twine, this file is also protected by an LGPL.  Please
**  see LICENSE for full details on this license.
**  
**
**      Copyright 1997 Willows Software, Inc.
**------------------------ < File Content Description > --------------------
**
**  Module:	windows/Module.c
**
**  Description:
**      
**
**  Functions defined:
**    
**------------------------- < Revision Information > -----------------------
** $Header: /twinecvs/src/windows/Module.c,v 1.7 1999/12/19 22:24:42 robf Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/
#include <stdio.h>

/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/
#include "windows.h"
#include "Resources.h"
#include "Module.h"
#include "DPMI.h"
#include "Log.h"
#include "thread.h"

#include "peexe.h"

static LPOBJHEAD lpModuleHead = (LPOBJHEAD)NULL;
static MODULEINFO *lpModuleTable[256];

static BYTE bModIndex;

MODULEINFO *GetModuleFromIndex(int module)
{
	MODULEINFO *modinfo;
	
	if(module == 0)
		modinfo = (MODULEINFO *)lpModuleHead;
	else
		modinfo = lpModuleTable[module];

	return modinfo;
}

int
NewModuleEntry(MODULEINFO *modinfo)
{
    lpModuleTable[++bModIndex] = modinfo;
    return bModIndex;
}

void
FreeModuleEntry(MODULEINFO *modinfo)
{
    MODULEINFO *lpModuleTemp;

    if (bModIndex == modinfo->bModIndex) 
	bModIndex--;

    lpModuleTable[modinfo->bModIndex] = (LPMODULEINFO)0;

    if (modinfo == (LPMODULEINFO)lpModuleHead)
	lpModuleHead = (LPOBJHEAD)modinfo->ObjHead.lpObjNext;
    else {
	for (lpModuleTemp = (LPMODULEINFO)lpModuleHead;
	     lpModuleTemp &&
		(LPMODULEINFO)lpModuleTemp->ObjHead.lpObjNext != modinfo;
	     lpModuleTemp = (LPMODULEINFO)lpModuleTemp->ObjHead.lpObjNext);

	if (lpModuleTemp)
	    lpModuleTemp->ObjHead.lpObjNext = modinfo->ObjHead.lpObjNext;
    }
}


/* Supplemental functions */

HMODULE
CreateModuleInstance(LPSTR lpszModuleName)
{
    MODULEINFO *modinfo;
    HMODULE hModule;
    int bModIndex;

    if (!(modinfo = CREATEHMODULE(hModule))) {
	return (HMODULE)0;
    }

    modinfo->lpModuleName = WinStrdup(lpszModuleName);

    modinfo->atmModuleName = AddAtom(lpszModuleName);

    bModIndex = NewModuleEntry(modinfo);
	
    modinfo->bModIndex = bModIndex;

    modinfo->ObjHead.lpObjNext = (LPOBJHEAD)lpModuleHead;
    lpModuleHead = (LPOBJHEAD)modinfo;

    RELEASEMODULEINFO(modinfo);
    return hModule;
}

void
FreeModuleInstance(HMODULE hModule)
{
    MODULEINFO *modinfo;

    if (!(modinfo = GETMODULEINFO(hModule))) {
	return;
    }

    DeleteAtom(modinfo->atmModuleName);

    modinfo->atmModuleName = 0;
    WinFree(modinfo->lpModuleName);

    FreeModuleEntry(modinfo);

    RELEASEMODULEINFO(modinfo);
    FREEHMODULE(hModule);
}

int WINAPI
GetModuleUsage(HINSTANCE hInstance)
{
    MODULEINFO *modinfo;
    HMODULE hModule;
    int refcnt;

    
    hModule = GetModuleFromInstance(hInstance);
    if(hModule == 0)
	return 0;

    if (!(modinfo = GETMODULEINFO(hModule))) {
	return 0;
    }

    refcnt = modinfo->nRefCnt;

    RELEASEMODULEINFO(modinfo);

    if(refcnt == 0) {
	refcnt = 1;
	
    }	
    return refcnt;
}

MODULEINFO *
GetModuleFromSelector(WORD uSel)
{
    MODULEINFO *modinfo;

    if (!(modinfo = lpModuleTable[GetModuleIndex(uSel)]))
	return 0;

    return modinfo;
}

#ifdef TWIN_BIN32
char * GetModuleAlias(LPCTSTR,char *,int);
DWORD
GetModuleHandleA(LPCTSTR lpModuleName)
{
	HINSTANCE	hInst;
	HMODULE		hModule;
	DWORD		PEB;
	char 	       *modname;
	char		FileName[256];

	APISTR((LF_APICALL,"GetModuleHandleA(%s)\n",
		lpModuleName?lpModuleName:"NULL"));

	if(lpModuleName && strcmp(lpModuleName,"KERNEL32") == 0)
		return GetModuleHandleA("kernel32.dll");

	modname = GetModuleAlias(lpModuleName,FileName,256);

	hModule = GetModuleHandle(modname);
	hInst = GetInstanceFromModule(hModule);
	
	if(lpModuleName == 0) {
		MODULEINFO     *modinfo;
		modinfo = GETMODULEINFO(hModule);
		PEB = 	(DWORD) modinfo->lpNTBaseAddress;
    		RELEASEMODULEINFO(modinfo);

		APISTR((LF_APIRET,"GetModuleHandleA returns BASEADDRESS %p\n",
			PEB));
		
		return PEB;
	}

	APISTR((LF_APIRET,"GetModuleHandleA returns HMODULE %x\n",hInst));

	return hInst;
}
#endif
HMODULE
GetModuleHandle(LPCSTR lpszModule)
{
    MODULEINFO *mi;
    ATOM atom;
    LPOBJHEAD lpModuleTemp;
    LPSTR lp;
    LPSTR lpModName;
    LPSTR lpStr;

    if (!HIWORD(lpszModule)) {
	return GetModuleFromInstance((HINSTANCE)LOWORD((DWORD)lpszModule));
    }

    /* get a copy of the string, we are going to modify it */
    lpStr = WinMalloc(strlen(lpszModule)+1);
    strcpy(lpStr,lpszModule);

    /* extract the filename portion */
    if ((lpModName = strrchr(lpStr,'/'))) {
	lpModName++;
    } else
	lpModName = lpStr;

    /* drop off any extensions */
    if ((lp = strchr(lpModName,'.')))	/* strip off possible extension */
	*lp = '\0';

    if (!(atom = FindAtom(lpModName))) {
	WinFree(lpStr);
	return 0;
    }

    /* lpModName points to our internal copy */

    for (lpModuleTemp = lpModuleHead; lpModuleTemp;
	lpModuleTemp = (LPOBJHEAD)lpModuleTemp->lpObjNext) {
	mi = (MODULEINFO *)lpModuleTemp;

	if (mi->atmModuleName == atom) {
	    WinFree(lpStr);
	    return lpModuleTemp->hObj;
        }
    }

    /* can nolonger use the string */
    WinFree(lpStr);
    return 0;
}

void DEBUG_DumpModule(int module)
{
	logstr(LF_CONSOLE,"DEBUG_DumpModules %x\n",module);
}

void DEBUG_WalkModules(int bAll)
{
    	MODULEINFO *lpModuleTemp;
	int module;
    	PIMAGE_NT_HEADERS pNTHeader;

	logstr(LF_CONSOLE,"DEBUG_WalkModules\n");
	logstr(LF_CONSOLE,        "mod: windows   baseaddress instance name      filename\n");

	for (lpModuleTemp = (LPMODULEINFO)lpModuleHead,module = 0;
	     lpModuleTemp ;
	     lpModuleTemp = (LPMODULEINFO)lpModuleTemp->ObjHead.lpObjNext) {

		pNTHeader = (PIMAGE_NT_HEADERS) 0;

		if(lpModuleTemp->lpNTHeader)
		   pNTHeader = (PIMAGE_NT_HEADERS) lpModuleTemp->lpNTHeader;

		if(bAll == 0 && lpModuleTemp->lpNTBaseAddress == 0)
			continue;
		logstr(LF_CONSOLE,"%.3d: %8.8p %8.8p %4.4x   %8.8s  %s\n",
			module++,
			pNTHeader?
				(void *)pNTHeader->OptionalHeader.ImageBase:(void *)-1,
    			lpModuleTemp->lpNTBaseAddress?
    				lpModuleTemp->lpNTBaseAddress:(void *)-1,
			lpModuleTemp->hInst, 
			lpModuleTemp->lpModuleName ,
			lpModuleTemp->lpNTBaseAddress?
				lpModuleTemp->lpFileName:"");
	}
}

/*------------------------- < Full Revision History > ----------------------
** $Log: Module.c,v $
** Revision 1.7  1999/12/19 22:24:42  robf
** release moduleinfo
**
** Revision 1.6  1999/12/13 18:05:08  robf
** code cleanup
**
** Revision 1.5  1999/11/29 21:47:00  robf
** added DEBUG_WalkModules()
**
** Revision 1.3  1999/07/09 21:55:43  mwalsh
** Updated comments to standard header and footer
**
**-------------------------------------------------------------------------*/

